@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2019 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -43,12 +43,11 @@ CegoAVLIndexEntry::~CegoAVLIndexEntry()
 }
 
 void CegoAVLIndexEntry::initEntry(const CegoDataPointer& dp, char* idxPtr, int idxLen)
-{
-    
+{   
     if (_isLocal)
 	free(_pI);
 
-    CegoDataPointer zp(0,0,0);
+    CegoDataPointer zp(0,0);
 
     _len = 4 * zp.getEncodingLength() + sizeof(char) + idxLen;
 
