@@ -618,6 +618,7 @@ void CegoAdmAction::setDefault()
     _logSize = DEFLOGSIZE;
     _logNum = DEFLOGNUM;
     _sortAreaSize = DEFSORTAREASIZE;
+    _branchId = 0;
     _sysSize = DEFSYSSIZE;
     _appSize = DEFAPPSIZE;
     _tmpSize = DEFTMPSIZE;
@@ -1215,7 +1216,8 @@ void CegoAdmAction::defineTableSetAction()
 				  _appSize,
 				  _logSize,
 				  _logNum, 
-				  _sortAreaSize);
+				  _sortAreaSize,
+				  _branchId);
     handleMedResult(res);
 
     Chain msg;
@@ -3410,6 +3412,15 @@ void CegoAdmAction::storeTSRootAttr()
     _tsRoot = Chain(_stringBuf);
 }
 
+void CegoAdmAction::storeBranchAttr()
+{
+    Chain* pS = getTokenList().First();
+    if ( pS )
+    {
+	_branchId = pS->asInteger();
+    }
+}
+
 void CegoAdmAction::storePit()
 {
     _pit = Chain(_stringBuf);
