@@ -406,7 +406,8 @@ void CegoAdmNet::defineTableSet(const Chain& tableSet,
 				int appSize,
 				int logSize,
 				int logNum, 
-				unsigned long long orderSize)
+				unsigned long long orderSize,
+				int branchId)
 {
     CegoAdminHandler::ResultType res;
     res = _pAH->medDefineTableSet(tableSet,
@@ -418,7 +419,8 @@ void CegoAdmNet::defineTableSet(const Chain& tableSet,
 				  appSize,
 				  logSize,
 				  logNum, 
-				  orderSize);
+				  orderSize,
+				  branchId);
     handleMedResult(res);
 }
 
