@@ -700,6 +700,7 @@ Screen2::Panel* CegoAdmScreen::TableSet::Add::onConfirm(const ListT<Chain>& valL
     int logSize = valList[7].asInteger();
     int logNum = valList[8].asInteger();
     unsigned long long sortAreaSize = valList[9].asUnsignedLongLong();
+    int branchId = 0; // still not supported for screen mode
     
     CegoAdminHandler::ResultType res;
     res = _pAH->medDefineTableSet(tableSet,
@@ -711,7 +712,8 @@ Screen2::Panel* CegoAdmScreen::TableSet::Add::onConfirm(const ListT<Chain>& valL
 				  appSize,
 				  logSize,
 				  logNum, 
-				  sortAreaSize);
+				  sortAreaSize,
+				  branchId);
     
     Chain msg;
     _pAH->getMsg(msg);
@@ -1038,7 +1040,8 @@ Screen2::Panel* CegoAdmScreen::TableSet::Action::Edit::onConfirm(const ListT<Cha
     int logSize = valList[7].asInteger();
     int logNum = valList[8].asInteger();
     unsigned long long sortAreaSize = valList[9].asUnsignedLongLong();
-
+    int branchId = 0;
+    
     // to edit tableset in state DEFINED, we just remove the whole tableset and define it new
     
     CegoAdminHandler::ResultType res;
@@ -1060,7 +1063,8 @@ Screen2::Panel* CegoAdmScreen::TableSet::Action::Edit::onConfirm(const ListT<Cha
 				  appSize,
 				  logSize,
 				  logNum, 
-				  sortAreaSize);
+				  sortAreaSize,
+				  branchId);
     
     _pAH->getMsg(msg);
     
