@@ -61,6 +61,7 @@
 #define HISTFILE ".cgadm_history"
 #define DEFAULTPROMPT "CGADM > "
 #define PROMPT2 "> "
+#define PIPE_TIMEOUT 30 /* checking input pipe data timeout in ms */
 enum RunMode { INTERACTIVE, BATCH, COMMAND, SC };
 
 bool dispatchCmd(CegoAdmAction* pAdm, const Chain& cmd);
@@ -117,7 +118,8 @@ int main(int argc, char **argv)
     Chain homeDir(getenv("HOME"));
     Chain histPath = homeDir + "/" + Chain(HISTFILE);
 #endif
-    
+
+    // default run mode is interactive
     RunMode runMode = INTERACTIVE;
 
     bool debug=false;
@@ -156,24 +158,6 @@ int main(int argc, char **argv)
     Chain prompt = longOpt.getOptValue("prompt");
     int portNo = longOpt.getOptValue("port").asInteger();
 
-    Chain batchFileName;
-    Chain command;
-
-    if ( longOpt.isSet("batchfile") )
-    {
-	runMode = BATCH;
-	batchFileName = longOpt.getOptValue("batchfile");
-    }
-    else if ( longOpt.isSet("cmd") )
-    {
-	runMode = COMMAND;
-	command = longOpt.getOptValue("cmd");
-    }
-
-    else if ( longOpt.isSet("screen") )
-    {
-	runMode = SC;
-    }
 
     CegoModule *pModule = 0;
     CegoAdminHandler* pAH = 0;
@@ -226,6 +210,36 @@ int main(int argc, char **argv)
 	exit(1);
     }
 
+    Chain command;
+    
+    File *pInput = 0;
+
+    if ( longOpt.isSet("batchfile") )
+    {
+	runMode = BATCH;
+	Chain batchFileName = longOpt.getOptValue("batchfile");
+	pInput = new File(batchFileName);
+	pInput->open(File::READ);
+    }
+    else if ( longOpt.isSet("cmd") )
+    {
+	runMode = COMMAND;
+	command = longOpt.getOptValue("cmd");
+    }
+    else if ( longOpt.isSet("screen") )
+    {
+	runMode = SC;
+    }
+    else
+    {
+	// if data available from stdin, we also setup batch mode
+	pInput = new File(File::IN);
+	if ( pInput->hasData(PIPE_TIMEOUT) )
+	{
+	    runMode = BATCH;	
+	}
+    }
+    
     CegoAdmAction* pAdm = new CegoAdmAction(pAH, pModule, resultMode, rawMode);
 
     int exitCode = 0;
@@ -240,19 +254,21 @@ int main(int argc, char **argv)
 	    screen.showScreen();
 	    break;	    	    
 	}
+	case COMMAND:
+        {                           
+            Chain cmd = command.cutTrailing(" ;") + Chain(";");
+            dispatchCmd(pAdm, cmd);         
+            break;
+        }
 	case BATCH:
 	{	    
 	    int lineNo=0;
-	    	    
-	    File batchFile(batchFileName);
-	    
-	    batchFile.open(File::READ);	    
-	    
+	    	    	    
 	    Chain cmd;
 	    Chain line;
 	    bool doQuit = false;
 	    
-	    while (doQuit == false && batchFile.readLine(line))
+	    while (doQuit == false && pInput->readLine(line))
 	    {
 		lineNo++;
 		line = line.cutTrailing(" \t");
@@ -293,7 +309,11 @@ int main(int argc, char **argv)
 				if ( ignoreError == true )
 				    cerr << "Ignored" << endl;
 				else
-				    throw Exception(EXLOC, msg);		
+				{
+				    pInput->close();
+				    delete pInput;				    
+				    throw Exception(EXLOC, msg);
+				}
 			    }
 			    cmd = Chain();			    
 			}
@@ -305,16 +325,11 @@ int main(int argc, char **argv)
 		cout << "Incomplete command <" << cmd << ">"  << endl;
 	    }
 	    
-	    batchFile.close();
+	    pInput->close();
+	    delete pInput;
 	    
 	    break;
 	}
-	case COMMAND:
-	{		    	    
-	    Chain cmd = command.cutTrailing(" ;") + Chain(";");
-	    dispatchCmd(pAdm, cmd);	    
-	    break;
-	}
 	case INTERACTIVE:
 	{	    	    	
 	    read_history((char*)histPath);
