@@ -8,7 +8,7 @@
 //                                                        
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -25,6 +25,7 @@
 #include <lfcbase/Thread.h>
 #include <lfcbase/NetHandler.h>
 #include <lfcbase/NanoTimer.h>
+#include <lfcbase/OutStream.h>
 
 // CEGO INCLUDES
 #include "CegoAdminHandler.h"
@@ -33,35 +34,89 @@
 #include "CegoDbThreadPool.h"
 #include "CegoLogThreadPool.h"
 
-
 class CegoAdminThreadPool;
 
 class CegoAdminThread : public Thread {
     
 public:
-
+    
     CegoAdminThread();
     CegoAdminThread(CegoAdminThreadPool *pPool, CegoDatabaseManager *pDBMng, CegoDbThreadPool *pDbPool, CegoLogThreadPool *pLogPool);
     ~CegoAdminThread();
-
+    
     const Chain& lastAction() const;
     void* job(void* arg);
+    
+private:
+    
+    class BackupStream : public OutStream {
+    public:
+	
+        BackupStream(CegoAdminHandler* pAH)
+	{
+	    _pAH = pAH;
+	}
+	
+	~BackupStream()
+	{
+	}
+	
+	bool nextLine(const Chain& line)
+	{
+	    return _pAH->syncWithInfo(Chain("local"), Chain("local"), line, 0);
+	}
+	
+    private:
+	
+	CegoAdminHandler* _pAH;
+    };
+
+    class LogInfoStream : public OutStream {	
+    public:
+	
+        LogInfoStream() : OutStream()
+	{
+	};
+    
+	bool nextLine(const Chain& line)
+	{	    
+	    _result = line;
+	    return true;
+	};
+
+	Chain getResult()
+	{
+	    return _result;
+	}
+	
+    private:
+
+	Chain _result;
+    };
 
- private:
 
     void copyTableSet(int copyId, const Chain& tableSet, const Chain& secondary, const Chain& mediator, const Chain& user, const Chain& passwd, bool copyOnline);
-
+    
     void serveSession(CegoAdminHandler *pAH);
     bool serveRequest(CegoAdminHandler *pAH, CegoAdminHandler::RequestType reqType);
-
+    
     void srvRetrieveTableSet(CegoAdminHandler *pAH);
     void srvLockInfo(CegoAdminHandler *pAH);
     void srvLockStat(CegoAdminHandler *pAH);
+    void srvParameterInfo(CegoAdminHandler *pAH);
     void srvPoolInfo(CegoAdminHandler *pAH);
     void srvPoolEntryList(CegoAdminHandler *pAH);
     void srvThreadInfo(CegoAdminHandler *pAH);
 
     void srvDbThreadInfo(CegoAdminHandler *pAH);
+    void srvDbThreadHistory(CegoAdminHandler *pAH);
+
+    void srvQueryHistoryLast(CegoAdminHandler *pAH);
+    void srvQueryHistoryCost(CegoAdminHandler *pAH);
+
+    void srvNumQueryLast(CegoAdminHandler *pAH);
+    void srvNumQueryCost(CegoAdminHandler *pAH);
+    
     void srvAdmThreadInfo(CegoAdminHandler *pAH);
     void srvLogThreadInfo(CegoAdminHandler *pAH);
     void srvDbThreadAbort(CegoAdminHandler *pAH);
@@ -71,14 +126,12 @@ public:
     void srvShowQueryCache(CegoAdminHandler *pAH);
     void srvListQueryCache(CegoAdminHandler *pAH);
     void srvCleanQueryCache(CegoAdminHandler *pAH);
-    void srvSetQueryCacheEntry(CegoAdminHandler *pAH);
-    void srvSetQueryCacheSize(CegoAdminHandler *pAH);
+    void srvSetQueryCacheParam(CegoAdminHandler *pAH);
 
     void srvShowTableCache(CegoAdminHandler *pAH);
     void srvListTableCache(CegoAdminHandler *pAH);
     void srvCleanTableCache(CegoAdminHandler *pAH);
-    void srvSetTableCacheEntry(CegoAdminHandler *pAH);
-    void srvSetTableCacheSize(CegoAdminHandler *pAH);
+    void srvSetTableCacheParam(CegoAdminHandler *pAH);
 
     void srvDefineTableSet(CegoAdminHandler *pAH);
     void srvRemoveTableSet(CegoAdminHandler *pAH);
@@ -92,6 +145,8 @@ public:
     void srvDumpObject(CegoAdminHandler *pAH);
     void srvSetLogMng(CegoAdminHandler *pAH);
     void srvShowLogMng(CegoAdminHandler *pAH);
+    void srvSetBackupMng(CegoAdminHandler *pAH);
+    void srvShowBackupMng(CegoAdminHandler *pAH);
     void srvSetCheckpoint(CegoAdminHandler *pAH);
     void srvSetTSInitFile(CegoAdminHandler *pAH);
     void srvSetTSSysSize(CegoAdminHandler *pAH);
@@ -101,6 +156,7 @@ public:
     void srvSetTSLogNum(CegoAdminHandler *pAH);
     void srvSetTSLogSize(CegoAdminHandler *pAH);
     void srvSetTSSortAreaSize(CegoAdminHandler *pAH);
+    void srvSetTSLogUser(CegoAdminHandler *pAH);
 
     void srvStartRecovery(CegoAdminHandler *pAH);	      	
     void srvStopRecovery(CegoAdminHandler *pAH);
@@ -109,6 +165,7 @@ public:
     void srvGetTSInfo(CegoAdminHandler *pAH);
 
     void srvGetDetailedTSInfo(CegoAdminHandler *pAH);
+    void srvGetArchLogInfo(CegoAdminHandler *pAH);
     void srvGetTransactionInfo(CegoAdminHandler *pAH);
     void srvGetBUStatInfo(CegoAdminHandler *pAH);
 
@@ -147,7 +204,11 @@ public:
     void srvRemoveArchLog(CegoAdminHandler *pAH);
 
     void srvBeginBackup(CegoAdminHandler *pAH);
+    void srvExecuteBackup(CegoAdminHandler *pAH);
     void srvEndBackup(CegoAdminHandler *pAH);
+    void srvListBackup(CegoAdminHandler *pAH);
+    
+    void srvRestore(CegoAdminHandler *pAH);
     void srvRecover(CegoAdminHandler *pAH);
     void srvSecSwitch(CegoAdminHandler *pAH);
 
@@ -180,6 +241,7 @@ public:
 
     void medGetTableSetList(CegoAdminHandler *pAH);
     void medGetDetailedTableSetInfo(CegoAdminHandler *pAH);
+    void medGetArchLogInfo(CegoAdminHandler *pAH);
 
     void medGetObjectInfo(CegoAdminHandler *pAH);
     void medGetTransactionInfo(CegoAdminHandler *pAH);
@@ -187,7 +249,10 @@ public:
     void medResetBUStat(CegoAdminHandler *pAH);
 
     void medBeginBackup(CegoAdminHandler *pAH);
+    void medExecuteBackup(CegoAdminHandler *pAH);
     void medEndBackup(CegoAdminHandler *pAH);
+    void medListBackup(CegoAdminHandler *pAH);
+    void medRestore(CegoAdminHandler *pAH);
     void medRecover(CegoAdminHandler *pAH);
     void medSecSwitch(CegoAdminHandler *pAH);
 
@@ -198,6 +263,9 @@ public:
     void medEnableArchLog(CegoAdminHandler *pAH);
     void medDisableArchLog(CegoAdminHandler *pAH);
 
+    void medEnableAppendMode(CegoAdminHandler *pAH);
+    void medDisableAppendMode(CegoAdminHandler *pAH);
+
     void medEnableAutoCorrect(CegoAdminHandler *pAH);
     void medDisableAutoCorrect(CegoAdminHandler *pAH);
 
@@ -205,19 +273,27 @@ public:
     void secMedSwitch(CegoAdminHandler *pAH);
     void secMedRelocate(CegoAdminHandler *pAH);
 
-
     CegoAdminHandler* getSession(const Chain& role, const Chain& hostname, const Chain& user, const Chain& pwd);
     void closeSession(CegoAdminHandler* pAH);
 
     void addTableSetUsage(Element *pTSE);
 
     Element* getDetailedTSInfo(const Chain& tableSet);
+    Element* getArchLogInfo(const Chain& tableSet);
     Element* getObjectInfo(const Chain& tableSet);
     Element* getTransactionInfo(const Chain& tableSet);
     Element* getBUStatInfo(const Chain& tableSet);
+
+    Element* getBackupInfo(const Chain& tableSet, const Chain& pit);
     
     Element* checkTS(const Chain& hostType, const Chain& hostName, Element *pInfo, CegoAdminHandler *pPAH, CegoAdminHandler *pSAH);
 
+    Chain executeBackup(CegoAdminHandler* pAH, const Chain& tableSet, const Chain& msg);
+    ListT<Chain> executeListBackup(const Chain& tableSet, const Chain& pit);
+    Chain executeRestore(CegoAdminHandler* pAH, const Chain& tableSet, const Chain& backupId);
+
+    Chain executeLogInfo(const Chain& tableSet, const Chain& archPath);
+	
     unsigned long long _idx;
 
     CegoDatabaseManager* _pDBMng;
