@@ -6,7 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke
 //               
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -71,8 +71,7 @@ CegoAdminThreadPool::CegoAdminThreadPool(int poolLimit, CegoDatabaseManager *pDB
 
     int i=0;
     while ( i < _poolLimit )
-    {
-	
+    {	
 	_threadState[i] = READY;
 	_threadList[i]= new CegoAdminThread(this, pDBMng, pDbPool, pLogPool);
 	_threadId[i]=i;
@@ -165,25 +164,31 @@ void CegoAdminThreadPool::setTid(int i, unsigned long long tid)
 
 void* CegoAdminThreadPool::job(void* arg)
 {
-    try {
-
+    try
+    {
 	NanoTimer tim;
     
-	Net net( NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN );
+	Net net( NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN, NETMNG_MAXSENDLEN );
 	
 #ifdef CGDEBUG
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Serving admin requests on port ") + Chain(_adminPortNo));
 #endif
 
 	// cout << "Serving portno " << _adminPortNo << endl;
-	net.serve(_adminHostName, _adminPortNo);
-	
+	if ( _adminHostName != Chain() )
+	    net.serve(_adminHostName, _adminPortNo);
+	else
+	    net.serve6(_adminPortNo);
+
 	unsigned long long usedTime[THRMNG_NUMLOADSAMPLE];
 	for ( int i = 0; i < THRMNG_NUMLOADSAMPLE; i++ )
 	{
 	    usedTime[i] = 0;
 	}
 
+	int selectTimeout = _pDBMng->getSelectTimeout();
+	int queueDelay = _pDBMng->getQueueDelay();
+
 	while ( ! _terminated ) 
 	{	    
 	    // take the mutex to avoid cpu load during wait position
@@ -201,12 +206,11 @@ void* CegoAdminThreadPool::job(void* arg)
 	    if ( noReq )
 		lockQueue();
 	    
-	    NetHandler* pHandle;
+	    NetHandler* pHandle = 0;
 	    
-	    try {
-
-		pHandle = net.nextRequest( NETMNG_SELECT_TIMEOUT );
-
+	    try
+	    {
+		pHandle = net.nextRequest( selectTimeout );
 	    }
 	    catch ( Exception e)
 	    {
@@ -221,21 +225,28 @@ void* CegoAdminThreadPool::job(void* arg)
 		
 		// in the following, we go sleep for a while
 		// to give other threads a chance to aquire queue lock
-		// on solaris without this sleep, bad scheduling behaviour happened
 
 		Sleeper s;
-		s.nanoSleep(NETMNG_QUEUE_DELAY);
+		s.microSleep(queueDelay);
 	    }
 
 	    if ( pHandle )
 	    {
-
-		// pHandle->setSendTimeout(NETMNG_SEND_TIMEOUT);
-		// pHandle->setRecvTimeout(NETMNG_RECV_TIMEOUT);
+		_pDBMng->log(_modId, Logger::NOTICE, Chain("Connection request from <") + pHandle->getSource() + Chain(">"));
 
 		lockQueue();
-		_requestQueue.Insert(pHandle);
-		unlockQueue();
+		
+		if ( _requestQueue.Size() < NETMNG_MAXQUEUELEN )
+		{
+		    _requestQueue.Insert(pHandle);
+		}
+		else
+		{
+		    delete pHandle;		    
+		    _pDBMng->log(_modId, Logger::NOTICE, Chain("Rejected incoming request since connection queue is full ( ") +  Chain(NETMNG_MAXQUEUELEN) + Chain(" max )"));
+		}
+		
+		unlockQueue();		
 	    }
 
 	    tim.stop();
@@ -281,12 +292,10 @@ void* CegoAdminThreadPool::job(void* arg)
 	int i=0;
 	while ( i < _poolLimit )
 	{
-
 #ifdef CGDEBUG
 	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Waiting for admin thread with tid ") 
 			 + Chain(_threadList[i]->getTid()) + Chain(" to terminate"));
 #endif
-
 	    join(_threadList[i]->getTid());
 	    i++;
 	}
@@ -315,7 +324,6 @@ NetHandler* CegoAdminThreadPool::nextRequest()
 
     if ( pRequest )
     {
-
 	NetHandler *pN = *pRequest;
 	_requestQueue.Remove(*pRequest);
 	unlockQueue();
