@@ -134,16 +134,21 @@ CegoAttrComp::CegoAttrComp(const Chain& tableName, const Chain& attrName, const
     _posSetup=false;
 }
 
-CegoAttrComp::CegoAttrComp(const Chain& tableName, const Chain& attrName, const Chain& pattern, bool isNot)
+CegoAttrComp::CegoAttrComp(const Chain& tableName, const Chain& attrName, const Chain& pattern, bool isNot, bool isCaseSensitive)
 {
     _tableName = tableName;
     _attrName = attrName;
     _pattern = pattern;
     
-    if ( isNot )
-	_compMode = ISNOTLIKE;
-    else
+    if ( ! isNot && isCaseSensitive)
 	_compMode = ISLIKE;
+    else if ( isNot && isCaseSensitive)
+	_compMode = ISNOTLIKE;
+    else if ( ! isNot && ! isCaseSensitive)
+	_compMode = ISNCLIKE;
+    else if ( isNot && ! isCaseSensitive)
+	_compMode = ISNOTNCLIKE;
+    
     _pMatcher = 0;
     _pos=0;
 }
@@ -262,8 +267,15 @@ const Chain& CegoAttrComp::getPattern() const
 Matcher* CegoAttrComp::getMatcher()
 {
     if ( _pMatcher == 0)
-    {	
-	Chain regexPattern = CegoQueryHelper::sql2Regex(_pattern);
+    {
+	Chain regexPattern;
+	
+	if ( _compMode == ISLIKE
+	     || _compMode == ISNOTLIKE )	
+	    regexPattern = CegoQueryHelper::sql2Regex(_pattern);
+	else
+	    regexPattern = CegoQueryHelper::sql2Regex(_pattern.toLower());
+	
 	_pMatcher = new Matcher(regexPattern);
 	_pMatcher->prepare();	
     }
@@ -476,7 +488,7 @@ CegoAttrComp& CegoAttrComp::operator = ( const CegoAttrComp& ac)
 }
     
 bool CegoAttrComp::operator == ( const CegoAttrComp& ac) const
-{   
+{       
     if ( _compMode != ac._compMode )
 	return false;    
     if ( _compMode == ATTR )
@@ -485,8 +497,10 @@ bool CegoAttrComp::operator == ( const CegoAttrComp& ac) const
 		 && _comp == ac._comp
 		 && _attrDesc == ac._attrDesc);
     if ( _compMode == VAL )
+    {
 	// we don't compare the values ( _fv == ac._fv ), since CegoAttrCond::update sets it up 
-	return ( _tableName == ac._tableName && _attrName == ac._attrName && _comp == ac._comp ); 
+	return ( _tableName == ac._tableName && _attrName == ac._attrName && _comp == ac._comp );
+    }
     if ( _compMode == BTWN )
     {
 	// we don't compare the values ( _fv == ac._fv && _fv2 == ac._fv2 ), since CegoAttrCond::update sets it up
@@ -503,7 +517,10 @@ bool CegoAttrComp::operator == ( const CegoAttrComp& ac) const
 	    return ( ( _tableName == ac._tableName && _attrName == ac._attrName )  && ( _attrDesc == ac._attrDesc ) && ( _attrDesc2 == ac._attrDesc2 ) );
 	}
     }
-    if ( _compMode == ISLIKE || _compMode == ISNOTLIKE )
+    if ( _compMode == ISLIKE
+	 || _compMode == ISNOTLIKE
+	 || _compMode == ISNCLIKE
+	 || _compMode == ISNOTNCLIKE ) 
 	return ( _tableName == ac._tableName && _attrName == ac._attrName && _pattern == ac._pattern  );
 
     return false;
@@ -610,7 +627,10 @@ bool CegoAttrComp::operator < ( const CegoAttrComp& ac) const
 	    return ( _fv2 < ac._fv2 ); 	    
 	}
     }
-    else if ( _compMode == ISLIKE || _compMode == ISNOTLIKE )
+    else if ( _compMode == ISLIKE
+	      || _compMode == ISNOTLIKE
+	      || _compMode == ISNCLIKE
+	      || _compMode == ISNOTNCLIKE ) 
     {	
 	return ( _pattern < ac._pattern  );
     }
@@ -725,7 +745,10 @@ bool CegoAttrComp::operator > ( const CegoAttrComp& ac) const
 	    return ( _fv2 > ac._fv2 ); 	    
 	}
     }
-    else if ( _compMode == ISLIKE || _compMode == ISNOTLIKE )
+    else if ( _compMode == ISLIKE
+	      || _compMode == ISNOTLIKE
+	      || _compMode == ISNCLIKE 
+	      || _compMode == ISNOTNCLIKE )
     {	
 	return ( _pattern > ac._pattern  );
     }
@@ -771,6 +794,15 @@ Chain CegoAttrComp::getId() const
     {
 	s += Chain("nlk'") + _pattern + Chain("'"); 
     }
+    else if ( _compMode == ISNCLIKE )
+    {
+	s += Chain("nclk'") + _pattern + Chain("'"); 
+    }
+    else if ( _compMode == ISNOTNCLIKE )
+    {
+	s += Chain("nnclk'") + _pattern + Chain("'"); 
+    }
+    
     else
     {
 	switch ( _comp )
@@ -840,6 +872,15 @@ Chain CegoAttrComp::toChain() const
     {
 	s += Chain(" not like '") + _pattern + Chain("'"); 
     }
+    else if ( _compMode == ISNCLIKE )
+    {
+	s += Chain(" nclike '") + _pattern + Chain("'"); 
+    }
+    else if ( _compMode == ISNOTNCLIKE )
+    {
+	s += Chain(" not nclike '") + _pattern + Chain("'"); 
+    }
+
     else
     {
 	switch ( _comp )
