@@ -339,7 +339,7 @@ CegoPredicate* CegoAttrCond::getPred4Comp(const ListT<CegoExpr*> exprList, CegoA
 	
 	// Depending on the attribute comparison mode, an appropriate predicate is
 	// build up
-	CegoPredicate *pP;	    
+	CegoPredicate *pP = 0;	    
 	
 	if ( pAC->getCompMode() == CegoAttrComp::ATTR && pAC->isSetup() == false )   
 	{
