@@ -8,7 +8,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -26,6 +26,7 @@
 #include <lfcbase/AVLTreeT.h>
 
 // CEGO INCLUDES
+#include "CegoDefs.h"
 #include "CegoDataType.h"
 #include "CegoField.h"
 #include "CegoDataPointer.h"
@@ -49,9 +50,10 @@ public:
     void insertBTreeWithCommit(const CegoDataPointer& dp,
 		     const CegoBTreeValue &iv,
 		     unsigned long long tid);
-
+    
     void deleteBTree(const CegoBTreeValue &iv,
-		     const CegoDataPointer& ddp);
+		     const CegoDataPointer& dp,
+		     unsigned long long tid);
 
     void insertBTree(const CegoDataPointer& dp, 
 		      const CegoBTreeValue &iv,
@@ -59,7 +61,8 @@ public:
 
     void deleteBTree(const CegoDataPointer& sysEntry, 
 		     const CegoBTreeValue &iv,
-		     const CegoDataPointer& dp);
+		     const CegoDataPointer& dp,
+		     unsigned long long tid);
 
     bool verifyBTree();
 
@@ -68,6 +71,8 @@ public:
     int getNumPages();
     int freeBTree();
 
+    int traceBTree();
+	
 private:
 
     class BTreeCache {
@@ -77,12 +82,11 @@ private:
 	BTreeCache();
 	~BTreeCache();
 
-	CegoBufferPage newCachePage(const CegoBufferPage& bp, bool copyPage);
-	void updateCachePage(CegoBufferPage& bp);
-	bool getCachePage(int fileId, int pageId, CegoBufferPage& bp);
+	CegoBufferPage* newCachePage(CegoBufferPage& bp, const CegoBufferPage::PageType pageType, bool copyPage);
+	CegoBufferPage* getCachePage(PageIdType pageId);
 
-	bool getFirst(CegoBufferPage& bp);
-	bool getNext(CegoBufferPage& bp);
+	CegoBufferPage* getFirst();
+	CegoBufferPage* getNext();
 	
     private:
 
@@ -94,96 +98,92 @@ private:
 	    {
 	    }
 
-	    CacheEntry(CegoBufferPage bp)
+	    CacheEntry(PageIdType pageId)
 	    {
-		_bp = bp;
+		_pageId = pageId;
+	    }
+	    
+	    CacheEntry(CegoBufferPage* pBP)
+	    {
+		_pBP = pBP;
+		_pageId = pBP->getPageId();
 	    }
 	    	    
 	    ~CacheEntry()
 	    {
 	    }
 
-	    CegoBufferPage getPage() const
+	    CegoBufferPage* getPage() const
 	    {
-		return _bp;
+		return _pBP;
 	    }
 	    
-	    void setPage(CegoBufferPage& bp)
+	    void setPage(CegoBufferPage* pBP)
 	    {
-		_bp = bp;
+		_pBP = pBP;
+		_pageId = pBP->getPageId();
 	    }
 	    
 	    CacheEntry& operator = ( const CacheEntry& e)
 	    {
-		_bp = e._bp;
+		_pBP = e._pBP;
+		_pageId = e._pageId;
 		return (*this);
 	    }
 
 	    bool operator == ( const CacheEntry& e)
 	    {
-		if ( _bp.getFileId() == e.getPage().getFileId() && _bp.getPageId() == e.getPage().getPageId() )
+		if ( _pageId == e._pageId )
 		    return true;
 		return false;
 	    }
 
 	    bool operator < ( const CacheEntry& e)
 	    {
-		if ( _bp.getFileId() < e.getPage().getFileId() )
-		    return true;
-		if ( _bp.getFileId() > e.getPage().getFileId() )
-		    return false;
-		
-		if ( _bp.getPageId() < e.getPage().getPageId() )
-		    return true;
-		if ( _bp.getPageId() > e.getPage().getPageId() )
-		    return false;
-		
+		if ( _pageId < e._pageId )
+		    return true;		
 		return false;
 	    }
 
 	    bool operator > ( const CacheEntry& e)
 	    {
-		if ( _bp.getFileId() > e.getPage().getFileId() )
+		if ( _pageId > e._pageId )
 		    return true;
-		if ( _bp.getFileId() < e.getPage().getFileId() )
-		    return false;
-		
-		if ( _bp.getPageId() > e.getPage().getPageId() )
-		    return true;
-		if ( _bp.getPageId() < e.getPage().getPageId() )
-		    return false;
-		
 		return false;
 	    }
 	    
 	    friend ostream& operator << (ostream& s, const CacheEntry& e)
 	    {
-		s << "(" << e.getPage().getFileId() << "," << e.getPage().getPageId() << ")";
+		s << "(" << e._pageId << ")";
 		return s;
 	    }
 	    
 	private:
 
-	    CegoBufferPage _bp;
-	    
+	    PageIdType _pageId;
+	    CegoBufferPage* _pBP;	    
 	};
 
 	AVLTreeT<CacheEntry> _cache;
     };
 
+    bool checkDuplicate(const CegoBTreeValue &iv, const CegoBTreeNode &leaf, unsigned long long tid);
+    			
+    CegoBufferPage* allocPage(CegoBufferPage::PageType type);
+    CegoBufferPage* getPage(PageIdType pageId);
+    void putPage(CegoBufferPage* pBP);
+    
+    bool verifyNode(PageIdType pageId);
+    void dumpNode(int level, PageIdType pageId);
+    
+    int countNodePages(PageIdType pageId, PageIdType& firstLeafPageId, bool& isFirst);
+    int countLeafPages(PageIdType firstLeafPageId);
+    
+    int freeNodePages(PageIdType pageId, PageIdType& firstLeafPageId, bool& isFirst);
+    int freeLeafPages(PageIdType firstLeafPageId);
 
-    void allocPage(CegoBufferPage::PageType type, CegoBufferPage& bp);
-    void getPage(CegoBufferPage& bp, int fileId, int pageId);
-    void putPage(CegoBufferPage& bp);
-    
-    bool verifyNode(int fileId, int pageId);
-    void dumpNode(int level, int fileId, int pageId);
-    
-    int countNodePages(int fileId, int pageId, int& firstLeafFileId, int& firstLeafPageId, bool& isFirst);
-    int countLeafPages(int firstLeafFileId, int firstLeafPageId);
-    
-    int freeNodePages(int fileId, int pageId, int& firstLeafFileId, int& firstLeafPageId, bool& isFirst);
-    int freeLeafPages(int firstLeafFileId, int firstLeafPageId);
+    int traceNodePages(PageIdType pageId, PageIdType& firstLeafPageId, bool& isFirst);
+    int traceLeafPages(PageIdType firstLeafPageId);
 
     CegoBTreeObject* _pBTO;
     CegoObjectManager* _pObjMng;
@@ -195,9 +195,6 @@ private:
     int _keyLen;
     int _tabSetId;
 
-    // CegoBufferPage _rootBP;
-    // bool _rootFixed;
-
     BTreeCache* _pCache;
     
     unsigned long _modId;
