@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -34,31 +34,26 @@
 #include <stdlib.h>
 
 // must match the value of method CegoDataPointer.getEncodingLength 
-#define DPENCODINGLENGTH 3*sizeof(int)
-
-char __allowDuplicateNull = 0;
+#define DPENCODINGLENGTH CegoDataPointer::getEncodingLength()
 
 CegoBTreeNode::CegoBTreeNode()
 {
     _pI = 0;
     _len = 0;
     _keyLen = 0;
-    _fileId = 0;
+    _entrySize = 0;
+    
+    _maxEntries = 0;
     _pageId = 0;
-    _nextFileId = 0;
     _nextPageId = 0;
     _pNextChild = 0;
+    _nt = CegoBTreeNode::UNDEFINED;
 }
 
 CegoBTreeNode::~CegoBTreeNode()
 {
 }
 
-void CegoBTreeNode::setType(NodeType nt)
-{
-    _nt = nt;
-}
-
 const CegoBTreeNode::NodeType CegoBTreeNode::getType() const
 {
     return _nt;
@@ -80,42 +75,22 @@ int CegoBTreeNode::getLen() const
     return _len;
 }
 
-void CegoBTreeNode::setFileId(int fileId)
-{
-    _fileId = fileId;
-}
-
-int CegoBTreeNode::getFileId() const
-{
-    return _fileId;
-}
-
-void CegoBTreeNode::setPageId(int pageId)
+void CegoBTreeNode::setPageId(PageIdType pageId)
 {
     _pageId = pageId;
 }
 
-int CegoBTreeNode::getPageId() const
+PageIdType CegoBTreeNode::getPageId() const
 {
     return _pageId;
 }
 
-void CegoBTreeNode::setNextFileId(int fileId)
-{
-    _nextFileId = fileId;
-}
-
-int CegoBTreeNode::getNextFileId() const
-{
-    return _nextFileId;
-}
-
-void CegoBTreeNode::setNextPageId(int pageId)
+void CegoBTreeNode::setNextPageId(PageIdType pageId)
 {
     _nextPageId = pageId;
 }
 
-int CegoBTreeNode::getNextPageId() const
+PageIdType CegoBTreeNode::getNextPageId() const
 {
     return _nextPageId;
 }
@@ -124,8 +99,7 @@ void CegoBTreeNode::initNode()
 {
     if ( _pI )
     {
-	int numEntry = 0;
-	memcpy(_pI, &numEntry, sizeof(int));
+	(*(int*)(_pI)) = 0;
 	return;
     }
     throw Exception(EXLOC, Chain("Btree node not set up"));
@@ -135,9 +109,7 @@ int CegoBTreeNode::numEntries() const
 {
     if ( _pI )
     {
-	int n;
-	memcpy(&n, _pI, sizeof(int));
-	return n;
+	return (*(int*)(_pI));
     }
     throw Exception(EXLOC, Chain("Btree node not set up"));
 }
@@ -146,10 +118,7 @@ void CegoBTreeNode::incEntries()
 {
     if ( _pI )
     {
-	int n;
-	memcpy(&n, _pI, sizeof(int));
-	n++;
-	memcpy(_pI, &n, sizeof(int));
+	(*(int*)(_pI))++;
 	return;
     }
     throw Exception(EXLOC, Chain("Btree node not set up"));
@@ -159,49 +128,20 @@ void CegoBTreeNode::decEntries()
 {
     if ( _pI )
     {
-	int n;
-	memcpy(&n, _pI, sizeof(int));
-	n--;
-	memcpy(_pI, &n, sizeof(int));
+	(*(int*)(_pI))--;
 	return;
     }
     throw Exception(EXLOC, Chain("Btree node not set up"));
 }
 
-int CegoBTreeNode::maxEntries() const
-{
-    if ( _nt == LEAF )
-    {
-	return ( _len - sizeof(int) )  / ( _keyLen + DPENCODINGLENGTH );
-    }
-    else if ( _nt == NODE )
-    {	
-	return ( _len - sizeof(int) - 2 * sizeof(int) ) / ( _keyLen + 2 * sizeof(int) );
-    }
-    else
-    {
-	throw Exception(EXLOC, Chain("Unknown btree node type"));
-    }
-}
-
 int CegoBTreeNode::getEntrySize() const
 {
-    if ( _nt == LEAF )
-    {
-	return _keyLen + DPENCODINGLENGTH;
-    }
-    else if ( _nt == NODE )
-    {	
-	return _keyLen + 2 * sizeof(int);
-    }
-    else
-    {
-	throw Exception(EXLOC, Chain("Unknown btree node type"));
-    }
+    return _entrySize;
 }
 
-void CegoBTreeNode::getChildPage(const CegoBTreeValue& iv, int& fileId, int& pageId)
+void CegoBTreeNode::getChildPage(const CegoBTreeValue& iv, PageIdType& pageId, bool traceMin)
 {
+
     if ( _nt == LEAF )
 	throw Exception(EXLOC, Chain("Invalid method for node type"));
 
@@ -210,91 +150,104 @@ void CegoBTreeNode::getChildPage(const CegoBTreeValue& iv, int& fileId, int& pag
     if ( ne == 0 )
 	throw Exception(EXLOC, "No child entries avaiable");
 
+    int lb=0;
+    int rb=ne;
+
+    int i;
     bool posFound=false;
-    int i=0;
-    while ( i < ne && posFound == false)
+    
+    while ( posFound == false)
     {
+	i = (lb+rb) / 2;
 
-	char* pKey = (char*)((long long)_pI + sizeof(int)) + 2 * sizeof(int) + i * getEntrySize();
+	char* pKey = (char*)((long long)_pI + sizeof(int)) + sizeof(PageIdType) + i * getEntrySize();
 	CegoBTreeValue nv;
-	nv.setPtr(pKey);
+	nv.setPtr(pKey, _keyLen);
 
-	// cout << "getChildPage : Comparing " << iv.toChain(_pSchema) << " and " << nv.toChain(_pSchema) << "i=" << i << "ne=" << ne << endl;
+	// cout << "getChildPage : Comparing " << iv.toChain(_pSchema) << " and " << nv.toChain(_pSchema) << " i=" << i << " ne=" << ne << endl;
 
-	if ( iv.isHigher(nv, _pSchema) )
+	bool traceMatch;
+	if ( traceMin )
+	    traceMatch = iv.isHigher(nv, _pSchema);
+	else
+	    traceMatch = iv.isEqualHigher(nv, _pSchema);
+	
+	if ( traceMatch )
 	{
-	    i++;
+	    lb=i;
 	}
 	else
 	{
-	    posFound = true;
+	    rb=i;	    
 	}
+	
+	if ( rb - lb <= 1 )
+	{
+	    if ( lb != rb )
+	    {
+		// cout << "Checking left value .." << endl;
+		// we still have to check the left value
+		char* pLeftKey = (char*)((long long)_pI + sizeof(int)) + sizeof(PageIdType) + lb * getEntrySize();
+		CegoBTreeValue lv;
+		lv.setPtr(pLeftKey, _keyLen);
+		
+		if ( iv.isHigher(lv, _pSchema) )
+		{	    
+		    i=rb;
+		}
+		else
+		{
+		    i=lb;
+		}
+	    }
+	    else
+	    {
+		// evaluation can be skipped
+		i=rb;
+	    }
+	    posFound=true;
+	}      
     }
 
-    /*
-    cout << "Offset is " <<  sizeof(int) + 2 * sizeof(int) + i * getEntrySize() << endl;
-    cout << "Position is " << i << endl;
-    cout << "NumEntries are " << ne << endl;
-    */
-
     char* p = (char*)((long long)_pI + sizeof(int)) + i * getEntrySize();
-    
-    memcpy(&fileId, p, sizeof(int));
-    p+=sizeof(int);
-    memcpy(&pageId, p, sizeof(int));
-
-    // cout << "Found child page " << pageId << endl;
 
+    memcpy(&pageId, p, sizeof(PageIdType));    
     return;
 }
 
-void CegoBTreeNode::setSchema(ListT<CegoField>* pSchema, int keyLen)
+void CegoBTreeNode::setSchema(NodeType nt, ListT<CegoField>* pSchema, int keyLen)
 {
     _pSchema = pSchema;
     _keyLen = keyLen;
+    _nt = nt;
+
+    if ( _nt == LEAF )
+    {
+	_maxEntries = ( _len - sizeof(int) )  / ( _keyLen + DPENCODINGLENGTH );
+	_entrySize = _keyLen + DPENCODINGLENGTH;
+    }
+    else if ( _nt == NODE )
+    {	
+	_maxEntries = ( _len - sizeof(int) - sizeof(PageIdType) ) / ( _keyLen + sizeof(PageIdType) );
+	_entrySize = _keyLen + sizeof(PageIdType);
+    }
+    else
+    {
+	throw Exception(EXLOC, Chain("Unknown btree node type"));
+    }
 }
 
-bool CegoBTreeNode::addValue(const CegoBTreeValue& iv, const CegoDataPointer& dp, const Chain& btreeName, bool isUnique, CegoObjectManager *pOM, int tabSetId, unsigned long long tid, bool& duplicateAdded)
-{
-    duplicateAdded = false;
-
-    if ( isUnique && __allowDuplicateNull && iv.hasNull(*_pSchema))
-	isUnique=false;
-    
+bool CegoBTreeNode::valueExists(const CegoBTreeValue& iv, CegoObjectManager *pOM, int tabSetId, unsigned long long tid) const
+{    
     if ( _nt == NODE )
-	     throw Exception(EXLOC, Chain("Invalid method for node type"));
+      throw Exception(EXLOC, Chain("Invalid method for node type"));
     
     int ne = numEntries();
     
-    if ( ne == maxEntries() )
-    {
-	CegoDataPointer dp1;
-	if ( valueExists(iv, dp1) )
-	{
-	    // if we have detected duplicate on unique btree, we can throw exception
-	    if ( isUnique && ! ( tid && checkForDeleted(pOM, dp1, tabSetId, tid) ) )
-	    {		
-		Chain msg = Chain("Duplicate key ") + iv.toChain(_pSchema) + Chain(" on unique btree ") + btreeName;
-		throw Exception(EXLOC, msg);		    
-	    }
-	    
-	    duplicateAdded=true;
-	}
-	return false;
-    }
-
     if ( ne == 0 )
     {
-	char* pKey = (char*)((long long)_pI + sizeof(int));
-	memcpy(pKey, iv.getPtr(), _keyLen);
-	char* pDP = (char*)((long long)pKey + _keyLen);
-	dp.encode(pDP);
-	
-	incEntries();
-	return true;	
+	return false;
     }
-
-    bool isInserted = false;
         
     int lb=0;
     int rb=ne;
@@ -306,79 +259,185 @@ bool CegoBTreeNode::addValue(const CegoBTreeValue& iv, const CegoDataPointer& dp
     while ( posFound == false )
     {
 	// cout << "lb=" << lb << " rb=" << rb << endl;
-
+	
 	i = (lb+rb) / 2;
 
+	// cout << "i=" << i << endl;
+	
 	// cout << "Checking position " << i << endl;
 
+	CegoBTreeValue nv;
+	CegoDataPointer dp;
+
+	nv.setPtr((char*)((long long)_pI + sizeof(int)) + i * getEntrySize(), _keyLen);
+	dp.decode((char*)((long long)_pI + sizeof(int)) + i * getEntrySize() + _keyLen);
+	
+	CegoBTreeValue::Comparison valueComp = iv.comp(nv, _pSchema);
+	
+	if ( valueComp == CegoBTreeValue::EQUAL )
+	{
+	    if ( isTupleVisible(pOM, dp, tabSetId, tid) )
+	    {
+		return true;
+	    }
+	}
+
+	if ( rb - lb <= 1 )
+	{
+	    posFound=true;
+	}
+	else
+	{
+	    if ( valueComp == CegoBTreeValue::MORE ) 
+	    {
+		// have we reached end of page ? if so, no more entries to check
+		if ( i == ne )
+		    posFound = true;
+		else
+		    lb=i;
+	    }
+	    else if ( valueComp == CegoBTreeValue::LESS )
+	    {	   
+		// have we beginning of page ? if so, no more entries to check
+		if ( i == 0 )
+		    posFound = true;
+		else
+		    rb=i;
+	    }
+	    else // if ( valueComp == CegoBTreeValue::EQUAL ) 
+	    {
+
+		// cout << "Searching down and up .." << endl;
+		// if comparison is equal, we have to look downstairs and upstairs for
+		// other equal and check for visibility
+		
+		// search down
+		
+		int d = i-1;
+		bool goOn = true;
+		while ( d >= 0 && goOn )
+		{
+		    // cout << "Searching down " << d << endl;
+		    nv.setPtr((char*)((long long)_pI + sizeof(int)) + (d)*getEntrySize(), _keyLen);
+		    dp.decode((char*)((long long)_pI + sizeof(int)) + (d)*getEntrySize() + _keyLen);
+
+		    CegoBTreeValue::Comparison downComp = iv.comp(nv, _pSchema);
+
+		    if ( downComp == CegoBTreeValue::EQUAL )
+		    {
+			if ( isTupleVisible(pOM, dp, tabSetId, tid) )
+			{
+			    return true;
+			}
+		    }
+		    else
+		    {
+			goOn = false;
+		    }
+		    
+		    d--;
+		}
+
+		// search up
+		
+		int u = i+1;
+		goOn = true;
+		while ( u < ne && goOn )
+		{
+		    // cout << "Searching up " << u << endl;
+		    nv.setPtr((char*)((long long)_pI + sizeof(int)) + (u)*getEntrySize(), _keyLen);
+		    dp.decode((char*)((long long)_pI + sizeof(int)) + (u)*getEntrySize() + _keyLen);
+
+		    CegoBTreeValue::Comparison upComp = iv.comp(nv, _pSchema);
+
+		    if ( upComp == CegoBTreeValue::EQUAL )
+		    {
+			if ( isTupleVisible(pOM, dp, tabSetId, tid) )
+			{
+			    return true;
+			}
+		    }
+		    else
+		    {
+			goOn = false;
+		    }
+		    u++;
+		}
+
+		return false;
+	    }
+	}
+    }
+    return false;
+}
+
+bool CegoBTreeNode::addValue(const CegoBTreeValue& iv, const CegoDataPointer& dp)
+{    
+    if ( _nt == NODE )
+      throw Exception(EXLOC, Chain("Invalid method for node type"));
+    
+    int ne = numEntries();
+    
+    if ( ne == _maxEntries )
+    {
+	return false;
+    }
+
+    if ( ne == 0 )
+    {
+	char* pKey = (char*)((long long)_pI + sizeof(int));
+	memcpy(pKey, iv.getPtr(), _keyLen);
+	char* pDP = (char*)((long long)pKey + _keyLen);
+	dp.encode(pDP);
+	
+	incEntries();
+
+	return true;	
+    }
+        
+    int lb=0;
+    int rb=ne;
+
+    int i = (lb+rb) / 2;
+    
+    bool posFound=false;
+
+    while ( posFound == false )
+    {
+	i = (lb+rb) / 2;
+
 	CegoBTreeValue nv1;
 	CegoBTreeValue nv2;
 
 	CegoDataPointer dp1;
 	CegoDataPointer dp2;
 
-	nv1.setPtr((char*)((long long)_pI + sizeof(int)) + i * getEntrySize());
+	nv1.setPtr((char*)((long long)_pI + sizeof(int)) + i * getEntrySize(), _keyLen);
 	dp1.decode((char*)((long long)_pI + sizeof(int)) + i * getEntrySize() + _keyLen);
 
 	CegoBTreeValue::Comparison valueComp1 = iv.comp(nv1, _pSchema);
 	CegoBTreeValue::Comparison valueComp2;
-
+	
 	if ( i+1 == ne )
 	{
-	    // cout << "XXXX Fixing v2 to less .." << endl;
 	    valueComp2 = CegoBTreeValue::LESS;
 	}
 	else
 	{
-	    nv2.setPtr((char*)((long long)_pI + sizeof(int)) + (i+1)*getEntrySize());
+	    nv2.setPtr((char*)((long long)_pI + sizeof(int)) + (i+1)*getEntrySize(), _keyLen);
 	    dp2.decode((char*)((long long)_pI + sizeof(int)) + (i+1)*getEntrySize() + _keyLen);
 
 	    valueComp2 = iv.comp(nv2, _pSchema);
 	}
-		
-	// cout << "VC1=" << valueComp1 << endl;
-	// cout << "VC2=" << valueComp2 << endl;
-
-	// cout << "Comparing " << iv.toChain(_pSchema) << " with " << nv1.toChain(_pSchema) << " and " << nv2.toChain(_pSchema) << endl;
-
-	if ( valueComp1 == CegoBTreeValue::EQUAL || valueComp2 == CegoBTreeValue::EQUAL )
-	{       	    
-	    if ( valueComp1 == CegoBTreeValue::EQUAL )
-	    {
-		if ( isUnique && ! ( tid && checkForDeleted(pOM, dp1, tabSetId, tid) ) )
-		{
-		    Chain msg = Chain("Duplicate key ") + iv.toChain(_pSchema) + Chain(" on unique btree ") + btreeName;
-		    throw Exception(EXLOC, msg);		    
-		}
-		else
-		{
-		    duplicateAdded = true;
-		}
-	    }
-	    if (  valueComp2 == CegoBTreeValue::EQUAL )
-	    {
-		if ( isUnique && ! ( tid && checkForDeleted(pOM, dp2, tabSetId, tid) ) )
-		{
-		    Chain msg = Chain("Duplicate key ") + iv.toChain(_pSchema) + Chain(" on unique btree ") + btreeName;
-		    throw Exception(EXLOC, msg);		    
-		}
-		else
-		{
-		    duplicateAdded = true;
-		}
-	    }
-	}
-
+	
 	if ( ( valueComp1 == CegoBTreeValue::MORE || valueComp1 == CegoBTreeValue::EQUAL )
 	     && ( valueComp2 == CegoBTreeValue::LESS || valueComp2 == CegoBTreeValue::EQUAL ) )
 	{
-	    // cout << "CASE-A" << endl;
 	    i++;
 	    posFound=true;
 	}
 	else if ( valueComp2 == CegoBTreeValue::MORE )
 	{
-	    // cout << "CASE-B" << endl;
 	    if ( i == ne )
 		posFound = true;
 	    else
@@ -386,17 +445,13 @@ bool CegoBTreeNode::addValue(const CegoBTreeValue& iv, const CegoDataPointer& dp
 	}
 	else
 	{
-	    // cout << "CASE-C" << endl;
 	    if ( i == 0 )
 		posFound = true;
 	    else
 		rb=i;
 	}
-	// sleep(1);
     }
 
-    // cout << "Position found at : " << i << endl;
-
     if ( i < ne )
     {
 	shiftEntries(i, 1);
@@ -408,6 +463,7 @@ bool CegoBTreeNode::addValue(const CegoBTreeValue& iv, const CegoDataPointer& dp
     dp.encode(pDP);
 
     incEntries();
+
     return true;
 }
 
@@ -424,64 +480,26 @@ bool CegoBTreeNode::deleteValue(const CegoBTreeValue& iv, const CegoDataPointer&
     char* pKey = (char*)((long long)_pI + sizeof(int));
 
     CegoBTreeValue lv;
-
-    bool isInserted = false;
     
     int i=0;
     while ( i < ne )
     {
-	lv.setPtr(pKey);
+	lv.setPtr(pKey, _keyLen);
 
 	CegoDataPointer ldp;
 	ldp.decode(pKey + _keyLen);
-
+	
 	if ( lv.isEqual(iv, _pSchema) && ldp == dp)
 	{
 	    shiftEntries(i+1, -1);	    
 	    decEntries();
 	    return true; 	    
 	}
+	
 	pKey += getEntrySize();
 	i++;
     }
-
-    return false;
-}
-
-bool CegoBTreeNode::valueExists(const CegoBTreeValue& iv)
-{
-    CegoDataPointer dp;
-    return valueExists(iv, dp);
-}
-
-bool CegoBTreeNode::valueExists(const CegoBTreeValue& iv, CegoDataPointer& dp)
-{
-    if ( _nt == NODE )
-	throw Exception(EXLOC, Chain("Invalid method for node type"));
-
-    int ne = numEntries();
-
-    if ( ne == 0 )
-	return false;
-
-    char* pKey = (char*)((long long)_pI + sizeof(int));
-
-    CegoBTreeValue lv;
     
-    int i=0;
-    while ( i < ne )
-    {
-	lv.setPtr(pKey);
-
-	if ( lv.isEqual(iv, _pSchema))
-	{
-	    dp.decode(pKey + _keyLen);
-	    return true; 	    
-	}
-	pKey += getEntrySize();
-	i++;
-    }
-
     return false;
 }
 
@@ -491,18 +509,19 @@ bool CegoBTreeNode::addNode(const CegoBTreeValue& iv, const CegoBTreeNode& node)
 	throw Exception(EXLOC, Chain("Invalid method for node type"));
     
     int ne = numEntries();       	
-    if ( ne == maxEntries() || ne == 0 )
+    if ( ne == _maxEntries || ne == 0 )
+    {
 	return false;
-    
+    }
+
     bool posFound=false;
     char* p = 0;
     int i=0;	    
     
     while ( i < ne && posFound == false)
-    {
-	
-	p = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + i * getEntrySize());
-	CegoBTreeValue nv(p);
+    {	
+	p = (char*)((long long)_pI + sizeof(int) + sizeof(PageIdType) + i * getEntrySize());
+	CegoBTreeValue nv(p, _keyLen);
 	if ( nv.isHigher(iv, _pSchema) )
 	{
 	    shiftEntries(i, 1);
@@ -531,19 +550,16 @@ bool CegoBTreeNode::addNode(const CegoBTreeValue& iv, const CegoBTreeNode& node)
     // Setting up new node information //
     ////////////////////////////////////
 
-    p = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + i * getEntrySize());
+    p = (char*)((long long)_pI + sizeof(int) + sizeof(PageIdType) + i * getEntrySize());
     
     // put index value
     memcpy(p, iv.getPtr(), _keyLen);
     p+=_keyLen;
     
     // put child pointer
-    int fileId = node.getFileId();
-    memcpy(p, &fileId, sizeof(int));
-    p+=sizeof(int);
-    int pageId = node.getPageId();
-    memcpy(p, &pageId, sizeof(int));
-    p+=sizeof(int);
+    PageIdType pageId = node.getPageId();
+    memcpy(p, &pageId, sizeof(PageIdType));
+    p+=sizeof(PageIdType);
     
     incEntries();
     
@@ -552,7 +568,7 @@ bool CegoBTreeNode::addNode(const CegoBTreeValue& iv, const CegoBTreeNode& node)
 
 void CegoBTreeNode::split(CegoBTreeNode& n)
 {
-    int pos; 
+    int pos = 0; 
 
     if ( _nt == LEAF )
     {
@@ -560,17 +576,22 @@ void CegoBTreeNode::split(CegoBTreeNode& n)
 	char* srcPtr = (char*)((long long)_pI + sizeof(int) + pos * getEntrySize());
 	char* destPtr = (char*)((long long)n._pI + sizeof(int));
 	int s =  ( numEntries() - pos ) * getEntrySize();
-	memcpy(destPtr, srcPtr, s);
-	
+
+	memcpy(destPtr, srcPtr, s);	
     }
     else if ( _nt == NODE )
     {
 	pos = numEntries() / 2;
 	char* srcPtr = (char*)((long long)_pI + sizeof(int) + pos * getEntrySize());
 	char* destPtr = (char*)((long long)n._pI + sizeof(int));
-	int s =  ( numEntries() - pos ) * getEntrySize() + 2 * sizeof(int);
+	int s =  ( numEntries() - pos ) * getEntrySize() + sizeof(PageIdType);
+
 	memcpy(destPtr, srcPtr, s);	
     }
+    else
+    {
+	throw Exception(EXLOC, Chain("Invalid method for undefined node type"));
+    }
 
     int s1 = pos;
     int s2 = numEntries() - pos;
@@ -585,13 +606,15 @@ void CegoBTreeNode::shiftEntries(int pos, int offset)
 	char* srcPtr = (char*)((long long)_pI + sizeof(int) + pos * getEntrySize());
 	char* destPtr = (char*)((long long)_pI + sizeof(int) + ( pos + offset) * getEntrySize());
 	int shiftSize = ( numEntries() - pos ) * getEntrySize();
+
 	memmove(destPtr, srcPtr, shiftSize);
     }
     else if ( _nt == NODE )
     {
-	char* srcPtr = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + pos * getEntrySize());
-	char* destPtr = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + ( pos + offset ) * getEntrySize());
+	char* srcPtr = (char*)((long long)_pI + sizeof(int) + sizeof(PageIdType) + pos * getEntrySize());
+	char* destPtr = (char*)((long long)_pI + sizeof(int) + sizeof(PageIdType) + ( pos + offset ) * getEntrySize());
 	int shiftSize = ( numEntries() - pos ) * getEntrySize();
+
 	memmove(destPtr, srcPtr, shiftSize);
     }
 }
@@ -604,13 +627,13 @@ CegoBTreeValue CegoBTreeNode::getMin() const
     if ( _nt == LEAF )    
     {
 	char* p = (char*)((long long)_pI + sizeof(int));
-	CegoBTreeValue iv(p);
+	CegoBTreeValue iv(p, _keyLen);
 	return iv;
     }
     else if ( _nt == NODE )
     {
-	char* p = (char*)((long long)_pI + sizeof(int) + 2*sizeof(int));
-	CegoBTreeValue iv(p);
+	char* p = (char*)((long long)_pI + sizeof(int) + sizeof(PageIdType));
+	CegoBTreeValue iv(p, _keyLen);
 	return iv;
     }
     else
@@ -623,7 +646,7 @@ CegoBTreeValue CegoBTreeNode::getMax() const
 {
     int pos = numEntries() - 1;
 
-    if ( pos > 0 )
+    if ( pos >= 0 )
     {
 	char* p;
 	if ( _nt == LEAF )
@@ -632,9 +655,9 @@ CegoBTreeValue CegoBTreeNode::getMax() const
 	}
 	else
 	{
-	    p = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + pos * getEntrySize());
+	    p = (char*)((long long)_pI + sizeof(int) + sizeof(PageIdType) + pos * getEntrySize());
 	}
-	CegoBTreeValue iv(p);
+	CegoBTreeValue iv(p, _keyLen);
 	return iv;
 
     }
@@ -647,36 +670,32 @@ bool CegoBTreeNode::propagate(const CegoBTreeValue& iv, const CegoBTreeNode& lef
 	throw Exception(EXLOC, Chain("Invalid method for node type"));
  
     int ne = numEntries();       	
-    if ( ne == maxEntries() )
+    if ( ne == _maxEntries )
+    {
 	return false;
-    
+    }
+
     if ( ne == 0 )
     {
 	// first entry
 	
 	char* p = (char*)((long long)_pI + sizeof(int));
-	int fileId, pageId, nextFileId, nextPageId;
+	PageIdType pageId, nextPageId;
 	
 	// put left child pointer
-	fileId = leftNode.getFileId();
-	memcpy(p, &fileId, sizeof(int));
-	p+=sizeof(int);
 	
 	pageId = leftNode.getPageId();
-	memcpy(p, &pageId, sizeof(int));
-	p+=sizeof(int);
+	memcpy(p, &pageId, sizeof(PageIdType));
+	p+=sizeof(PageIdType);
 
 	// put index value
 	memcpy(p, iv.getPtr(), _keyLen);
 	p+=_keyLen;
 	
 	// put right child pointer
-	nextFileId = rightNode.getFileId();
-	memcpy(p, &nextFileId, sizeof(int));
-	p+=sizeof(int);
 	nextPageId = rightNode.getPageId();
-	memcpy(p, &nextPageId, sizeof(int));
-	p+=sizeof(int);
+	memcpy(p, &nextPageId, sizeof(PageIdType));
+	p+=sizeof(PageIdType);
 	
 	incEntries();
 	
@@ -684,15 +703,14 @@ bool CegoBTreeNode::propagate(const CegoBTreeValue& iv, const CegoBTreeNode& lef
     }
     else
     {
-
 	bool posFound=false;
 	char* p = 0;
 	int i=0;	    
 
 	while ( i < ne && posFound == false)
 	{
-	    p = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + i * getEntrySize());
-	    CegoBTreeValue nv(p);
+	    p = (char*)((long long)_pI + sizeof(int) + sizeof(PageIdType) + i * getEntrySize());
+	    CegoBTreeValue nv(p, _keyLen);
 
 	    // cout << "Comparing " << iv.toChain(_pSchema) << " and " << nv.toChain(_pSchema) << "i=" << i << "ne=" << ne << endl;
 	    
@@ -724,24 +742,20 @@ bool CegoBTreeNode::propagate(const CegoBTreeValue& iv, const CegoBTreeNode& lef
 	// Setting up new node information //
 	////////////////////////////////////
 
-	p = (char*)((long long)_pI + sizeof(int) + 2 * sizeof(int) + i * getEntrySize());
+	p = (char*)((long long)_pI + sizeof(int) + sizeof(PageIdType) + i * getEntrySize());
 	
 	// put index value
 	memcpy(p, iv.getPtr(), _keyLen);
 	p+=_keyLen;
 	
 	// put child pointer
-	int fileId = rightNode.getFileId();
-	memcpy(p, &fileId, sizeof(int));
-	p+=sizeof(int);
-	int pageId = rightNode.getPageId();
-	memcpy(p, &pageId, sizeof(int));
-	p+=sizeof(int);
+	PageIdType pageId = rightNode.getPageId();
+	memcpy(p, &pageId, sizeof(PageIdType));
+	p+=sizeof(PageIdType);
 	
 	incEntries();
 	
 	return true;
-
     }
 }
  
@@ -750,11 +764,11 @@ CegoBTreeNode& CegoBTreeNode::operator = (const CegoBTreeNode& n)
     _nt = n._nt;
     _pI = n._pI;
     _len = n._len;
-    _fileId = n._fileId;
     _pageId = n._pageId;
-    _nextFileId = n._nextFileId;
     _nextPageId = n._nextPageId;
     _keyLen = n._keyLen;
+    _entrySize = n._entrySize;
+    _maxEntries = n._maxEntries;
     _pSchema = n._pSchema;
     return (*this);
 }
@@ -770,13 +784,13 @@ bool CegoBTreeNode::verify()
 	{	    
 	    char* srcPtr = (char*)((long long)_pI + sizeof(int) + i * getEntrySize());
 	    CegoBTreeValue iv;
-	    iv.setPtr(srcPtr);
+	    iv.setPtr(srcPtr, _keyLen);
 	    srcPtr += _keyLen;
 	    CegoDataPointer dp;
 	    dp.decode(srcPtr);
 
-	    // we check fileId and offset, must not be zero in any case
-	    if ( dp.getFileId() == 0 || dp.getOffset() == 0 )
+	    // we check pageId and offset, must not be zero in any case
+	    if ( dp.getPageId() == 0 || dp.getOffset() == 0 )
 	    {
 		return false;
 	    }
@@ -786,18 +800,16 @@ bool CegoBTreeNode::verify()
     }
     else if ( _nt == NODE )
     {
-	int fileId, pageId;
+	PageIdType pageId;
 
 	char* p = (char*)((long long)_pI + sizeof(int));
 	
-	memcpy(&fileId, p, sizeof(int));
-	p+=sizeof(int);
-	memcpy(&pageId, p, sizeof(int));
+	memcpy(&pageId, p, sizeof(PageIdType));
 			  
 	// cout << levelIndent(level) << "Child : " << fileId << "," << pageId << endl;
 
 	// we check fileId, must not be zero in any case
-	if ( fileId == 0  )
+	if ( pageId == 0  )
 	{
 	    return false;
 	}
@@ -807,22 +819,20 @@ bool CegoBTreeNode::verify()
 	{
 	    // cout << levelIndent(level) << "Key " << i << " : ";
 	    
-	    p = (char*)((long long)_pI + sizeof(int) + 2*sizeof(int) + i * getEntrySize());
+	    p = (char*)((long long)_pI + sizeof(int) + sizeof(PageIdType) + i * getEntrySize());
 	    CegoBTreeValue iv;
-	    iv.setPtr(p);
+	    iv.setPtr(p, _keyLen);
 	    // cout << iv.toChain(_pSchema) << endl;
 	    p += _keyLen;
 	    
-	    memcpy(&fileId, p, sizeof(int));
-	    p+=sizeof(int);
-	    memcpy(&pageId, p, sizeof(int));
+	    memcpy(&pageId, p, sizeof(PageIdType));
 
 	    // we check fileId, must not be zero in any case
-	    if ( fileId == 0 )
+	    if ( pageId == 0 )
 	    {
 		return false;
 	    }
-
+	    
 	    // cout << levelIndent(level) << "Child : " << fileId << "," << pageId << endl;
 
 	    i++;
@@ -833,19 +843,96 @@ bool CegoBTreeNode::verify()
     {
 	return false;
     }
-
+    
     return true;
 }
 
+bool CegoBTreeNode::verifyLeafFull(int tabSetId, CegoObjectManager *pOM)
+{
+    int n = numEntries();
+    
+    if ( _nt == LEAF )
+    {
+	int i = 0;
+	while ( i < n )
+	{	    
+	    char* srcPtr = (char*)((long long)_pI + sizeof(int) + i * getEntrySize());
+	    CegoBTreeValue iv;
+	    iv.setPtr(srcPtr, _keyLen);
+	    
+	    srcPtr += _keyLen;
+	    CegoDataPointer dp;
+	    dp.decode(srcPtr);
+
+	    // we check pageId and offset, must not be zero in any case
+	    if ( dp.getPageId() == 0 || dp.getOffset() == 0 )
+	    {
+		return false;
+	    }
+	    else
+	    {		
+		char* p;
+		int len;
+
+		CegoBufferPage bp;
+		pOM->claimDataPtrUnlocked(tabSetId, CegoBufferPool::SYNC, dp, p, len, bp);
+		
+		// skipping tid
+		
+		unsigned long long tid;
+		unsigned long long tastep;
+		CegoTupleState ts;
+
+		CegoQueryHelper qh;
+		int toff = qh.decodeTupleHeader(tid, tastep, ts, p);
+		
+		char* tp = p + toff;
+		int tlen = len - toff;
+
+		ListT<CegoField> fl = *_pSchema;
+		qh.decodeFVL(fl, tp, tlen);
+		
+		ListT<CegoFieldValue> idxFVL = iv.valueToFVL(*_pSchema);
+
+		CegoField *pF = fl.First();
+		CegoFieldValue *pIV = idxFVL.First();
+
+		bool mismatchDetected = false;
+		while ( pF && pIV )
+		{
+		    // cout << "Comparing " << pF->getValue().toChain() << " and " << pIV->toChain() << endl;
+		    if ( *pIV != pF->getValue() )
+			mismatchDetected = true;
+		    
+		    pF = fl.Next();
+		    pIV = idxFVL.Next();
+		}
+		    
+		pOM->releaseDataPtrUnlocked(bp, false);
+
+		if ( mismatchDetected )
+		    return false;	       
+	    }
+
+	    i++;
+	}
+
+	return true;
+    }
+    else
+    {
+	throw Exception(EXLOC, Chain("Invalid node type"));
+    }
+}
+
 void CegoBTreeNode::printNode(int level)
 {
     int n = numEntries();
 
     if ( _nt == LEAF )
     {
-
 	cout << levelIndent(level) << "------- LEAF -------" << endl;
-	cout << levelIndent(level) << "NumEntries=" << n << " FileId=" << _fileId << " PageId=" << _pageId << endl;
+	cout << levelIndent(level) << "NumEntries=" << n << " PageId=" << _pageId << endl;
 
 	int i = 0;
 	while ( i < n )
@@ -854,48 +941,43 @@ void CegoBTreeNode::printNode(int level)
 	    
 	    char* srcPtr = (char*)((long long)_pI + sizeof(int) + i * getEntrySize());
 	    CegoBTreeValue iv;
-	    iv.setPtr(srcPtr);
+	    iv.setPtr(srcPtr, _keyLen);
 	    srcPtr += _keyLen;
 	    CegoDataPointer dp;
 	    dp.decode(srcPtr);
 	    cout << iv.toChain(_pSchema) << " " << dp << endl;	  
 	    i++;
 	}
-	cout << levelIndent(level) << "Next Page : " << _nextFileId << " " << _nextPageId << endl;
+	cout << levelIndent(level) << "Next Page : " << _nextPageId << endl;
 	cout << levelIndent(level) << "--------------------" << endl;
     }
     else if ( _nt == NODE )
     {
 	cout << levelIndent(level) << "####### NODE ####### " << endl;
-	cout << levelIndent(level) << "NumEntries=" << n << " FileId=" << _fileId << " PageId=" << _pageId << endl;
+	cout << levelIndent(level) << "NumEntries=" << n << " PageId=" << _pageId << endl;
 
-	int fileId, pageId;
+	PageIdType pageId;
 
 	char* p = (char*)((long long)_pI + sizeof(int));
 	
-	memcpy(&fileId, p, sizeof(int));
-	p+=sizeof(int);
-	memcpy(&pageId, p, sizeof(int));
+	memcpy(&pageId, p, sizeof(PageIdType));
 			  
-	cout << levelIndent(level) << "Child : " << fileId << "," << pageId << endl;
+	cout << levelIndent(level) << "Child : " << pageId << endl;
 
 	int i = 0;
 	while ( i < n )
 	{
 	    cout << levelIndent(level) << "Key " << i << " : ";
 	    
-	    p = (char*)((long long)_pI + sizeof(int) + 2*sizeof(int) + i * getEntrySize());
+	    p = (char*)((long long)_pI + sizeof(int) + sizeof(PageIdType) + i * getEntrySize());
 	    CegoBTreeValue iv;
-	    iv.setPtr(p);
+	    iv.setPtr(p, _keyLen);
 	    cout << iv.toChain(_pSchema) << endl;
 	    p += _keyLen;
 	    
-	    memcpy(&fileId, p, sizeof(int));
-	    p+=sizeof(int);
-	    memcpy(&pageId, p, sizeof(int));
+	    memcpy(&pageId, p, sizeof(PageIdType));
 	    
-	    cout << levelIndent(level) << "Child : " << fileId << "," << pageId << endl;
-
+	    cout << levelIndent(level) << "Child : " << pageId << endl;
 
 	    i++;
 	}
@@ -912,7 +994,12 @@ void CegoBTreeNode::reset()
     _pNextChild = (char*)((long long)_pI + sizeof(int));
 }
 
-bool CegoBTreeNode::nextChildPointer(int &fileId, int &pageId)
+void CegoBTreeNode::setPosition(int pos)
+{
+    _pNextChild = (char*)((long long)_pI + sizeof(int) + pos * getEntrySize());
+}
+
+bool CegoBTreeNode::nextChildPointer(PageIdType &pageId)
 {
     if ( _nt == LEAF )
 	throw Exception(EXLOC, Chain("Invalid method for node type"));
@@ -920,46 +1007,65 @@ bool CegoBTreeNode::nextChildPointer(int &fileId, int &pageId)
     if ( (long long)_pNextChild > (long long)_pI + sizeof(int) + numEntries() * getEntrySize() )
 	return false;
     
-    memcpy(&fileId, _pNextChild, sizeof(int));
-    _pNextChild+=sizeof(int);
-    memcpy(&pageId, _pNextChild, sizeof(int));
-    _pNextChild+=sizeof(int);
+    memcpy(&pageId, _pNextChild, sizeof(PageIdType));
+    _pNextChild+=sizeof(PageIdType);
     
     _pNextChild+=_keyLen;
     
     return true;
 }
 
-void CegoBTreeNode::getLastChildPointer(int &fileId, int &pageId) const
+int CegoBTreeNode::getMedPage(int lb, int rb, PageIdType& pageId)
+{
+    if ( _nt == LEAF )
+	throw Exception(EXLOC, Chain("Invalid method for node type"));
+    
+    int med = (lb + rb) / 2;
+    char* _pMed = (char*)((long long)_pI + sizeof(int) + med * getEntrySize());
+    
+    memcpy(&pageId, _pMed, sizeof(PageIdType));
+   
+    return med;
+}
+
+bool CegoBTreeNode::rightValue(int pos, CegoBTreeValue& val)
+{
+    if ( _nt == LEAF )
+	throw Exception(EXLOC, Chain("Invalid method for node type"));
+
+    char* pChild = (char*)((long long)_pI + sizeof(int) + pos * getEntrySize());
+    if ( (long long)pChild > (long long)_pI + sizeof(int) + numEntries() * getEntrySize() )
+	return false;
+        
+    pChild+=sizeof(PageIdType);
+    
+    val.setPtr(pChild, _keyLen);
+    return true;
+}
+
+void CegoBTreeNode::getLastChildPointer(PageIdType &pageId) const
 {
     if ( _nt == LEAF )
 	throw Exception(EXLOC, Chain("Invalid method for node type"));
 
     if ( numEntries() == 0 )
     {
-	fileId = 0;
 	pageId = 0;
     }
     else
     {
-	char *p = (char*)((long long)_pI + sizeof(int) + numEntries() * getEntrySize());
-    
-	memcpy(&fileId, p, sizeof(int));
-	p+=sizeof(int);
-	memcpy(&pageId, p, sizeof(int));
+	char *p = (char*)((long long)_pI + sizeof(int) + numEntries() * getEntrySize());    
+	memcpy(&pageId, p, sizeof(PageIdType));
     }
 }
 
-void CegoBTreeNode::setFirstChildPointer(int fileId, int pageId)
+void CegoBTreeNode::setFirstChildPointer(PageIdType pageId)
 {
     if ( _nt == LEAF )
 	throw Exception(EXLOC, Chain("Invalid method for node type"));
 
-    char *p = (char*)((long long)_pI + sizeof(int));
-    
-    memcpy(p, &fileId, sizeof(int));
-    p+=sizeof(int);
-    memcpy(p, &pageId, sizeof(int));
+    char *p = (char*)((long long)_pI + sizeof(int));    
+    memcpy(p, &pageId, sizeof(PageIdType));
 }
 
 bool CegoBTreeNode::rightChild(CegoBTreeValue& val)
@@ -970,10 +1076,25 @@ bool CegoBTreeNode::rightChild(CegoBTreeValue& val)
     if ( (long long)_pNextChild > (long long)_pI + sizeof(int) + numEntries() * getEntrySize() )
 	return false;
     
-    val.setPtr(_pNextChild - _keyLen);
+    val.setPtr(_pNextChild - _keyLen, _keyLen);
     return true;
 }
 
+void CegoBTreeNode::getValue(int pos, CegoBTreeValue& val, CegoDataPointer& dp)
+{
+    if ( _nt == NODE )
+	throw Exception(EXLOC, Chain("Invalid method for node type"));
+
+    if ( pos >= numEntries() )
+	throw Exception(EXLOC, Chain("Node position out of range"));
+
+    char* pVal = (char*)((long long)_pI + sizeof(int) + pos * getEntrySize());
+    
+    val.setPtr(pVal, _keyLen);
+    pVal +=_keyLen;
+    dp.decode(pVal);
+}
+
 bool CegoBTreeNode::nextValue(CegoBTreeValue& val, CegoDataPointer& dp)
 {
     if ( _nt == NODE )
@@ -982,7 +1103,7 @@ bool CegoBTreeNode::nextValue(CegoBTreeValue& val, CegoDataPointer& dp)
     if ( (long long)_pNextChild >= (long long)_pI + sizeof(int) + numEntries() * getEntrySize() )
 	return false;
     
-    val.setPtr(_pNextChild);
+    val.setPtr(_pNextChild, _keyLen);
     _pNextChild +=_keyLen;
     dp.decode(_pNextChild);
     _pNextChild += DPENCODINGLENGTH;
@@ -990,6 +1111,7 @@ bool CegoBTreeNode::nextValue(CegoBTreeValue& val, CegoDataPointer& dp)
     return true;    
 }
 
+
 Chain CegoBTreeNode::levelIndent(int level)
 {
     Chain s;
@@ -1001,12 +1123,13 @@ Chain CegoBTreeNode::levelIndent(int level)
     return s;
 }
 
-bool CegoBTreeNode::checkForDeleted(CegoObjectManager* pOM, const CegoDataPointer &dp, int tabSetId, unsigned long long tid)
+bool CegoBTreeNode::isTupleVisible(CegoObjectManager* pOM, const CegoDataPointer &dp, int tabSetId, unsigned long long tid) const
 {    
     char* p;
     int len;
 
-    CegoBufferPage bp = pOM->claimDataPtrUnlocked(tabSetId, CegoBufferPool::SYNC, dp, p, len);
+    CegoBufferPage bp;
+    pOM->claimDataPtrUnlocked(tabSetId, CegoBufferPool::SYNC, dp, p, len, bp);
     
     // skipping tid
     unsigned long long dataTid;
@@ -1018,18 +1141,22 @@ bool CegoBTreeNode::checkForDeleted(CegoObjectManager* pOM, const CegoDataPointe
     pOM->releaseDataPtrUnlocked(bp, false);
 
     /*
+    
     cout << "Datapointer = " << dp << endl;
     cout << "tid = " << tid << endl;
     cout << "dataTid = " << dataTid << endl;
     cout << "ts = " << ts << endl;
     */
 
-    if (tid == dataTid && ( ts == DELETED || ts == OBSOLETE ))
+    if ( dataTid == 0
+	 || ( tid == dataTid && ( ts == CegoTupleState::INSERTED ) )
+	 || ( tid != dataTid && ( ts == CegoTupleState::DELETED  ) ) )
     {
 	return true;
     }
     else
     {
+	// cout << "TID = " << tid << " DATATID = " << dataTid << " TS = " << ts << endl;
 	return false;
     }
 }
