@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -32,16 +32,32 @@
 
 CegoBTreeValue::CegoBTreeValue()
 {
-    _pI = _idxArray;
+    _idxArray = 0;
+    _pI = 0;
+    _len = 0;
 }
 
-CegoBTreeValue::CegoBTreeValue(char* p)
+CegoBTreeValue::CegoBTreeValue(char* p, int len)
 {
+    _idxArray = 0;
     _pI = p;
+    _len = len;
 }
 
 CegoBTreeValue::~CegoBTreeValue()
 {
+    // is local memory allocated ? ( just used for valueFromSchema )
+    if ( _idxArray )
+	free(_idxArray);
+}
+
+bool CegoBTreeValue::isNullTerminated(CegoDataType type)
+{
+    if ( type == VARCHAR_TYPE
+	 || type == BIGINT_TYPE
+	 || type == DECIMAL_TYPE )
+	return true;
+    return false;
 }
 
 int CegoBTreeValue::getKeyLen(const ListT<CegoField>& schema)
@@ -51,12 +67,11 @@ int CegoBTreeValue::getKeyLen(const ListT<CegoField>& schema)
     while ( pF )
     {
 	keyLen += 1; // null value indicator
-	keyLen += pF->getLength();
-
-	if ( pF->getType() == VARCHAR_TYPE
-	     || pF->getType() == BIGINT_TYPE
-	     || pF->getType() == DECIMAL_TYPE
-	     || pF->getType() == FIXED_TYPE )
+	keyLen += getReservedLength(pF); 
+	          
+	// we have to add an additional character, in case of the full length
+	// of the attribute size defintion is used by the value
+	if ( isNullTerminated( pF->getType() ) )
 	    keyLen += 1; // termination character
 	 	
 	pF = schema.Next();
@@ -64,6 +79,56 @@ int CegoBTreeValue::getKeyLen(const ListT<CegoField>& schema)
     return keyLen;
 }
 
+int CegoBTreeValue::getKeyPos(const Chain& attrName, const ListT<CegoField>& schema)
+{
+    CegoField *pF = schema.First();
+    int keypos = 0;
+    while ( pF )
+    {
+	keypos += 1; // null value indicator
+	if ( pF->getAttrName() == attrName )
+	    return keypos;
+	
+	keypos += getReservedLength(pF);
+	
+	if ( isNullTerminated(pF->getType() ) )
+	    keypos++;
+
+	pF = schema.Next();
+    }
+    throw Exception(EXLOC, "Attribute not found");
+}
+
+bool CegoBTreeValue::isNull(const ListT<CegoField>& schema) const
+{
+    CegoField* pF = schema.First();
+    char* idxPtr = _pI;
+    while (pF)
+    {
+	int len = getReservedLength(pF);
+	
+	char nullIndicator;
+	memcpy(&nullIndicator, idxPtr, 1);
+	idxPtr += 1;
+
+	if ( nullIndicator == 1 )
+	{	    
+	    return false;
+	}
+	else
+	{
+	    idxPtr += len + 1;
+	    if ( isNullTerminated(pF->getType() ) )
+	    {
+		idxPtr += 1;
+	    }
+	}
+	
+	pF = schema.Next();
+    }
+    return true;
+}
+
 bool CegoBTreeValue::hasNull(const ListT<CegoField>& schema) const
 {
     CegoField* pF = schema.First();
@@ -76,21 +141,18 @@ bool CegoBTreeValue::hasNull(const ListT<CegoField>& schema) const
 
 	if ( nullIndicator == 1 )
 	{	    
-	    int len = pF->getLength();
+	    int len = getReservedLength(pF);
 	    
 	    idxPtr+=len;
-	    
-	    if ( pF->getType() == VARCHAR_TYPE
-		 || pF->getType() == BIGINT_TYPE
-		 || pF->getType() == DECIMAL_TYPE
-		 || pF->getType() == FIXED_TYPE )
+
+	    if ( isNullTerminated(pF->getType() ) )
 		idxPtr+=1;
 	}
 	else
 	{
 	    return true;
 	}
-	
+
 	pF = schema.Next();
     }
     return false;
@@ -98,49 +160,53 @@ bool CegoBTreeValue::hasNull(const ListT<CegoField>& schema) const
 
 void CegoBTreeValue::valueFromSchema(ListT<CegoField>* pSchema)
 {
-
+    // check if local memory already allocated, do if not
+    if ( _idxArray == 0 )
+	_idxArray = (char*)malloc(TABMNG_MAXINDEXVALUE);
+    _pI = _idxArray;
+    
     char* idxPtr = _pI;
     CegoField *pF = pSchema->First();
+
+    int vlen = 0;
     while (pF)
     {
-	int vlen = pF->getValue().getLength();
+	int len = pF->getValue().getLength();
+
+	// checking reserved length + 1 because value length includes zero byte  
+	if ( len > getReservedLength(pF) + 1 )
+	    throw Exception(EXLOC, "Reserved btree length for floating type exceeded");
+	
+	vlen += getReservedLength(pF);
 	
 	if ( vlen > TABMNG_MAXINDEXVALUE )
 	    throw Exception(EXLOC, "Index value exceeded");
 
-	if ( vlen > 0 )
+	if ( len > 0 )
 	{
 	    char nullIndicator = 1;
 	    memcpy(idxPtr, &nullIndicator, 1);
 	    idxPtr += 1;
-
-	    memcpy(idxPtr, pF->getValue().getValue(), vlen);
-
-	    idxPtr += pF->getLength();
-
-	    if ( pF->getType() == VARCHAR_TYPE
-		 || pF->getType() == BIGINT_TYPE
-		 || pF->getType() == DECIMAL_TYPE
-		 || pF->getType() == FIXED_TYPE )
-		idxPtr+=1;
-
 	    
-	    /*
-	    if ( pF->getLength() > vlen )
-	    {	       		
-		// mark end with zero byte
-		*idxPtr = 0;
-		idxPtr += pF->getLength() - vlen;
-	    }
-	    */
+	    memcpy(idxPtr, pF->getValue().getValue(), len);
+
+	    idxPtr += getReservedLength(pF);
+
+	    // treat all string stored values
+	    if ( isNullTerminated(pF->getType() ) )
+		idxPtr+=1;
+	    
+	    // terminate in any case
+	    *idxPtr=0;
 	}
 	else
 	{
 	    char nullIndicator = 0;
 	    memcpy(idxPtr, &nullIndicator, 1);
-	    idxPtr += pF->getLength() + 1;
-	    
-	    // cout << "LEN IS 0" << endl;
+	    idxPtr += getReservedLength(pF) + 1;
+
+	    if ( isNullTerminated(pF->getType() ) )
+		idxPtr+=1;	    
 	}
 	pF = pSchema->Next();
     }
@@ -149,63 +215,68 @@ void CegoBTreeValue::valueFromSchema(ListT<CegoField>* pSchema)
 
 void CegoBTreeValue::valueFromSchema(const ListT<CegoField> tableSchema, const ListT<CegoField> indexSchema)
 {
-    // get Index len
     int len = 0;
     CegoField* pIF = indexSchema.First();
     while (pIF)
-    {	
-	len += pIF->getValue().getLength();       	
+    {
+	len += getReservedLength(pIF);	    	
 	pIF = indexSchema.Next();
     }
 
     if ( len > TABMNG_MAXINDEXVALUE )
 	throw Exception(EXLOC, "Index value exceeded");
 
+    // check if local memory already allocated, do if not
+    if ( _idxArray == 0 )
+	_idxArray = (char*)malloc(TABMNG_MAXINDEXVALUE);
+    _pI = _idxArray;
+
     char* idxPtr = _pI;
     pIF = indexSchema.First();
     while (pIF)
     {
-
 	CegoField *pTF = tableSchema.First();
 
 	while ( pTF )
 	{
 	    if ( (Chain)pTF->getAttrName() == (Chain)pIF->getAttrName() )
 	    {
-
 		int len = pTF->getValue().getLength();
+		
 		if ( len > 0 )
 		{
-
+		    // checking reserved length + 1 because value length includes zero byte  
+		    if ( pTF->getValue().getLength() > getReservedLength(pTF) + 1 )  
+			throw Exception(EXLOC, "Reserved btree length for floating type exceeded");
+		    
 		    char nullIndicator = 1;
 		    memcpy(idxPtr, &nullIndicator, 1);
 		    idxPtr += 1;
 
 		    memcpy(idxPtr, pTF->getValue().getValue(), len);
-		    
-		    if ( pTF->getLength() > len )
-		    {		
-			idxPtr += len;
-			// mark end with zero byte
-			*idxPtr = 0;
-			idxPtr += pTF->getLength() - ( len + 1 );
-		    }
-		    else
+
+		    idxPtr += getReservedLength(pTF);
+
+		    // treat all string stored values
+		    if ( isNullTerminated(pTF->getType() ) )
 		    {
-			idxPtr += len;
+			idxPtr+=1;
 		    }
+		    // terminate in any case
+		    *idxPtr=0;		    
 		}
 		else
 		{
 		    char nullIndicator = 0;
 		    memcpy(idxPtr, &nullIndicator, 1);
-		    idxPtr += len + 1;
+		    idxPtr += getReservedLength(pTF) + 1;
+
+		    if ( isNullTerminated(pTF->getType() ) )
+		     	idxPtr+=1;
 		}
-	    }
-	    
+	    }   
 	    pTF = tableSchema.Next();
 	}
-
 	pIF = indexSchema.Next();
     }    
 }
@@ -223,17 +294,14 @@ ListT<CegoFieldValue> CegoBTreeValue::valueToFVL(const ListT<CegoField>& schema)
 	idxPtr += 1;
 
 	CegoFieldValue fv;
-
+	int len = getReservedLength(pF);
+	
 	if ( nullIndicator == 1 )
-	{
-	    int len = pF->getLength();
-
-	    if ( pF->getType() == VARCHAR_TYPE
-		 || pF->getType() == BIGINT_TYPE
-		 || pF->getType() == DECIMAL_TYPE
-		 || pF->getType() == FIXED_TYPE )
-	    {		
-		fv = CegoFieldValue(VARCHAR_TYPE, Chain(idxPtr));
+	{	    
+	    // treat all string stored values
+	    if ( isNullTerminated(pF->getType() ) )
+	    {
+		fv = CegoFieldValue(pF->getType(), Chain(idxPtr));		
 		idxPtr += len + 1;
 	    }
 	    else
@@ -242,6 +310,14 @@ ListT<CegoFieldValue> CegoBTreeValue::valueToFVL(const ListT<CegoField>& schema)
 		idxPtr+=len;
 	    }	    
 	}
+	else
+	{
+	    idxPtr += len + 1;
+	    if ( isNullTerminated(pF->getType() ) )
+	    {
+		idxPtr += 1;
+	    }
+	}
 	
 	fvl.Insert(fv);
 	pF = schema.Next();
@@ -249,9 +325,10 @@ ListT<CegoFieldValue> CegoBTreeValue::valueToFVL(const ListT<CegoField>& schema)
     return fvl;
 }
 
-void CegoBTreeValue::setPtr(char* p)
+void CegoBTreeValue::setPtr(char* p, int len)
 {
     _pI = p;
+    _len = len;
 }
 
 char* CegoBTreeValue::getPtr() const
@@ -261,8 +338,19 @@ char* CegoBTreeValue::getPtr() const
  
 CegoBTreeValue& CegoBTreeValue::operator = (const CegoBTreeValue& iv)
 {
-    _pI = iv._pI;
-    memcpy(_idxArray, iv._idxArray, TABMNG_MAXINDEXVALUE);
+    if ( iv._idxArray )
+    {
+	if ( _idxArray == 0 )
+	    _idxArray = (char*)malloc(TABMNG_MAXINDEXVALUE);	    
+	memcpy(_idxArray, iv._idxArray, _len);
+	_pI = _idxArray;
+    }
+    else
+    {
+	_pI = iv._pI;
+    }
+    _len = iv._len;
+	    
     return (*this);
 }
 
@@ -274,13 +362,7 @@ CegoBTreeValue::Comparison CegoBTreeValue::comp(const CegoBTreeValue& iv, ListT<
     CegoField* pF = pSchema->First();
     while (pF)
     {
-	int len = pF->getLength();
-
-	if ( pF->getType() == VARCHAR_TYPE
-	     || pF->getType() == BIGINT_TYPE
-	     || pF->getType() == DECIMAL_TYPE
-	     || pF->getType() == FIXED_TYPE )
-	    len++;
+	int len = getReservedLength(pF);
 	
 	CegoFieldValue fv1;
 	CegoFieldValue fv2;
@@ -298,12 +380,31 @@ CegoBTreeValue::Comparison CegoBTreeValue::comp(const CegoBTreeValue& iv, ListT<
 	idxPtr2 += 1;
 
 	if ( nullIndicator1 == 1 )
-	    fv1 = CegoFieldValue(pF->getType(), idxPtr1, len);
-	if ( nullIndicator2 == 1 )
-	    fv2 = CegoFieldValue(pF->getType(), idxPtr2, len);	    
-	
-	CegoFieldValue::Comparison c = fv1.comp(fv2);
+	{
+	    if ( isNullTerminated(pF->getType() ) )
+	    {
+		fv1 = CegoFieldValue(pF->getType(), Chain(idxPtr1));
+	    }
+	    else
+	    {
+		fv1 = CegoFieldValue(pF->getType(), idxPtr1, len);
+	    }
+	}
 
+	if ( nullIndicator2 == 1 )
+	{
+	    if ( isNullTerminated(pF->getType() ) )
+	    {
+		fv2 = CegoFieldValue(pF->getType(), Chain(idxPtr2));
+	    }
+	    else
+	    {
+		fv2 = CegoFieldValue(pF->getType(), idxPtr2, len);
+	    }
+	}
+
+	CegoFieldValue::Comparison c = fv1.comp(fv2);
+	
 	if ( c == CegoFieldValue::MORE )
 	{
 	    return MORE;
@@ -313,8 +414,13 @@ CegoBTreeValue::Comparison CegoBTreeValue::comp(const CegoBTreeValue& iv, ListT<
 	    return LESS;
 	}
 
+	if ( isNullTerminated(pF->getType() ) )
+	{
+	    len++;
+	}
+
 	idxPtr1+=len;
-	idxPtr2+=len;	    
+	idxPtr2+=len;
 	pF = pSchema->Next();
 		
     }
@@ -323,101 +429,21 @@ CegoBTreeValue::Comparison CegoBTreeValue::comp(const CegoBTreeValue& iv, ListT<
 }
 
 bool CegoBTreeValue::isHigher(const CegoBTreeValue& iv, ListT<CegoField>* pSchema) const
-{
-    char* idxPtr1 = _pI;
-    char* idxPtr2 = iv.getPtr();
-    
-    CegoField* pF = pSchema->First();
-    while (pF)
-    {
-	int len = pF->getLength();
+{   
+    Comparison c = comp(iv, pSchema);
+    return c == MORE;
+}
 
-	if ( pF->getType() == VARCHAR_TYPE
-	     || pF->getType() == BIGINT_TYPE
-	     || pF->getType() == DECIMAL_TYPE
-	     || pF->getType() == FIXED_TYPE )
-	    len++;
-
-	CegoFieldValue fv1;
-	CegoFieldValue fv2;
-
-	char nullIndicator1;
-	memcpy(&nullIndicator1, idxPtr1, 1);
-	idxPtr1 += 1;
-
-	char nullIndicator2;
-	memcpy(&nullIndicator2, idxPtr2, 1);
-	idxPtr2 += 1;
-
-	if ( nullIndicator1 == 1 )
-	     fv1 = CegoFieldValue(pF->getType(), idxPtr1, len);
-	if ( nullIndicator2 == 1 )
-	    fv2 = CegoFieldValue(pF->getType(), idxPtr2, len);	    
-
-	CegoFieldValue::Comparison c = fv1.comp(fv2);
-
-	if ( c == CegoFieldValue::MORE )
-	    return true;
-	else if ( c == CegoFieldValue::LESS )
-	    return false;
-
-	idxPtr1+=len;
-	idxPtr2+=len;	    
-	pF = pSchema->Next();		
-    }
-    return false;
+bool CegoBTreeValue::isEqualHigher(const CegoBTreeValue& iv, ListT<CegoField>* pSchema) const
+{   
+    Comparison c = comp(iv, pSchema);
+    return c == MORE || c == EQUAL;
 }
 
 bool CegoBTreeValue::isEqual(const CegoBTreeValue& iv, ListT<CegoField>* pSchema) const
-{    
-    char* idxPtr1 = _pI;
-    char* idxPtr2 = iv.getPtr();
-    
-    CegoField* pF = pSchema->First();
-    while (pF)
-    {
-	int len = pF->getLength();
-
-	if ( pF->getType() == VARCHAR_TYPE
-	     || pF->getType() == BIGINT_TYPE
-	     || pF->getType() == DECIMAL_TYPE
-	     || pF->getType() == FIXED_TYPE )
-	    len++;
-	
-	CegoFieldValue fv1;
-	CegoFieldValue fv2;
-	
-	char nullIndicator1;
-	memcpy(&nullIndicator1, idxPtr1, 1);
-	idxPtr1 += 1;
-
-	char nullIndicator2;
-	memcpy(&nullIndicator2, idxPtr2, 1);
-	idxPtr2 += 1;
-
-	if ( nullIndicator1 == 1 )
-	    fv1 = CegoFieldValue(pF->getType(), idxPtr1, len);
-	if ( nullIndicator2 == 1 )
-	    fv2 = CegoFieldValue(pF->getType(), idxPtr2, len);	    
-	
-	CegoFieldValue::Comparison c = fv1.comp(fv2);
-
-	if ( c == CegoFieldValue::MORE )
-	    return false;
-	else if ( c == CegoFieldValue::LESS )
-	    return false;
-
-	idxPtr1+=len;
-	idxPtr2+=len;	    
-	pF = pSchema->Next();
-		
-    }
-    return true;
-}
-
-bool CegoBTreeValue::operator > ( const CegoBTreeValue& iv) const
 {
-    return false;
+    Comparison c = comp(iv, pSchema);
+    return c == EQUAL;
 }
 
 Chain CegoBTreeValue::toChain(ListT<CegoField>* pSchema) const
@@ -434,16 +460,12 @@ Chain CegoBTreeValue::toChain(ListT<CegoField>* pSchema) const
 
 	CegoFieldValue fv;
 
-	int len = pF->getLength();
+	int len = getReservedLength(pF);
 	if ( nullIndicator == 1 )
 	{
-
-	    if ( pF->getType() == VARCHAR_TYPE
-		 || pF->getType() == BIGINT_TYPE
-		 || pF->getType() == DECIMAL_TYPE
-		 || pF->getType() == FIXED_TYPE )
+	    if ( isNullTerminated(pF->getType() ) )
 	    {
-		fv = CegoFieldValue(VARCHAR_TYPE, Chain(idxPtr));
+		fv = CegoFieldValue(pF->getType(), Chain(idxPtr));
 		idxPtr += len + 1;
 	    }
 	    else
@@ -453,8 +475,12 @@ Chain CegoBTreeValue::toChain(ListT<CegoField>* pSchema) const
 	    }
 	}
 	else
-	{
+	{	   
 	    idxPtr+=len;
+	    if ( isNullTerminated(pF->getType() ) )
+	    {
+		idxPtr+=1;
+	    }
 	}
 	
 	s += fv.valAsChain();
@@ -465,3 +491,30 @@ Chain CegoBTreeValue::toChain(ListT<CegoField>* pSchema) const
     
     return s;
 }
+
+int CegoBTreeValue::getReservedLength(CegoField* pF)
+{
+    switch ( pF->getType() )
+    {
+    case DECIMAL_TYPE:
+	return pF->getLength();
+    case BIGINT_TYPE:
+    case VARCHAR_TYPE:
+	return pF->getLength();
+    case INT_TYPE:
+    case LONG_TYPE:
+    case BOOL_TYPE:
+    case DATETIME_TYPE:	
+    case FLOAT_TYPE:
+    case DOUBLE_TYPE:
+    case SMALLINT_TYPE:
+    case TINYINT_TYPE:
+	return pF->getLength();
+    default:
+    // case BLOB_TYPE:	
+    // case CLOB_TYPE:
+    // case NULL_TYPE:
+    // case PAGEID_TYPE:
+	throw Exception(EXLOC, Chain("Data type not supported"));
+    }
+}
