@@ -8,7 +8,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -23,12 +23,14 @@
 // LFC INCLUDES
 #include <lfcbase/Chain.h>
 
+#include "CegoDefs.h"
+
 class CegoBufferPoolEntry {
 
 public:
   
     CegoBufferPoolEntry();  
-    CegoBufferPoolEntry(int seg, int pos, const Chain& occState, bool isDirty, int numFixes, int tabSetId, int fileId, int pageId, int fixStat, unsigned long long numUsage);
+    CegoBufferPoolEntry(int seg, int pos, const Chain& occState, bool isDirty, int numFixes, int tabSetId, PageIdType pageId, unsigned fixStat, unsigned long long numUsage);
     ~CegoBufferPoolEntry();
 
     int getSegment() const;
@@ -37,9 +39,8 @@ public:
     bool isDirty() const;
     int getNumFixes() const;
     int getTabSetId() const;
-    int getFileId() const;
-    int getPageId() const;
-    int getFixStat() const;
+    PageIdType getPageId() const;
+    unsigned getFixStat() const;
     unsigned long long getNumUsage() const;
 
     CegoBufferPoolEntry& operator = ( const CegoBufferPoolEntry& b);
@@ -56,9 +57,8 @@ private:
     bool _isDirty;
     int _numFixes;
     int _tabSetId;
-    int _fileId;
-    int _pageId;
-    int _fixStat;
+    PageIdType _pageId;
+    unsigned _fixStat;
     unsigned long long _numUsage;
 };
 #endif
