@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -46,15 +46,22 @@ bool CegoCheckpoint::checkpointReached(const Chain& tableSet, int interval)
     CheckpointEntry* pCPE;
     if ( ( pCPE = _tsList.Find(CheckpointEntry(tableSet))) != 0 )
     {
-	if ( pCPE->getNextCP() < ts.asInt() )
+	if ( pCPE->getNextCP() < ts.asLong() )
 	{
-	    pCPE->setNextCP(ts.asInt() + interval);
+	    // if checkpoint has reached, we set new interval and return true
+	    pCPE->setNextCP(ts.asLong() + interval);
 	    return true;
 	}
+	else if ( interval < (int)(pCPE->getNextCP() - ts.asLong()) )
+	{
+	    // if interval was reduced, we just set new interval
+	    pCPE->setNextCP(ts.asLong() + interval);
+	    return false;
+	}	   
     }
     else
     {
-	_tsList.Insert(CheckpointEntry(tableSet, ts.asInt() + interval));
+	_tsList.Insert(CheckpointEntry(tableSet, ts.asLong() + interval));
     }
     return false;
 }
