@@ -8,7 +8,7 @@
 //        
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -41,11 +41,11 @@ public:
 
 	CheckpointEntry() { };
 	CheckpointEntry(const Chain& ts) { _ts = ts; };
-	CheckpointEntry(const Chain& ts, int nextCP) { _ts = ts; _nextCP = nextCP; };
+	CheckpointEntry(const Chain& ts, unsigned long long nextCP) { _ts = ts; _nextCP = nextCP; };
 	~CheckpointEntry() {};
 	       
-	int getNextCP() { return _nextCP; };
-	void setNextCP(int nextCP) { _nextCP = nextCP; };
+	unsigned long long getNextCP() { return _nextCP; };
+	void setNextCP(unsigned long long nextCP) { _nextCP = nextCP; };
 
 	CheckpointEntry& operator = ( const CheckpointEntry& cp) { 
 	    _ts = cp._ts;
@@ -59,10 +59,9 @@ public:
 	
     private:
 	Chain _ts;
-	int _nextCP;
+	unsigned long long _nextCP;
     };
 
     ListT<CheckpointEntry> _tsList;
-
 };
 #endif
