@@ -338,6 +338,7 @@ unsigned long modId;
 CegoDbHandler::ProtocolType protType;
 Chain histPath;
 Chain pgHistPath;
+File *pBatchInput = 0;
 
 int main(int argc, char **argv)
 {    
@@ -407,9 +408,6 @@ int main(int argc, char **argv)
 
     if ( longOpt.isSet("nohead") )
 	noHead=true;
-
-    if ( longOpt.isSet("pager") )
-	runMode = PAGER;
     
     if ( longOpt.isSet("ignore") )
 	ignoreError=true;
@@ -439,17 +437,48 @@ int main(int argc, char **argv)
     }
 
     logFile = longOpt.getOptValue("logfile");
-
+    
     if ( longOpt.isSet("batchfile") )
     {
 	runMode = BATCH;
 	batchFileName = longOpt.getOptValue("batchfile");
+
+	pBatchInput = new File(batchFileName);
+	if ( pBatchInput->exists() == false )
+	{
+	    delete pBatchInput;
+	    pBatchInput=0;
+	    Chain msg = Chain("File ") + batchFileName + Chain(" does not exist");		    
+	    throw Exception(EXLOC, msg);
+	}
+	pBatchInput->open(File::READ);
+
     }
-    if ( longOpt.isSet("dumpfile") )
+    else if ( longOpt.isSet("dumpfile") )
     {
 	runMode = DUMP;
 	dumpFileName = longOpt.getOptValue("dumpfile");
     }
+    else if ( longOpt.isSet("pager") )
+    {
+	runMode = PAGER;
+    }	
+    else
+    {
+	// check for data from stdin, if available, set batch mode
+	
+	pBatchInput = new File(File::IN);
+	if ( pBatchInput->hasData(PIPE_TIMEOUT) )
+	{
+	    runMode = BATCH;	
+	}
+	else
+	{
+	    delete pBatchInput;
+	    pBatchInput = 0;
+	}
+    }
+
 
     if ( longOpt.isSet("profile") )
     {
@@ -589,18 +618,6 @@ int execute()
     Chain logMode("notice");
     if ( debug )
 	logMode=Chain("debug");
-
-    File *pInput = 0;
-    pInput = new File(File::IN);
-    if ( pInput->hasData(PIPE_TIMEOUT) )
-    {
-	runMode = BATCH;	
-    }
-    else
-    {
-	delete pInput;
-	pInput = 0;
-    }
     
     switch ( runMode )
     {	
@@ -612,15 +629,10 @@ int execute()
 	
 	try
 	{	    
-	    if ( pInput == 0 )
+	    if ( pBatchInput == 0 )
 	    {
-		pInput = new File(batchFileName);
-		if ( pInput->exists() == false )
-		{
-		    Chain msg = Chain("File ") + batchFileName + Chain(" does not exist");		    
-		    throw Exception(EXLOC, msg);
-		}
-		pInput->open(File::READ);
+		Chain msg = Chain("No data available in batch mode ");
+		throw Exception(EXLOC, msg);
 	    }
 
 	    pCegoNet->connect(serverName, portNo, tableSet, user, password);
@@ -634,7 +646,7 @@ int execute()
 	    bool disableDelimiter=false;
 	    bool goOn = true;
 	    
-	    while (goOn && pInput->readLine(line, maxLine))
+	    while (goOn && pBatchInput->readLine(line, maxLine))
 	    {
 		lineNo++;
 		if ( line.truncRight(" \t") == Chain("@") )
@@ -705,12 +717,10 @@ int execute()
 	    {
 		cout << "Incomplete command ..<" << cmd << ">" << endl;
 	    }
-
-	    if ( runMode == BATCH )
-	    {	    
-		pInput->close();
-	    }
-	    delete pInput;
+	    
+	    pBatchInput->close();
+	    delete pBatchInput;
+	    pBatchInput=0;
 	    
 	    pCegoNet->disconnect();
 
@@ -721,8 +731,8 @@ int execute()
 	    Chain msg;
 	    e.pop(msg);
 
-	    if ( pInput )
-		delete pInput;
+	    if ( pBatchInput )
+		delete pBatchInput;
 	    
 	    cerr << msg << endl;
 	    delete pCegoNet;
