@@ -144,7 +144,11 @@ int CegoContentObject::getBaseContentSize() const
     // object tabname size
     baseContentSize += 1; 
     baseContentSize += _tabName.length();
-    
+
+    // object tabalias size
+    baseContentSize += 1; 
+    baseContentSize += _tabAlias.length();
+
     // schema size
     baseContentSize += sizeof(int);
     baseContentSize += getSchemaSize();
@@ -168,6 +172,13 @@ void CegoContentObject::encodeBaseContent(char *buf, int size) const
     memcpy(bufPtr, (char*)_tabName, _tabName.length());
     bufPtr=bufPtr + _tabName.length();
 
+    c = (char)_tabAlias.length();
+    memcpy (bufPtr, &c , 1);
+    bufPtr++;
+    
+    memcpy(bufPtr, (char*)_tabAlias, _tabAlias.length());
+    bufPtr=bufPtr + _tabAlias.length();
+
     int schemaSize = getSchemaSize();
     memcpy (bufPtr, &schemaSize, sizeof(int));
     bufPtr += sizeof(int);
@@ -235,12 +246,15 @@ void CegoContentObject::decodeBaseContent(char *buf, int& size)
     CegoObject::decodeBase(bufPtr, size);
     bufPtr += CegoObject::getBaseSize();
 
+    // skip tablename length information
     bufPtr++;
     _tabName = Chain(bufPtr);    
     bufPtr=bufPtr + _tabName.length();
 
-    // setting alias to tabname on default
-    _tabAlias = _tabName;
+    // skip tablealias length information
+    bufPtr++;
+    _tabAlias = Chain(bufPtr);    
+    bufPtr=bufPtr + _tabAlias.length();
     
     int schemaSize;
 
@@ -298,8 +312,8 @@ void CegoContentObject::decodeBaseContent(char *buf, int& size)
 	attrName = Chain(bufPtr);
 	
 	bufPtr=bufPtr + attrName.length();
-	
-	_schema.Insert( CegoField( _tabName, _tabName, attrName, type, len, dim, defValue, isNullable, id));
+
+	_schema.Insert( CegoField( _tabName, _tabAlias, attrName, type, len, dim, defValue, isNullable, id));
     }
 }
 
