@@ -6,13 +6,13 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
 // Class: CegoDataPointer
 //
-// Description: The CegoDataPointer class represents the location of a data entry in a data file.
+// Description: The CegoDataPointer class represents the location of a data entry in any data file.
 //              The class provides also encoding and decoding methods to store the pointer information
 //              into the database. 
 //
@@ -29,85 +29,78 @@
 
 CegoDataPointer::CegoDataPointer()
 {
-    _idArray[0] = 0;
-    _idArray[1] = 0;
-    _idArray[2] = 0;
+    _pageId = 0;
+    _offset = 0;
 }
 
-CegoDataPointer::CegoDataPointer(int fileId, int pageId, int offset)
+CegoDataPointer::CegoDataPointer(PageIdType pageId, int offset)
 {
-    _idArray[0] = fileId;
-    _idArray[1] = pageId;
-    _idArray[2] = offset;
+    _pageId = pageId;
+    _offset = offset;
 }
 
 CegoDataPointer::~CegoDataPointer()
 {
 }
 
-void CegoDataPointer::setFileId(int fileId)
+int CegoDataPointer::getEncodingLength()
 {
-    _idArray[0] = fileId;
+    return ( sizeof(PageIdType) + sizeof(int));
 }
 
-int CegoDataPointer::getFileId() const
+void CegoDataPointer::setPageId(PageIdType pageId)
 {
-    return _idArray[0];
+    _pageId = pageId;
 }
 
-void CegoDataPointer::setPageId(int pageId)
+PageIdType CegoDataPointer::getPageId() const
 {
-    _idArray[1] = pageId;
-}
-
-int CegoDataPointer::getPageId() const
-{
-    return _idArray[1];
+    return _pageId;
 }
  
 void CegoDataPointer::setOffset(int offset)
 {
-    _idArray[2] = offset;
+    _offset = offset;
 }
 
 int CegoDataPointer::getOffset() const
 {
-    return _idArray[2];
-}
-
-int CegoDataPointer::getEncodingLength() const
-{
-    return ( 3 * sizeof(int) ) ;
+    return _offset;
 }
 
 void CegoDataPointer::encode(void* p) const
 {
-    void* ip = p;
-    memcpy(ip, _idArray, 3 * sizeof(int));    
+    char* pI = (char*)p;
+    memcpy(pI, &_pageId, sizeof(PageIdType));
+    pI+=sizeof(PageIdType);
+    memcpy(pI, &_offset, sizeof(int));
 }
 
 void CegoDataPointer::decode(void* p)
 {
-    void* ip = p;
-    memcpy(_idArray, ip, 3 * sizeof(int));
+    char* pI = (char*)p;
+    memcpy(&_pageId, pI, sizeof(PageIdType));
+    pI+=sizeof(PageIdType);
+    memcpy(&_offset, pI, sizeof(int));
 }
 
 CegoDataPointer& CegoDataPointer::operator = ( const CegoDataPointer& d)
 {
-    memcpy(_idArray, d._idArray, 3 * sizeof(int));
+    _pageId = d._pageId;
+    _offset = d._offset;
     return (*this);
 }
 
 bool CegoDataPointer::operator == ( const CegoDataPointer& d) const
 {
-    if (_idArray[0] == d._idArray[0] && _idArray[1] == d._idArray[1] && _idArray[2] == d._idArray[2])
+    if ( _pageId == d._pageId && _offset == d._offset )
 	return true;
     return false;
 }
 
 bool CegoDataPointer::operator != ( const CegoDataPointer& d) const
 {
-    if (_idArray[0] != d._idArray[0] || _idArray[1] != d._idArray[1] || _idArray[2] != d._idArray[2])
+    if ( _pageId != d._pageId || _offset != d._offset )
 	return true;
     return false;
 }
@@ -115,12 +108,12 @@ bool CegoDataPointer::operator != ( const CegoDataPointer& d) const
 Chain CegoDataPointer::toChain() const
 {
     Chain s;
-    s = "[" + Chain(getFileId()) + "," + Chain(getPageId()) + "," + Chain(getOffset()) + "]";
+    s = "[" + Chain(getPageId()) + "," + Chain(getOffset()) + "]";
     return s;
 }
 
 ostream& operator << (ostream& s, const CegoDataPointer& dp)
 {
-    s << "[" << dp.getFileId() << "," << dp.getPageId() << "," << dp.getOffset() << "]";
+    s << "[" << dp.getPageId() << "," << dp.getOffset() << "]";
     return s;
 }
