@@ -8,13 +8,13 @@
 //        
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
 // Class: CegoDataPointer
 // 
-// Description: The CegoDataPointer class represents the location of a data entry in a data file.
+// Description: The CegoDataPointer class represents the location of a data entry in any data file.
 //              The class provides also encoding and decoding methods to store the pointer information
 //              into the database. 
 //
@@ -25,25 +25,25 @@
 // LFC INCLUDES
 #include <lfcbase/Chain.h>
 
+// CEGO INCLUDES
+#include "CegoDefs.h"
+
 class CegoDataPointer {
 
 public:
    
     CegoDataPointer();
-    CegoDataPointer(int fileId, int pageId, int offset);
+    CegoDataPointer(PageIdType pageId, int offset);
     ~CegoDataPointer();
 
-    int getFileId() const;
-    void setFileId(int fileId);
+    static int getEncodingLength();
 
-    int getPageId() const;
-    void setPageId(int pageId);
+    PageIdType getPageId() const;
+    void setPageId(PageIdType pageId);
 
     int getOffset() const;
     void setOffset(int offset);
 
-    int getEncodingLength() const;
-
     void encode(void* p) const;
     void decode(void* p);
 
@@ -56,6 +56,7 @@ public:
 
 private:
 
-    int _idArray[3];
+    PageIdType _pageId;
+    int _offset;
 };
 #endif
