@@ -6,7 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -100,13 +100,20 @@ CegoDbHandler::CegoDbHandler(NetHandler *pN, CegoDbHandler::ProtocolType pt, Ceg
     
     if ( _protType == CegoDbHandler::XML )
     {
+	_protTypeName = Chain("XML");
 	Document *pDoc = new Document(XML_CEGO);
 	pDoc->setAttribute(XML_VERSION_ATTR, XML_VERSION_VALUE);    
 	_xml.setDocument(pDoc);
     }
-    else
+    else if ( _protType == CegoDbHandler::SERIAL )
     {
-	_pSer = new CegoSerial(_pN);
+	_protTypeName = Chain("SERIAL");
+	_pSer = new CegoSerial(_pN, false);
+    }
+    else // if ( _protType == CegoDbHandler::FASTSERIAL )
+    {
+	_protTypeName = Chain("FASTSERIAL");
+	_pSer = new CegoSerial(_pN, true);
     }
   
     _wasReset = false;
@@ -134,9 +141,13 @@ CegoDbHandler::ResultType CegoDbHandler::requestSession(const Chain& tableSet, c
     _pModule->log(_modId, Logger::DEBUG, Chain("Request session for user ") + user + Chain("/") + password);
 #endif
 
+    // client settings
+    _user= user;
+    _password = password;
+    _tableSet = tableSet;
+    
     if ( _protType == CegoDbHandler::XML )
     {
-
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	
 	pRoot->setAttribute(XML_TABLESET_ATTR, tableSet);   
@@ -156,8 +167,7 @@ CegoDbHandler::ResultType CegoDbHandler::requestSession(const Chain& tableSet, c
 	return sendXMLReq(XML_DBSESSION_REQUEST, pRoot);
     }
     else
-    {
-	
+    {	
 	_tableSet = tableSet;
 	
 	Chain pwd;
@@ -202,8 +212,8 @@ bool CegoDbHandler::acceptSession()
     _pModule->log(_modId, Logger::DEBUG, Chain("Accepting session"));
 #endif
 
-    try {
-
+    try
+    {
 	if ( _protType == CegoDbHandler::XML )
 	{
 	    _xml.getDocument()->clear();
@@ -227,8 +237,7 @@ bool CegoDbHandler::acceptSession()
 #endif
 	    
 	    if ( docType != Chain(XML_DBSESSION_REQUEST) )
-	    {
-	    
+	    {	    
 		_pModule->log(_modId, Logger::LOGERR, Chain("Invalid request"));
 		
 		_xml.getDocument()->clear();
@@ -257,8 +266,7 @@ bool CegoDbHandler::acceptSession()
 		    _user = pRoot->getAttributeValue(XML_USER_ATTR);
 		    _password = pRoot->getAttributeValue(XML_PASSWD_ATTR);
 		    
-		    // response is sent after authorization check
-		    
+		    // response is sent after authorization check	    
 		    return true;
 		}
 		else
@@ -270,9 +278,12 @@ bool CegoDbHandler::acceptSession()
 	else
 	{
 #ifdef CGDEBUG
-	    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
-	    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
-	    _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	    if ( _protType == CegoDbHandler::SERIAL )
+	    {	    		
+		_pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
+		_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
+		_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	    }
 #endif
 	    _pSer->reset();
 	    Chain req = _pSer->readChain();
@@ -290,7 +301,8 @@ bool CegoDbHandler::acceptSession()
 	    else
 	    {
 		_tableSet = _pSer->readChain();
-		_user = _pSer->readChain();
+
+		_user = _pSer->readChain();		
 		_password = _pSer->readChain();
 		// cout << "SESSION OK" << endl;
 		return true;
@@ -299,7 +311,6 @@ bool CegoDbHandler::acceptSession()
     }
     catch ( Exception e)
     {
-	
 	Chain msg;
 	e.pop(msg);
 	
@@ -415,21 +426,31 @@ unsigned long long CegoDbHandler::getTid() const
     return _serTid;
 }
 
-const Chain& CegoDbHandler::getDbName()
+const Chain& CegoDbHandler::getProtocol() const
+{
+    return _protTypeName;
+}
+
+const Chain& CegoDbHandler::getDbName() const
 {
     return _serDbName;
 }
 
-const Chain& CegoDbHandler::getDbVersion()
+const Chain& CegoDbHandler::getDbVersion() const
 {
     return _serDbVersion;
 }
 
-const Chain& CegoDbHandler::getDateFormat()
+const Chain& CegoDbHandler::getDateFormat() const
 {
     return _serDateFormat;
 }
 
+char CegoDbHandler::getQuoteEscapeFlag() const
+{
+    return _serQuoteEscapeFlag;
+}
+
 const Chain& CegoDbHandler::getTableSet()
 {
     return _tableSet;
@@ -477,6 +498,8 @@ const ListT<CegoField>& CegoDbHandler::getSchema()
 		Chain colName = (*pRow)->getAttributeValue(XML_COLNAME_ATTR);
 		Chain colType = (*pRow)->getAttributeValue(XML_COLTYPE_ATTR);
 		Chain colSize = (*pRow)->getAttributeValue(XML_COLSIZE_ATTR);
+		Chain colDim = (*pRow)->getAttributeValue(XML_COLDIM_ATTR);
+		
 		Chain colNullable = (*pRow)->getAttributeValue(XML_COLNULLABLE_ATTR);
 		Chain colDefValue = (*pRow)->getAttributeValue(XML_COLDEFVALUE_ATTR);
 		
@@ -490,11 +513,11 @@ const ListT<CegoField>& CegoDbHandler::getSchema()
 		    isNullable=false;
 		
 		CegoFieldValue defValue;
-		if ( colDefValue != Chain("") )
+		if ( colDefValue != Chain() )
 		{
 		    defValue = CegoFieldValue(dataType, colDefValue);
 		}
-		CegoField f(CegoField(colTable, colTable, colName, dataType, colSize.asInteger(), defValue, isNullable));	
+		CegoField f(CegoField(colTable, colTable, colName, dataType, colSize.asInteger(), colDim.asInteger(), defValue, isNullable));	
 		
 		_serSchema.Insert(f);
 		
@@ -575,14 +598,14 @@ void CegoDbHandler::getProcResult(ListT<CegoProcVar>& outParamList, CegoFieldVal
 		CegoTypeConverter tc;
 		CegoDataType datatype = tc.getTypeId(paramType);
 		
-		if ( paramName == Chain("") )
+		if ( paramName == Chain() )
 		{
 		    retValue = CegoFieldValue(datatype, paramValue);
 		}
 		else
 		{
 		    CegoFieldValue fv(datatype, paramValue);
-		    outParamList.Insert( CegoProcVar(paramName, CegoProcVar::OUTVAR, fv.getType(), fv.getLength(), fv));
+		    outParamList.Insert( CegoProcVar(paramName, CegoProcVar::OUTVAR, fv.getType(), fv.getLength(), fv.getDim(), fv));
 		}
 		
 		pOP = outParamElementList.Next();
@@ -661,8 +684,7 @@ CegoDbHandler::ResultType CegoDbHandler::receiveTableData(const ListT<CegoField>
 		fvl.Insert(fv);	    
 	    }
 	    else
-	    {
-		
+	    {		
 		Chain colVal = (*_pRow)->getAttributeValue(colPos);
 			   
 		if ( pF->getType() == INT_TYPE  )
@@ -722,11 +744,6 @@ CegoDbHandler::ResultType CegoDbHandler::receiveTableData(const ListT<CegoField>
 		    CegoFieldValue fv(DECIMAL_TYPE, colVal);	    
 		    fvl.Insert(fv);
 		}
-		else if ( pF->getType() == FIXED_TYPE )
-		{
-		    CegoFieldValue fv(FIXED_TYPE, colVal);	    
-		    fvl.Insert(fv);
-		}
 		else if ( pF->getType() == BLOB_TYPE )
 		{
 		    CegoFieldValue fv(BLOB_TYPE, colVal);	    
@@ -741,11 +758,9 @@ CegoDbHandler::ResultType CegoDbHandler::receiveTableData(const ListT<CegoField>
     }
     else
     {
-	// cout << "Num Ahead = " << _pSer->numAhead() << endl;
-
 	if ( _pSer->numAhead() > 0 )
 	{
-	    fvl = _pSer->readRow(schema);
+	    _pSer->readRow(schema, fvl);
 	    return DB_DATA;
 	}
 	else
@@ -754,7 +769,7 @@ CegoDbHandler::ResultType CegoDbHandler::receiveTableData(const ListT<CegoField>
 	    switch ( res )
 	    {
 	    case DB_DATA:
-		fvl = _pSer->readRow(schema);
+		_pSer->readRow(schema, fvl);
 		break;
 	    case DB_OK:
 	    case DB_ERROR:
@@ -780,7 +795,6 @@ void CegoDbHandler::abortQuery()
 
     if ( _protType == CegoDbHandler::XML )
     {
-
 	_xml.getDocument()->clear();
 	_xml.setChain( _pN->getMsg() );
 	
@@ -857,10 +871,8 @@ void CegoDbHandler::sendCollectedData()
     else if ( c == QUERY_MOREDATA )
     {
 	if ( _protType == CegoDbHandler::XML )
-	{
-	    
-	    Chain xmlString;
-	    
+	{	    
+	    Chain xmlString;	    
 	    _xml.getDocument()->setDocType(XML_DATA_DOC);
 	    _xml.getXMLChain(xmlString);
 	    
@@ -878,13 +890,14 @@ void CegoDbHandler::sendCollectedData()
 	}
 	else
 	{
-	    
 #ifdef CGDEBUG
-	    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
-	    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
-	    _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	    if ( _protType == CegoDbHandler::SERIAL )
+	    {
+		_pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
+		_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
+		_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	    }
 #endif
-
 	    _pN->writeMsg();
 	    _pSer->reset();
 	    _serSync = true;
@@ -913,7 +926,7 @@ bool CegoDbHandler::wasReset()
 
 void CegoDbHandler::sendFinishData()
 {
-    char c;
+    char c = 0;
 
     if ( _protType == CegoDbHandler::XML )
     {
@@ -943,7 +956,6 @@ void CegoDbHandler::sendFinishData()
     
     if ( _protType == CegoDbHandler::XML )
     {
-	
 	_xml.getDocument()->clear();
 	_xml.getDocument()->setDocType(XML_OK_DOC);
 	
@@ -958,20 +970,21 @@ void CegoDbHandler::sendFinishData()
 	_pModule->log(_modId, Logger::DEBUG, xmlString);
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
 #endif
-	_pN->setMsg(xmlString, xmlString.length());
-	
+	_pN->setMsg(xmlString, xmlString.length());	
     }
     else
     {	
 	_pSer->reset();
 	_pSer->writeChain(SER_FIN);
-	
+		
 #ifdef CGDEBUG
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
-	_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	if ( _protType == CegoDbHandler::SERIAL )
+	{	    
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
+	    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	}
 #endif
-
     }
 
     _pN->writeMsg();
@@ -979,7 +992,7 @@ void CegoDbHandler::sendFinishData()
 
 void CegoDbHandler::sendErrorData(const Chain& msg)
 {
-    char c;
+    char c = 0;
 
     if ( _protType == CegoDbHandler::XML )
     {
@@ -1021,8 +1034,7 @@ void CegoDbHandler::sendErrorData(const Chain& msg)
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- XML ---"));
 	_pModule->log(_modId, Logger::DEBUG, xmlString);
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
-#endif
-	
+#endif	
 	_pN->setMsg(xmlString, xmlString.length());
     }
     else
@@ -1032,13 +1044,14 @@ void CegoDbHandler::sendErrorData(const Chain& msg)
 	_pSer->writeChain(msg);
 	
 #ifdef CGDEBUG
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
-	_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	if ( _protType == CegoDbHandler::SERIAL )
+	{	    
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
+	    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	}
 #endif
-
     }
-    
     _pN->writeMsg();
 }
 
@@ -1046,7 +1059,8 @@ void CegoDbHandler::sendSessionConfirm(const Chain& msg,
 				       unsigned long long tid, 
 				       const Chain& dbName, 
 				       const Chain& dbVersion,
-				       const Chain& dateTimeFormat)
+				       const Chain& dateTimeFormat,
+				       char quoteEscapeFlag)
 {    
     if ( _protType == CegoDbHandler::XML )
     {
@@ -1059,6 +1073,11 @@ void CegoDbHandler::sendSessionConfirm(const Chain& msg,
 	pRoot->setAttribute(XML_DBPRODNAME_ATTR, dbName);
 	pRoot->setAttribute(XML_DBPRODVERSION_ATTR, dbVersion);
 	pRoot->setAttribute(XML_DATETIMEFORMAT_ATTR, dateTimeFormat);
+
+	if ( quoteEscapeFlag )
+	    pRoot->setAttribute(XML_QESCMODE_ATTR, XML_ON_VALUE);
+	else
+	    pRoot->setAttribute(XML_QESCMODE_ATTR, XML_OFF_VALUE);
 	
 	_xml.getDocument()->setRootElement(pRoot);
 	
@@ -1083,13 +1102,19 @@ void CegoDbHandler::sendSessionConfirm(const Chain& msg,
 	_pSer->writeChain(dbName);
 	_pSer->writeChain(dbVersion);
 	_pSer->writeChain(dateTimeFormat);
-	
+	if ( quoteEscapeFlag )
+	    _pSer->writeChain(Chain("Y"));
+	else
+	    _pSer->writeChain(Chain("N"));
+			      
 #ifdef CGDEBUG
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
-	_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
-#endif
-
+	if ( _protType == CegoDbHandler::SERIAL )
+	{	    
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
+	    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	}
+#endif	
     }
     
     _pN->writeMsg();
@@ -1127,11 +1152,13 @@ void CegoDbHandler::sendResponse(const Chain& msg, unsigned long long affected)
 	_pSer->writeChain(Chain(affected));
 
 #ifdef CGDEBUG
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
-	_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
-#endif
-	
+	if ( _protType == CegoDbHandler::SERIAL )
+	{	    
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
+	    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	}
+#endif	
     }
     _pN->writeMsg();
 }
@@ -1157,8 +1184,7 @@ void CegoDbHandler::sendError(const Chain& msg)
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
 #endif
 	
-	_pN->setMsg(xmlString, xmlString.length());
-	
+	_pN->setMsg(xmlString, xmlString.length());	
     }
     else
     {
@@ -1167,16 +1193,18 @@ void CegoDbHandler::sendError(const Chain& msg)
 	_pSer->writeChain(msg);
 
 #ifdef CGDEBUG
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
-	_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	if ( _protType == CegoDbHandler::SERIAL )
+	{	    	    
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
+	    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	}
 #endif
-	
     }
     _pN->writeMsg();
 }
 
-void CegoDbHandler::sendBlobInfo(int fileId, int pageId)
+void CegoDbHandler::sendBlobInfo(PageIdType pageId)
 {
     if ( _protType == CegoDbHandler::XML )
     {
@@ -1184,7 +1212,6 @@ void CegoDbHandler::sendBlobInfo(int fileId, int pageId)
 	_xml.getDocument()->setDocType(XML_OK_DOC);
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
-	pRoot->setAttribute(XML_FILEID_ATTR, Chain(fileId));
 	pRoot->setAttribute(XML_PAGEID_ATTR, Chain(pageId));
 	
 	_xml.getDocument()->setRootElement(pRoot);
@@ -1196,15 +1223,13 @@ void CegoDbHandler::sendBlobInfo(int fileId, int pageId)
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- XML ---"));
 	_pModule->log(_modId, Logger::DEBUG, xmlString);
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
-#endif
-	
+#endif	
 	_pN->setMsg(xmlString, xmlString.length());
     }
     else
     {
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_BLOBINFO));
-	_pSer->writeChain(Chain(fileId));
 	_pSer->writeChain(Chain(pageId));
     }
     _pN->writeMsg();
@@ -1214,7 +1239,6 @@ void CegoDbHandler::sendBlobSize(unsigned long long blobSize)
 {
     if ( _protType == CegoDbHandler::XML )
     {
-
 	_xml.getDocument()->clear();
 	_xml.getDocument()->setDocType(XML_OK_DOC);
 	
@@ -1230,10 +1254,8 @@ void CegoDbHandler::sendBlobSize(unsigned long long blobSize)
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- XML ---"));
 	_pModule->log(_modId, Logger::DEBUG, xmlString);
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
-#endif
-	
-	_pN->setMsg(xmlString, xmlString.length());
-	
+#endif	
+	_pN->setMsg(xmlString, xmlString.length());	
     }
     else
     {
@@ -1291,8 +1313,7 @@ CegoDbHandler::ResultType CegoDbHandler::putBlob(CegoBlob& blob)
 	pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
-	    blob.setFileId(pRoot->getAttributeValue(XML_FILEID_ATTR).asInteger());
-	    blob.setPageId(pRoot->getAttributeValue(XML_PAGEID_ATTR).asInteger());
+	    blob.setPageId(pRoot->getAttributeValue(XML_PAGEID_ATTR).asUnsignedLongLong());
 	}
     }
     else
@@ -1305,8 +1326,7 @@ CegoDbHandler::ResultType CegoDbHandler::putBlob(CegoBlob& blob)
 	    return DB_ERROR;
 	}
 
-	blob.setFileId(_pSer->readChain().asInteger());
-	blob.setPageId(_pSer->readChain().asInteger());
+	blob.setPageId(_pSer->readChain().asUnsignedLongLong());
     }
 	
     blob.reset();
@@ -1334,7 +1354,6 @@ CegoDbHandler::ResultType CegoDbHandler::getBlob(CegoBlob& blob)
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, _tableSet);
-	pRoot->setAttribute(XML_FILEID_ATTR, blob.getFileId());
 	pRoot->setAttribute(XML_PAGEID_ATTR, blob.getPageId());
 	
 	_xml.getDocument()->setRootElement(pRoot);
@@ -1350,14 +1369,13 @@ CegoDbHandler::ResultType CegoDbHandler::getBlob(CegoBlob& blob)
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_GETBLOB));
 	_pSer->writeChain(Chain(_tableSet));
-	_pSer->writeChain(Chain(blob.getFileId()));
 	_pSer->writeChain(Chain(blob.getPageId()));	
     }
     _pN->writeMsg();
     
     _pN->readMsg();
 
-    unsigned long long blobSize;
+    unsigned long long blobSize = 0;
 
     if ( _protType == CegoDbHandler::XML )
     {
@@ -1378,6 +1396,10 @@ CegoDbHandler::ResultType CegoDbHandler::getBlob(CegoBlob& blob)
 	{
 	    blobSize = pRoot->getAttributeValue(XML_SIZE_ATTR).asUnsignedLongLong();
 	}
+	else
+	{
+	    throw Exception(EXLOC, Chain("Cannot get blob size"));
+	}
     }
     else
     {
@@ -1390,7 +1412,6 @@ CegoDbHandler::ResultType CegoDbHandler::getBlob(CegoBlob& blob)
 	}
 
 	blobSize = _pSer->readChain().asInteger();
-
     }
 
     blob.allocate(blobSize);
@@ -1419,7 +1440,6 @@ CegoDbHandler::ResultType CegoDbHandler::delBlob(CegoBlob& blob)
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, _tableSet);
-	pRoot->setAttribute(XML_FILEID_ATTR, blob.getFileId());
 	pRoot->setAttribute(XML_PAGEID_ATTR, blob.getPageId());
 	
 	_xml.getDocument()->setRootElement(pRoot);
@@ -1435,9 +1455,7 @@ CegoDbHandler::ResultType CegoDbHandler::delBlob(CegoBlob& blob)
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_DELBLOB));
 	_pSer->writeChain(Chain(_tableSet));
-	_pSer->writeChain(Chain(blob.getFileId()));
 	_pSer->writeChain(Chain(blob.getPageId()));
-	
     }
     _pN->writeMsg();
     
@@ -1466,7 +1484,6 @@ CegoDbHandler::ResultType CegoDbHandler::delBlob(CegoBlob& blob)
     }
     else
     {
-
 	_pSer->reset();
 	Chain req = _pSer->readChain();
 
@@ -1478,7 +1495,7 @@ CegoDbHandler::ResultType CegoDbHandler::delBlob(CegoBlob& blob)
     }
 }
 
-void CegoDbHandler::sendClobInfo(int fileId, int pageId)
+void CegoDbHandler::sendClobInfo(PageIdType pageId)
 {
     if ( _protType == CegoDbHandler::XML )
     {
@@ -1486,7 +1503,6 @@ void CegoDbHandler::sendClobInfo(int fileId, int pageId)
 	_xml.getDocument()->setDocType(XML_OK_DOC);
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
-	pRoot->setAttribute(XML_FILEID_ATTR, Chain(fileId));
 	pRoot->setAttribute(XML_PAGEID_ATTR, Chain(pageId));
 	
 	_xml.getDocument()->setRootElement(pRoot);
@@ -1499,14 +1515,12 @@ void CegoDbHandler::sendClobInfo(int fileId, int pageId)
 	_pModule->log(_modId, Logger::DEBUG, xmlString);
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
 #endif
-	
 	_pN->setMsg(xmlString, xmlString.length());
     }
     else
     {
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_CLOBINFO));
-	_pSer->writeChain(Chain(fileId));
 	_pSer->writeChain(Chain(pageId));	
     }
     _pN->writeMsg();
@@ -1531,8 +1545,7 @@ void CegoDbHandler::sendClobSize(unsigned long long clobSize)
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- XML ---"));
 	_pModule->log(_modId, Logger::DEBUG, xmlString);
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
-#endif
-	
+#endif	
 	_pN->setMsg(xmlString, xmlString.length());	
     }
     else
@@ -1567,8 +1580,7 @@ CegoDbHandler::ResultType CegoDbHandler::putClob(CegoClob& clob)
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_PUTCLOB));
 	_pSer->writeChain(Chain(_tableSet));
-	_pSer->writeChain(Chain(clob.getSize()));
-	
+	_pSer->writeChain(Chain(clob.getSize()));	
     }
 
     _pN->writeMsg();
@@ -1592,8 +1604,7 @@ CegoDbHandler::ResultType CegoDbHandler::putClob(CegoClob& clob)
 	pRoot = _xml.getDocument()->getRootElement();
 	if ( pRoot )
 	{
-	    clob.setFileId(pRoot->getAttributeValue(XML_FILEID_ATTR).asInteger());
-	    clob.setPageId(pRoot->getAttributeValue(XML_PAGEID_ATTR).asInteger());
+	    clob.setPageId(pRoot->getAttributeValue(XML_PAGEID_ATTR).asUnsignedLongLong());
 	}
     }
     else
@@ -1606,8 +1617,7 @@ CegoDbHandler::ResultType CegoDbHandler::putClob(CegoClob& clob)
 	    return DB_ERROR;
 	}
 
-	clob.setFileId(_pSer->readChain().asInteger());
-	clob.setPageId(_pSer->readChain().asInteger());
+	clob.setPageId(_pSer->readChain().asUnsignedLongLong());
     }
 	
     clob.reset();
@@ -1629,14 +1639,12 @@ CegoDbHandler::ResultType CegoDbHandler::putClob(CegoClob& clob)
 
 CegoDbHandler::ResultType CegoDbHandler::getClob(CegoClob& clob)
 {
-
     if ( _protType == CegoDbHandler::XML )
     {
 	_xml.getDocument()->clear();
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, _tableSet);
-	pRoot->setAttribute(XML_FILEID_ATTR, clob.getFileId());
 	pRoot->setAttribute(XML_PAGEID_ATTR, clob.getPageId());
 	
 	_xml.getDocument()->setRootElement(pRoot);
@@ -1652,14 +1660,13 @@ CegoDbHandler::ResultType CegoDbHandler::getClob(CegoClob& clob)
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_GETCLOB));
 	_pSer->writeChain(Chain(_tableSet));
-	_pSer->writeChain(Chain(clob.getFileId()));
 	_pSer->writeChain(Chain(clob.getPageId()));	
     }
     _pN->writeMsg();
     
     _pN->readMsg();
 
-    unsigned long long clobSize;
+    unsigned long long clobSize = 0;
 
     if ( _protType == CegoDbHandler::XML )
     {
@@ -1680,6 +1687,10 @@ CegoDbHandler::ResultType CegoDbHandler::getClob(CegoClob& clob)
 	{
 	    clobSize = pRoot->getAttributeValue(XML_SIZE_ATTR).asUnsignedLongLong();
 	}
+	else
+	{
+	    throw Exception(EXLOC, Chain("Cannot get clob size"));
+	}
     }
     else
     {
@@ -1718,7 +1729,6 @@ CegoDbHandler::ResultType CegoDbHandler::delClob(CegoClob& clob)
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
 	pRoot->setAttribute(XML_TABLESET_ATTR, _tableSet);
-	pRoot->setAttribute(XML_FILEID_ATTR, clob.getFileId());
 	pRoot->setAttribute(XML_PAGEID_ATTR, clob.getPageId());
 	
 	_xml.getDocument()->setRootElement(pRoot);
@@ -1734,7 +1744,6 @@ CegoDbHandler::ResultType CegoDbHandler::delClob(CegoClob& clob)
 	_pSer->reset();
 	_pSer->writeChain(Chain(SER_DELCLOB));
 	_pSer->writeChain(Chain(_tableSet));
-	_pSer->writeChain(Chain(clob.getFileId()));
 	_pSer->writeChain(Chain(clob.getPageId()));	
     }
     _pN->writeMsg();
@@ -1826,7 +1835,6 @@ void CegoDbHandler::sendProcResult(const Chain& msg, const ListT<CegoProcVar>& o
 	_pModule->log(_modId, Logger::DEBUG, xmlString);
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
 #endif
-	
 	_pN->setMsg(xmlString, xmlString.length());
     }
     else
@@ -1854,12 +1862,14 @@ void CegoDbHandler::sendProcResult(const Chain& msg, const ListT<CegoProcVar>& o
 	}
 
 #ifdef CGDEBUG
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
-	_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	if ( _protType == CegoDbHandler::SERIAL )
+	{	    	    
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
+	    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	}
 #endif
     }
-    
     _pN->writeMsg();
 }
 
@@ -1951,13 +1961,14 @@ CegoDbHandler::ResultType CegoDbHandler::getMoreTableData()
     }
     else
     {
-
 #ifdef CGDEBUG
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
-	_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	if ( _protType == CegoDbHandler::SERIAL )
+	{	    	    
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
+	    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	}
 #endif
-
 	_pSer->reset();
 	Chain req = _pSer->readChain();
 
@@ -2002,13 +2013,11 @@ CegoDbHandler::RequestType CegoDbHandler::acceptRequest()
 
     if ( _protType == CegoDbHandler::XML )
     {
-
 #ifdef CGDEBUG
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- XML ---"));
 	_pModule->log(_modId, Logger::DEBUG, _pN->getMsg());
 	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
-#endif
-		
+#endif		
 	_xml.getDocument()->clear();
 	_xml.setChain( _pN->getMsg() );
 	_xml.parse();
@@ -2152,11 +2161,13 @@ CegoDbHandler::RequestType CegoDbHandler::acceptRequest()
     {
 
 #ifdef CGDEBUG
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
-	_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	if ( _protType == CegoDbHandler::SERIAL )
+	{	    	    
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
+	    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	}
 #endif
-
 	_pSer->reset();
 	Chain req = _pSer->readChain();
 	
@@ -2257,8 +2268,7 @@ CegoDbHandler::RequestType CegoDbHandler::acceptRequest()
 	else if (  req == Chain(SER_GETBLOB) ) 
 	{
 	    _serTableSet = _pSer->readChain();
-	    _serFileId = _pSer->readChain().asInteger();
-	    _serPageId = _pSer->readChain().asInteger();
+	    _serPageId = (PageIdType)_pSer->readChain().asUnsignedLongLong();
 	    return CegoDbHandler::GETBLOB;
 	}
 	else if (  req == Chain(SER_DELBLOB) ) 
@@ -2274,8 +2284,7 @@ CegoDbHandler::RequestType CegoDbHandler::acceptRequest()
 	else if (  req == Chain(SER_GETCLOB) ) 
 	{
 	    _serTableSet = _pSer->readChain();
-	    _serFileId = _pSer->readChain().asInteger();
-	    _serPageId = _pSer->readChain().asInteger();
+	    _serPageId = (PageIdType)_pSer->readChain().asUnsignedLongLong();
 	    return CegoDbHandler::GETCLOB;
 	}
 	else if (  req == Chain(SER_DELCLOB) ) 
@@ -2309,7 +2318,7 @@ CegoDbHandler::RequestType CegoDbHandler::acceptRequest()
     }
 }
 
-Chain CegoDbHandler::getQueryArg()
+const Chain& CegoDbHandler::getQueryArg()
 {
     if ( _protType == CegoDbHandler::XML )
     {	    
@@ -2323,7 +2332,6 @@ Chain CegoDbHandler::getQueryArg()
     return _serQueryCmd;    
 }
 
-
 void CegoDbHandler::getPutBlobArg(Chain& tableSet, unsigned long long& blobSize)
 {
     if ( _protType == CegoDbHandler::XML )
@@ -2342,7 +2350,7 @@ void CegoDbHandler::getPutBlobArg(Chain& tableSet, unsigned long long& blobSize)
     }
 }
 
-void CegoDbHandler::getGetBlobArg(Chain& tableSet, int& fileId, int& pageId)
+void CegoDbHandler::getGetBlobArg(Chain& tableSet, PageIdType& pageId)
 {
     if ( _protType == CegoDbHandler::XML )
     {	    
@@ -2350,14 +2358,12 @@ void CegoDbHandler::getGetBlobArg(Chain& tableSet, int& fileId, int& pageId)
 	if ( pRoot )
 	{
 	    tableSet = pRoot->getAttributeValue(XML_TABLESET_ATTR);
-	    fileId = pRoot->getAttributeValue(XML_FILEID_ATTR).asInteger();
-	    pageId = pRoot->getAttributeValue(XML_PAGEID_ATTR).asInteger();
+	    pageId = (PageIdType)pRoot->getAttributeValue(XML_PAGEID_ATTR).asUnsignedLongLong();
 	}
     }
     else
     {
 	tableSet = _serTableSet;
-	fileId = _serFileId;
 	pageId = _serPageId;
     }
 }
@@ -2380,7 +2386,7 @@ void CegoDbHandler::getPutClobArg(Chain& tableSet, unsigned long long& clobSize)
     }
 }
 
-void CegoDbHandler::getGetClobArg(Chain& tableSet, int& fileId, int& pageId)
+void CegoDbHandler::getGetClobArg(Chain& tableSet, PageIdType& pageId)
 {
     if ( _protType == CegoDbHandler::XML )
     {	    
@@ -2388,14 +2394,12 @@ void CegoDbHandler::getGetClobArg(Chain& tableSet, int& fileId, int& pageId)
 	if ( pRoot )
 	{
 	    tableSet = pRoot->getAttributeValue(XML_TABLESET_ATTR);
-	    fileId = pRoot->getAttributeValue(XML_FILEID_ATTR).asInteger();
-	    pageId = pRoot->getAttributeValue(XML_PAGEID_ATTR).asInteger();
+	    pageId = pRoot->getAttributeValue(XML_PAGEID_ATTR).asUnsignedLongLong();
 	}
     }
     else
     {
 	tableSet = _serTableSet;
-	fileId = _serFileId;
 	pageId = _serPageId;
     }
 }
@@ -2471,7 +2475,7 @@ void CegoDbHandler::sendObjInfo(CegoDecodableObject& oe)
 	_xml.getDocument()->clear();
 	
 	Element* pRoot = new Element(XML_FRAME_ELEMENT);
-	pRoot->addContent(oe.getElement());
+	// pRoot->addContent(oe.getElement());
 	
 	_xml.getDocument()->setRootElement(pRoot);
 	_xml.getDocument()->setDocType(XML_INFO_DOC);
@@ -2497,9 +2501,12 @@ void CegoDbHandler::sendObjInfo(CegoDecodableObject& oe)
 	_pSer->writeObject(oe);
 	
 #ifdef CGDEBUG
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
-	_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
-	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	if ( _protType == CegoDbHandler::SERIAL )
+	{	    	    
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
+	    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
+	    _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+	}
 #endif
 	
 	_pN->writeMsg();
@@ -2523,8 +2530,7 @@ void CegoDbHandler::collectData(const ListT<CegoField>& schema)
 	int col=1;
 	CegoField *pF = schema.First();
 	while ( pF )
-	{
-	    
+	{	    
 	    Chain colPos = Chain(NETMNG_COLPREFIX) + Chain(col);
 	    
 	    if ( ! pF->getValue().isNull() )
@@ -2586,6 +2592,19 @@ void CegoDbHandler::collectData(const ListT<CegoField>& schema, const ListT<Cego
     }
 }
 
+int CegoDbHandler::numCollected() const
+{
+    if ( _protType == CegoDbHandler::XML )
+    {
+	// not supported for XML protocol
+	return 0;
+    }
+    else
+    {
+	return _pSer->numWritten();
+    }	
+}
+
 NetHandler* CegoDbHandler::getNetHandler()
 {
     return _pN;
@@ -2633,15 +2652,13 @@ CegoDbHandler::ResultType CegoDbHandler::sendXMLReq(const Chain& reqType, Elemen
     {
 	docType = Chain("INVALID");
     }
-    
-    
+
     if ( docType == Chain(XML_OK_DOC) )
     {
 	return DB_OK;
     }
     else if ( docType == Chain(XML_SACK_DOC) )
     {
-
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	
 	if ( pRoot )
@@ -2651,12 +2668,12 @@ CegoDbHandler::ResultType CegoDbHandler::sendXMLReq(const Chain& reqType, Elemen
 	    _serDbName = pRoot->getAttributeValue(XML_DBPRODNAME_ATTR);
 	    _serDbVersion = pRoot->getAttributeValue(XML_DBPRODVERSION_ATTR);
 	    _serDateFormat = pRoot->getAttributeValue(XML_DATETIMEFORMAT_ATTR);
+	    _serQuoteEscapeFlag = ( pRoot->getAttributeValue(XML_QESCMODE_ATTR) == Chain(XML_ON_VALUE) ); 	    
 	}
 	return DB_OK;	    
     }
     else if ( docType == Chain(XML_ERROR_DOC) )
     {
-
 	Element *pRoot = _xml.getDocument()->getRootElement();
 	
 	if ( pRoot )
@@ -2679,13 +2696,16 @@ CegoDbHandler::ResultType CegoDbHandler::sendXMLReq(const Chain& reqType, Elemen
 
 CegoDbHandler::ResultType CegoDbHandler::sendSerialReq()
 {
-
 #ifdef CGDEBUG
-    _pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
-    _pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
-    _pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+    if ( _protType == CegoDbHandler::SERIAL )
+    {	    	
+	_pModule->log(_modId, Logger::DEBUG, Chain("--- SER ---"));
+	_pModule->log(_modId, Logger::DEBUG, Chain(_pN->getMsg(), _pN->getMsgSize()));
+	_pModule->log(_modId, Logger::DEBUG, Chain("--- --- ---"));
+    }
 #endif
 
+    // cout << "Writing req " << Chain(_pN->getMsg(), _pN->getMsgSize()) << endl;
     _pN->writeMsg();
     
     _pN->readMsg();
@@ -2693,6 +2713,8 @@ CegoDbHandler::ResultType CegoDbHandler::sendSerialReq()
     _pSer->reset();
     Chain req = _pSer->readChain();
 
+    // cout << "Reading req " << req << endl;
+    
     if ( req == Chain(SER_OK))
     {
 	_serMsg = _pSer->readChain();
@@ -2712,11 +2734,14 @@ CegoDbHandler::ResultType CegoDbHandler::sendSerialReq()
 	_serDbName = _pSer->readChain();
 	_serDbVersion = _pSer->readChain();
 	_serDateFormat = _pSer->readChain();
+
+	Chain qescmode = _pSer->readChain();
+	_serQuoteEscapeFlag = ( qescmode == Chain("Y") );
+	
 	return DB_OK;
     }
     else if ( req == Chain(SER_PROCRES))
     {
-
 	_serMsg = Chain("Procedure executed");	
 
 	while ( _pSer->numAhead() > 0 )
@@ -2735,7 +2760,7 @@ CegoDbHandler::ResultType CegoDbHandler::sendSerialReq()
 	    else
 	    {
 		CegoFieldValue fv(datatype, paramValue);
-		_outParamList.Insert( CegoProcVar(paramName, CegoProcVar::OUTVAR, fv.getType(), fv.getLength(), fv));
+		_outParamList.Insert( CegoProcVar(paramName, CegoProcVar::OUTVAR, fv.getType(), fv.getLength(), fv.getDim(), fv));
 	    }
 	}
 	return DB_OK;
