@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -38,7 +38,7 @@ class CegoDbHandler {
     
 public:
 
-    enum ProtocolType { XML, SERIAL };
+    enum ProtocolType { XML, SERIAL, FASTSERIAL };
 
     enum RequestType { QUERY,
 		       QUERYABORT,
@@ -77,7 +77,6 @@ public:
 
     enum ResultType { DB_OK, DB_ERROR, DB_DATA, DB_INFO, DB_FIN };
 
-
     CegoDbHandler(NetHandler *pN, ProtocolType pt, CegoModule *pModule);
     ~CegoDbHandler();
 
@@ -96,10 +95,11 @@ public:
     const Chain& getMsg();
     unsigned long long getTid() const;
 
-    const Chain& getDbName();
-    const Chain& getDbVersion();
-    const Chain& getDateFormat();
-    
+    const Chain& getProtocol() const;
+    const Chain& getDbName() const;
+    const Chain& getDbVersion() const;
+    const Chain& getDateFormat() const;
+    char getQuoteEscapeFlag() const;    
     
     unsigned long long getAffected();
     const ListT<CegoField>& getSchema();
@@ -119,12 +119,14 @@ public:
     bool acceptSession();
     RequestType acceptRequest();
     
-    Chain getQueryArg();
+    const Chain& getQueryArg();
 
     void collectSchema(const ListT<CegoField>& schema, const Chain& format = Chain(""));
     void collectData(const ListT<CegoField>& schema);
     void collectData(const ListT<CegoField>& schema, const ListT<CegoFieldValue>& fvl);
 
+    int numCollected() const;
+    
     void sendCollectedData();
     void sendFinishData();
     void sendErrorData(const Chain& msg);
@@ -135,12 +137,13 @@ public:
 			    unsigned long long tid, 
 			    const Chain& dbName, 
 			    const Chain& dbVersion,
-			    const Chain& dateTimeFormat);
+			    const Chain& dateTimeFormat,
+			    char quoteEscapeFlag);
 	
     void sendResponse(const Chain& msg, unsigned long long affected = 0 );
     void sendError(const Chain& msg );
 
-    void sendBlobInfo(int fileId, int pageId);
+    void sendBlobInfo(PageIdType pageId);
     void sendBlobSize(unsigned long long blobSize);
 
     ResultType putBlob(CegoBlob& blob);
@@ -148,9 +151,9 @@ public:
     ResultType delBlob(CegoBlob& blob);
 
     void getPutBlobArg(Chain& tableSet, unsigned long long& blobSize);
-    void getGetBlobArg(Chain& tableSet, int& fileId, int& pageId);
+    void getGetBlobArg(Chain& tableSet, PageIdType& pageId);
 
-    void sendClobInfo(int fileId, int pageId);
+    void sendClobInfo(PageIdType pageId);
     void sendClobSize(unsigned long long blobSize);
 
     ResultType putClob(CegoClob& clob);
@@ -158,8 +161,7 @@ public:
     ResultType delClob(CegoClob& clob);
     
     void getPutClobArg(Chain& tableset, unsigned long long& clobSize);
-    void getGetClobArg(Chain& tableset, int& fileId, int& pageId);
-
+    void getGetClobArg(Chain& tableset, PageIdType& pageId);
     
     void sendProcResult(const Chain& msg, const ListT<CegoProcVar>& outParamList, CegoFieldValue *pRetVal);
 
@@ -172,6 +174,7 @@ public:
 protected:
 
     ProtocolType _protType;
+    Chain _protTypeName;
 
     // xml request
     ResultType sendXMLReq(const Chain& reqType, Element* pRoot);
@@ -188,7 +191,6 @@ protected:
     Chain _user;
     Chain _password;
 
-
     // for xml protocol
     XMLSuite _xml;
     Element **_pRow;
@@ -205,13 +207,14 @@ protected:
     Chain _serTableSet;
     unsigned long long _serBlobSize;
     unsigned long long _serClobSize;
-    int _serFileId;
-    int _serPageId;
+
+    PageIdType _serPageId;
 
     bool _serSync;
     Chain _serDbName;
     Chain _serDbVersion;
     Chain _serDateFormat;
+    char _serQuoteEscapeFlag;
     
     ListT<CegoField> _serSchema;
 
