@@ -114,6 +114,11 @@ ListT<CegoDbThread::QueryEntry> CegoDbThread::getCostQuery() const
     return costList;
 }
 
+Chain CegoDbThread::lastAction() const
+{
+    return _lastAction;
+}
+
 unsigned long long CegoDbThread::allocatedSortArea() const
 {
     if ( _pTabMng )
@@ -467,13 +472,23 @@ void CegoDbThread::serveSession(CegoDistDbHandler *pSH)
 
 bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestType reqType)
 {
+
+    Chain tsState = _pDBMng->getTableSetRunState(pSH->getTableSet());
+
+    if ( tsState != Chain(XML_ONLINE_VALUE)
+	 && tsState != Chain(XML_BACKUP_VALUE)
+	 && tsState != Chain(XML_CHECKPOINT_VALUE) )	
+    {
+	Chain msg = Chain("Tableset ") + pSH->getTableSet() + Chain(" not online ( State is ") + tsState + Chain(" )");
+	pSH->sendError(msg);
+	return false;
+    }
+        
     bool isTerminated=false;   	       
 
     // at the beginning of the request, in any case we reset abort status
     _pTabMng->proceed();
 
-    Chain lastAction;
-
     _pCost->reset();
     _pCost->start();
     
@@ -497,14 +512,14 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
 	case CegoDbHandler::QUERY:
 	{  
-	    lastAction = pSH->getQueryArg(); 
+	    _lastAction = pSH->getQueryArg(); 
 				
 #ifdef CGDEBUG
-	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Query Arg = <<<") + lastAction + Chain(">>>"));
+	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(" : Query Arg = <<<") + _lastAction + Chain(">>>"));
 #endif
 	    
 	    _pPA->cleanUp();
-	    _pPA->setCommandChain( lastAction );
+	    _pPA->setCommandChain( _lastAction );
 	    _pPA->setDbHandle(pSH);
 	    
 	    _pPA->parse();
@@ -940,7 +955,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
 	case CegoDbHandler::PUTBLOB:
 	{
-	    lastAction = Chain("put blob");
+	    _lastAction = Chain("put blob");
 
 	    Chain tableSet;
 	    unsigned long long blobSize;
@@ -1076,7 +1091,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
 	case CegoDbHandler::GETBLOB:
 	{
-	    lastAction = Chain("get blob");
+	    _lastAction = Chain("get blob");
 
 	    Chain tableSet;
 
@@ -1165,7 +1180,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
 	case CegoDbHandler::DELBLOB:
 	{
-	    lastAction = Chain("del blob");
+	    _lastAction = Chain("del blob");
 
 	    Chain tableSet;
 
@@ -1182,7 +1197,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
 	case CegoDbHandler::PUTCLOB:
 	{
-	    lastAction = Chain("put clob");
+	    _lastAction = Chain("put clob");
 
 	    Chain tableSet;
 	    unsigned long long clobSize;
@@ -1316,7 +1331,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
 	case CegoDbHandler::GETCLOB:
 	{
-	    lastAction = Chain("get clob");
+	    _lastAction = Chain("get clob");
 
 	    Chain tableSet;
 	    
@@ -1405,7 +1420,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
 	case CegoDbHandler::DELCLOB:
 	{
-	    lastAction = Chain("del clob");
+	    _lastAction = Chain("del clob");
 
 	    Chain tableSet;
 	    PageIdType pageId;
@@ -1418,7 +1433,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
 	case CegoDbHandler::STARTTRANSACTION:
 	{
-	    lastAction = Chain("start transaction");
+	    _lastAction = Chain("start transaction");
 	    
 	    Chain tableSet;
 	    pSH->getArgValue(XML_TABLESET_ATTR, tableSet);
@@ -1429,7 +1444,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
 	case CegoDbHandler::COMMITTRANSACTION:
 	{
-	    lastAction = Chain("commit transaction");
+	    _lastAction = Chain("commit transaction");
 	    
 	    Chain tableSet;
 	    pSH->getArgValue(XML_TABLESET_ATTR, tableSet);
@@ -1441,7 +1456,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
 	case CegoDbHandler::ROLLBACKTRANSACTION:
 	{
-	    lastAction = Chain("rollback transaction");
+	    _lastAction = Chain("rollback transaction");
 	    
 	    Chain tableSet;
 	    pSH->getArgValue(XML_TABLESET_ATTR, tableSet);
@@ -1452,7 +1467,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
 	case CegoDbHandler::GETTID:
 	{
-	    lastAction = Chain("get tid");
+	    _lastAction = Chain("get tid");
 	    
 	    Chain tableSet;
 	    pSH->getArgValue(XML_TABLESET_ATTR, tableSet);
@@ -1462,7 +1477,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 	}
 	case CegoDbHandler::SESSION_CLOSE:
 	{
-	    lastAction = Chain("close session");
+	    _lastAction = Chain("close session");
 	    
 	    isTerminated=true;
 	    pSH->sendResponse(Chain("Session Closed"));
@@ -1502,7 +1517,7 @@ bool CegoDbThread::serveRequest(CegoDistDbHandler *pSH, CegoDbHandler::RequestTy
 
     Datetime ts;
     
-    addHistory(ts.asLong(), lastAction, _pCost->getSum() / 1000000 );
+    addHistory(ts.asLong(), _lastAction, _pCost->getSum() / 1000000 );
     
     return isTerminated;
 }
