@@ -226,7 +226,8 @@ public:
 
     ListT<QueryEntry> getLastQuery() const;
     ListT<QueryEntry> getCostQuery() const;
-    
+
+    Chain lastAction() const;
     unsigned long long allocatedSortArea() const;
     void* job(void* arg);
 
@@ -277,7 +278,8 @@ private:
 
     int _numLast;
     int _numCost;
-    
+
+    Chain _lastAction;
     unsigned long _modId;
 };
 #endif
