@@ -3,12 +3,12 @@
 ///////////////////////////////////////////////////////////////////////////////
 //                                                         
 // CegoDistDbHandler.h
-// ---------------------
+// -------------------
 // Cego distributed db handler class definition
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -27,7 +27,7 @@
 // CEGO INCLUDES
 #include "CegoAlterDesc.h"
 #include "CegoDbHandler.h"
-#include "CegoPredDesc.h"
+// #include "CegoPredicate.h"
 #include "CegoObject.h"
 #include "CegoTableObject.h"
 #include "CegoKeyObject.h"
@@ -43,6 +43,7 @@ public:
 
     ResultType reqInsertOp(const Chain& tableSet, const Chain& tableName, const ListT<CegoField>& fl);    
 
+    /*
     ResultType reqUpdateOp(const Chain& tableSet, 
 			   const Chain& tableName,
 			   const ListT<CegoField>& updSchema, 
@@ -50,10 +51,12 @@ public:
 			   CegoPredDesc* pPred);
 
     ResultType reqDeleteOp(const Chain& tableSet, const Chain& tableName, CegoPredDesc* pPred);
+    */
+    
     ResultType reqCreateTableOp(const Chain& tableSet, const Chain& tableName, CegoObject::ObjectType type, const ListT<CegoField>& fl, const ListT<CegoField>& idxList);
 
 
-    ResultType reqAlterTableOp(const Chain& tableSet, const Chain& tableName, const ListT<CegoAlterDesc>& alterList);
+    // ResultType reqAlterTableOp(const Chain& tableSet, const Chain& tableName, const ListT<CegoAlterDesc>& alterList);
     
     ResultType reqCreateIndexOp(const Chain& tableSet, const Chain& indexName, const Chain& tableName, CegoObject::ObjectType type, const ListT<CegoField>& idxList);
     ResultType reqCreateFKeyOp(int tabSetId, const Chain& fkey, const Chain& tableName, const ListT<CegoField>& keyList, const Chain& refTable, const ListT<CegoField>& refList);
@@ -65,6 +68,8 @@ public:
     ResultType reqCreateViewOp(const Chain& tableSet, const Chain& viewName, const ListT<CegoField>& fl, const Chain& viewText);
     ResultType reqCreateProcOp(const Chain& tableSet, const Chain& procName, const Chain& procText);
 
+    ResultType reqCreateTriggerOp(const Chain& tableSet, const Chain& triggerName, const Chain& tableName, const Chain& triggerText);
+
     ResultType reqTableDataOp(int tabSetId, const Chain& tableName, CegoObject::ObjectType type);
 
     ResultType reqGetObjectListOp(int tabSetId, CegoObject::ObjectType type);
@@ -78,7 +83,7 @@ public:
     ResultType reqReorgObjectOp(const Chain& tableSet, const Chain& tableName, CegoObject::ObjectType type);
 
 
-    ResultType reqCreateCheckOp(const Chain& tableSet, const Chain& checkName, const Chain& tableName, CegoPredDesc* pPred);
+    // ResultType reqCreateCheckOp(const Chain& tableSet, const Chain& checkName, const Chain& tableName, CegoPredDesc* pPred);
 
     ResultType reqStartTransactionOp(const Chain& tableSet);
     ResultType reqCommitTransactionOp(const Chain& tableSet);
@@ -88,15 +93,21 @@ public:
     void getArgValue(const Chain& arg, Chain& value);
 
     void getInsertArg(Chain& tableSet, Chain& tableName, ListT<CegoField>& fl);
-    void getUpdateArg(Chain& tableSet, Chain& tableName, ListT<CegoField>& fl, ListT<CegoExpr*>& exprList, CegoPredDesc*& pPred, CegoDistManager* pGTM);
-    void getDeleteArg(Chain& tableSet, Chain& tableName, CegoPredDesc*& pPred, CegoDistManager* pGTM);
+
+    
+    // deprecated
+    // void getDeleteArg(Chain& tableSet, Chain& tableName, CegoPredDesc*& pPred, CegoDistManager* pGTM);
+    // void getUpdateArg(Chain& tableSet, Chain& tableName, ListT<CegoField>& fl, ListT<CegoExpr*>& exprList, CegoPredDesc*& pPred, CegoDistManager* pGTM);
+    
     void getCreateTableArg(Chain& tableSet, Chain& tableName, ListT<CegoField>& fl, ListT<CegoField>& idxList);
 
     void getCreateViewArg(Chain& tableSet, Chain& viewName, ListT<CegoField>& fl, Chain& viewText);
     void getCreateProcedureArg(Chain& tableSet, Chain& procName, Chain& procText);
-    void getCreateCheckArg(Chain& tableSet, Chain& checkName, Chain& tableName, CegoPredDesc*& pPred, CegoDistManager* pGTM);
 
-    void getAlterTableArg(Chain& tableSet, Chain& tableName, ListT<CegoAlterDesc>& alterList);
+
+    // deprecated
+    // void getCreateCheckArg(Chain& tableSet, Chain& checkName, Chain& tableName, CegoPredDesc*& pPred, CegoDistManager* pGTM);
+    // void getAlterTableArg(Chain& tableSet, Chain& tableName, ListT<CegoAlterDesc>& alterList);
 
     void getDropTableArg(Chain& tableSet, Chain& tableName, CegoObject::ObjectType& type);
     void getCreateIndexArg(Chain& tableSet, Chain& indexName, Chain& tableName, ListT<CegoField>& idxList, CegoObject::ObjectType& type);
@@ -124,7 +135,6 @@ public:
     void getObjectByTableList(ListT<CegoTableObject>& idxList, ListT<CegoKeyObject>& keyList, ListT<CegoCheckObject>& checkList);
     void getPageCount(int& pageCount);    
 
-
 private:
 
     unsigned long _modId;
