@@ -6,7 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -38,6 +38,7 @@ CegoField::CegoField(const CegoField& f)
     _attrName = f._attrName;
     _type = f._type;
     _len = f._len;
+    _dim = f._dim;
     _id = f._id;
     _fv = (CegoFieldValue)f._fv;
 }
@@ -46,16 +47,20 @@ CegoField::CegoField(const Chain& tableName, const Chain& attrName)
 {
     _tableName = tableName;
     _attrName = attrName;
+    _type = NULL_TYPE;
+    _len = 0;
+    _dim = 0;
     _id = 0;
 }
 
-CegoField::CegoField(const Chain& tableName, const Chain& tableAlias, const Chain& attrName, CegoDataType type, int len, const CegoFieldValue& defVal, bool isNullable, int id)
+CegoField::CegoField(const Chain& tableName, const Chain& tableAlias, const Chain& attrName, CegoDataType type, int len, int dim, const CegoFieldValue& defVal, bool isNullable, int id)
 {
     _tableName = tableName;
     _tableAlias = tableAlias;
     _attrName = attrName;
     _type = type;
     _len = len;
+    _dim = dim;
     _isNullable = isNullable;
     _id = id;
     _fv = defVal;
@@ -125,7 +130,7 @@ void CegoField::setType(const CegoDataType t)
     _type = t;
 }
 
-const CegoDataType CegoField::getType() const
+const CegoDataType& CegoField::getType() const
 {
     return _type;
 }
@@ -140,6 +145,16 @@ int CegoField::getLength() const
     return _len;
 }
 
+void CegoField::setDim(int dim)
+{
+    _dim = dim;
+}
+
+int CegoField::getDim() const
+{
+    return _dim;
+}
+
 const CegoFieldValue& CegoField::getValue() const
 {
     return _fv;
@@ -150,9 +165,22 @@ void CegoField::setValue(const CegoFieldValue& fv)
     _fv = (CegoFieldValue)fv;
 }
 
+void CegoField::setupNull()
+{
+    _fv.setNull();
+}
+
+void CegoField::setupValue(CegoDataType type, void* pValue, int len)
+{
+    _fv.setType(type);
+    _fv.setValue(pValue);
+    _fv.setLength(len);		      
+    _fv.setLocalCopy(false);
+}
+
 int CegoField::size() const
 {
-    return _tableName.length() + _tableAlias.length() + _attrName.length() + sizeof(int) + _fv.size() + sizeof(bool) + sizeof(CegoDataType) + sizeof(int); 
+    return _tableName.length() + _tableAlias.length() + _attrName.length() + sizeof(int) + + sizeof(int) + _fv.size() + sizeof(bool) + sizeof(CegoDataType) + sizeof(int); 
 }
 
 CegoField& CegoField::operator = ( const CegoField& f)
@@ -163,6 +191,7 @@ CegoField& CegoField::operator = ( const CegoField& f)
     _isNullable = f._isNullable;
     _id = f._id;
     _len = f._len;
+    _dim = f._dim;
     _type = f._type;
     _fv = (CegoFieldValue)f._fv;
 
@@ -187,7 +216,6 @@ bool CegoField::operator != ( const CegoField& f)
     return ! ( *this == f );
 }
 
-
 Chain CegoField::typeToChain() const
 {
     Chain s;
@@ -200,14 +228,23 @@ Chain CegoField::typeToChain() const
 	break;
     }
     case VARCHAR_TYPE:
-    case FIXED_TYPE:
     case BIGINT_TYPE:
     case DECIMAL_TYPE:
     {
-	s = CEGO_TYPE_MAP[_type] + Chain("(") + Chain(_len) + Chain(")");
-	break;
+	s = CEGO_TYPE_MAP[_type] + Chain("(") + Chain(_len) + Chain(",") + Chain(_dim) + Chain(")");
+	break;	
     }
-    default:
+    case INT_TYPE:
+    case LONG_TYPE:
+    case BOOL_TYPE:
+    case DATETIME_TYPE:	
+    case FLOAT_TYPE:
+    case DOUBLE_TYPE:
+    case SMALLINT_TYPE:
+    case TINYINT_TYPE:
+    case BLOB_TYPE:	
+    case CLOB_TYPE:
+    case PAGEID_TYPE:
     {
 	s = CEGO_TYPE_MAP[_type];
 	break;
@@ -215,46 +252,3 @@ Chain CegoField::typeToChain() const
     }
     return s;
 }    
-
-Element* CegoField::toElement() const
-{
-    Element* pFieldElement = new Element(XML_FIELD_ELEMENT);
-    
-    pFieldElement->setAttribute( XML_TABLENAME_ATTR, _tableName );
-    pFieldElement->setAttribute( XML_TABLEALIAS_ATTR, _tableAlias );
-    pFieldElement->setAttribute( XML_ATTRNAME_ATTR, _attrName );
-
-    if ( _isNullable )
-	pFieldElement->setAttribute( XML_COLNULLABLE_ATTR, XML_TRUE_VALUE);
-    else
-	pFieldElement->setAttribute( XML_COLNULLABLE_ATTR, XML_FALSE_VALUE);
-
-    CegoTypeConverter tc;
-    pFieldElement->setAttribute( XML_COLTYPE_ATTR, tc.getTypeString(_type ));
-    pFieldElement->setAttribute( XML_COLSIZE_ATTR, Chain(_len) );	
-    return pFieldElement;
-}
-
-void CegoField::fromElement(Element *pFieldElement)
-{
-    
-    _tableName = pFieldElement->getAttributeValue( XML_TABLENAME_ATTR );
-    _tableAlias = pFieldElement->getAttributeValue( XML_TABLEALIAS_ATTR );
-    _attrName = pFieldElement->getAttributeValue( XML_ATTRNAME_ATTR );
-
-    Chain nullableString = pFieldElement->getAttributeValue( XML_COLNULLABLE_ATTR );
-    
-    if ( nullableString == Chain(XML_TRUE_VALUE) )
-    {
-	_isNullable = true;
-    }
-    else
-    {
-	_isNullable = false;
-    }
-
-    CegoTypeConverter tc;
-
-    _fv.setType( tc.getTypeId(pFieldElement->getAttributeValue( XML_COLTYPE_ATTR )) );
-    _fv.setLength( pFieldElement->getAttributeValue( XML_COLSIZE_ATTR ).asInteger() );    
-}
