@@ -8,7 +8,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //         
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -36,7 +36,10 @@ public:
     CegoField(const CegoField& f);
     CegoField(const Chain& tableName, const Chain& attrName);
     
-    CegoField(const Chain& tableName, const Chain& tableAlias, const Chain& attrName, CegoDataType type, int len, const CegoFieldValue& defVal = CegoFieldValue(), bool isNullable = false, int id = 0);
+    CegoField(const Chain& tableName, const Chain& tableAlias, const Chain& attrName,
+	      CegoDataType type, int len, int dim = 0,
+	      const CegoFieldValue& defVal = CegoFieldValue(),
+	      bool isNullable = false, int id = 0);
     
     CegoField(const CegoFieldValue& fv);
 
@@ -57,24 +60,28 @@ public:
     void setAttrName(const Chain& attrName);
     const Chain& getAttrName() const;
 
-    const CegoDataType getType() const;
+    const CegoDataType& getType() const;
     void setType(const CegoDataType t);
 
     int getLength() const;
     void setLength(int l);
-    
+
+    int getDim() const;
+    void setDim(int d);
+
     const CegoFieldValue& getValue() const;
     void setValue(const CegoFieldValue& fv);
 
+    // performance critical setup
+    void setupNull();
+    void setupValue(CegoDataType type, void* pValue, int len);
+
     int size() const;
     CegoField& operator = ( const CegoField& f);
     bool operator == ( const CegoField& f);
     bool operator != ( const CegoField& f);
 
     Chain typeToChain() const;
-
-    Element* toElement() const;
-    void fromElement(Element *pFieldElement);
     
 private:
     
@@ -87,5 +94,6 @@ private:
     
     CegoDataType _type;
     int _len;
+    int _dim;
 };
 #endif
