@@ -40,6 +40,7 @@ CegoFileHandler::CegoFileHandler(const Chain& logFile, const Chain& progName) :
 	_buMask[i]=0;
 	_commitMask[i]=0;
 	_pageOffset[i]=0;
+	_numPages[i]=0;
     }
     _appendFid = 0;
     _isReadOnly = false;
@@ -159,6 +160,19 @@ PageIdType CegoFileHandler::getPageOffset(int fileId)
     return _pageOffset[fileId];
 }
 
+
+PageIdType CegoFileHandler::getMaxPageOffset()
+{
+    PageIdType maxOffset=1;
+    
+    for (int i = 0; i<FILMNG_MAXDATAFILE; i++)
+    {
+	if (  _pageOffset[i] + _numPages[i] > maxOffset )
+	    maxOffset = _pageOffset[i] + _numPages[i];
+    }
+    return maxOffset;
+}
+
 int CegoFileHandler::getFBMSize(int fileId)
 {
     return (_numPages[fileId] / (BIT_PER_BYTE * sizeof(unsigned))) + 1;
