@@ -31,6 +31,7 @@
 #include "CegoProcedure.h"
 #include "CegoAction.h"
 #include "CegoTypeConverter.h"
+#include "CegoReplacer.h"
 
 // POSIX INCLUDES
 #include <string.h>
@@ -319,6 +320,11 @@ Chain CegoFunction::getName() const
 	funcString = Chain("replace");
 	break;	
     }
+    case CegoFunction::REGMATCH:
+    {
+	funcString = Chain("regmatch");
+	break;	
+    }
     case CegoFunction::LENGTH:
     {
 	funcString = Chain("length");
@@ -363,6 +369,7 @@ CegoDataType CegoFunction::getReturnType()
     case RIGHT:
     case SUBSTR:
     case REPLACE:
+    case REGMATCH:
     case RANDSTR:
     case CLOB2STR:
 	return VARCHAR_TYPE;
@@ -418,7 +425,7 @@ CegoDataType CegoFunction::getReturnType()
 	
 	try 
 	{	    
-	    _pProc = _pTabMng->getProcedure(_tabSetId, _funcName);	    
+	    _pProc = _pTabMng->getProcedure(_tabSetId, _funcName);
 	    returnType = _pProc->getReturnType();	  	    
 	}
 	catch ( Exception e )
@@ -510,6 +517,11 @@ int CegoFunction::getReturnTypeLen(const ListT<CegoField>& fl)
 	CegoExpr** pExpr = _exprList.First();
 	return (*pExpr)->evalField(fl).getLength();
     }
+    case REGMATCH:
+    {
+	CegoExpr** pExpr = _exprList.First();
+	return (*pExpr)->evalField(fl).getLength();
+    }
     case ROUND:
     case GETPOS:
     case LENGTH:
@@ -573,7 +585,45 @@ int CegoFunction::getReturnTypeLen(const ListT<CegoField>& fl)
     }
     }
 }
-    
+
+int CegoFunction::getReturnTypeDim()
+{
+    if ( _type == USERDEFINED )
+    {
+	if ( _pTabMng == 0 )
+	    throw Exception(EXLOC, "No valid table manager set up");
+	
+	int returnTypeDim;
+
+	if ( _pProc )
+	{
+	    returnTypeDim = _pProc->getReturnTypeDim();
+	    return returnTypeDim;
+	}
+	
+	if ( _pTabMng->isGraceMode()
+	     &&  _pTabMng->getDBMng()->objectExists(_tabSetId, _funcName, CegoObject::PROCEDURE) == false )
+	    return 0;
+
+	_pTabMng->getDBMng()->useObject(_tabSetId, _funcName, CegoObject::PROCEDURE, CegoDatabaseManager::SHARED, _pTabMng);
+	
+	try 
+	{	    
+	    _pProc = _pTabMng->getProcedure(_tabSetId, _funcName);	    
+	    returnTypeDim = _pProc->getReturnTypeDim();	  	    
+	}
+	catch ( Exception e )
+	{
+	    _pTabMng->getDBMng()->unuseObject(_tabSetId, _funcName, CegoObject::PROCEDURE);
+	    throw Exception(EXLOC, Chain("Cannot get user function return type dim"), e);
+	}
+
+	return returnTypeDim;	
+    }
+    return 0;
+}
+
+
 const CegoFunction::FunctionType CegoFunction::getType() const
 {
     return _type;
@@ -617,6 +667,8 @@ void CegoFunction::setExprList(ListT<CegoExpr*>& exprList)
 	throw Exception(EXLOC, Chain("Invalid parameter count for substr function"));
     if ( _type == REPLACE && exprList.Size() != 3)
 	throw Exception(EXLOC, Chain("Invalid parameter count for replace function"));
+    if ( _type == REGMATCH && ( exprList.Size() != 3 && exprList.Size() != 4 ) )
+	 throw Exception(EXLOC, Chain("Invalid parameter count for regmatch function"));
     if ( _type == LENGTH && exprList.Size() != 1)
 	throw Exception(EXLOC, Chain("Invalid parameter count for length function"));
     if ( _type == TRUNC && exprList.Size() != 1)
@@ -881,9 +933,9 @@ CegoFieldValue CegoFunction::evalFieldValue(ListT<CegoField>** pFLA, CegoProcBlo
 	    throw Exception(EXLOC, msg);	    
 	}
 
-	int val = 0; 
+	long long val = 0; 
 	if ( fv1.getValue() != 0 )
-	    memcpy(&val, fv1.getValue(), sizeof(int));
+	    memcpy(&val, fv1.getValue(), sizeof(long long));
 
 	Datetime dt;
 
@@ -913,9 +965,9 @@ CegoFieldValue CegoFunction::evalFieldValue(ListT<CegoField>** pFLA, CegoProcBlo
 	    throw Exception(EXLOC, msg);	    
 	}
 
-	unsigned long long dateVal = 0; 
+	long long dateVal = 0; 
 	if ( fv1.getValue() != 0 )
-	    memcpy(&dateVal, fv1.getValue(), sizeof(unsigned long long));
+	    memcpy(&dateVal, fv1.getValue(), sizeof(long long));
 	if ( dateVal == 0 )
 	{	   
 	    Datetime dt;
@@ -939,13 +991,13 @@ CegoFieldValue CegoFunction::evalFieldValue(ListT<CegoField>** pFLA, CegoProcBlo
 	    Chain msg = Chain("Cannot cast to long"); 
 	    throw Exception(EXLOC, msg);	    	    
 	}
-	unsigned long long *pVal = new unsigned long long;
+	long long *pVal = new long long;
 	if ( fv1.getValue() != 0 )
-	    memcpy(pVal, fv1.getValue(), sizeof(unsigned long long));
+	    memcpy(pVal, fv1.getValue(), sizeof(long long));
 	else
 	    *pVal = 0;
 
-	CegoFieldValue fv = CegoFieldValue(DATETIME_TYPE, pVal, sizeof(unsigned long long), true);
+	CegoFieldValue fv = CegoFieldValue(DATETIME_TYPE, pVal, sizeof(long long), true);
 	return fv;
     }
     case CegoFunction::NEWDATE:
@@ -1051,12 +1103,12 @@ CegoFieldValue CegoFunction::evalFieldValue(ListT<CegoField>** pFLA, CegoProcBlo
 	    }
 	}
 	
-	unsigned long long *pDateTimeValue = new unsigned long long;
+	long long *pDateTimeValue = new long long;
 	
 	Datetime dt(year, month, day, hour, minute, second);
 	*pDateTimeValue = dt.asLong();
 	
-	CegoFieldValue fv = CegoFieldValue(DATETIME_TYPE, pDateTimeValue, sizeof(unsigned long long), true);
+	CegoFieldValue fv = CegoFieldValue(DATETIME_TYPE, pDateTimeValue, sizeof(long long), true);
 	return fv;
     }    
     case CegoFunction::TRUNC:
@@ -1286,6 +1338,69 @@ CegoFieldValue CegoFunction::evalFieldValue(ListT<CegoField>** pFLA, CegoProcBlo
 	CegoFieldValue fv(VARCHAR_TYPE, res);
 	return fv;
     }
+    case CegoFunction::REGMATCH:
+    {    
+	CegoExpr** pExpr1 = _exprList.First();
+	CegoExpr** pExpr2 = _exprList.Next();
+	CegoExpr** pExpr3 = _exprList.Next();
+
+	CegoExpr** pExpr4 = _exprList.Next();
+
+	Chain matchVar;
+	
+	if ( pExpr4 )
+	{
+	    if ( (*pExpr4)->checkVar(matchVar) == false )
+	    {
+		Chain msg = Chain("Invalid argument for replace count variable");
+		throw Exception(EXLOC, msg);		
+	    }
+	}
+	CegoFieldValue fv1, fv2, fv3;
+	
+	fv1 = (*pExpr1)->evalFieldValue(pFLA, pBlock);
+
+	if  ( fv1.isNull() )
+        {
+            Chain msg = Chain("Search attern cannot be empty");
+            throw Exception(EXLOC, msg);
+        }
+
+	fv2 = (*pExpr2)->evalFieldValue(pFLA, pBlock);
+
+	if  ( fv2.isNull() )
+        {
+            Chain msg = Chain("Replace pattern cannot be empty");
+            throw Exception(EXLOC, msg);
+        }
+
+	fv3 = (*pExpr3)->evalFieldValue(pFLA, pBlock);
+       
+	if  ( fv3.isNull() )
+        {
+            Chain msg = Chain("Source string cannot be empty");
+            throw Exception(EXLOC, msg);
+        }
+
+	Chain pattern = fv1.valAsChain();
+	Chain replace = fv2.valAsChain();
+	Chain src = fv3.valAsChain();
+
+	CegoReplacer *pRep = _pTabMng->getReplacer(pattern, replace);
+	   
+	Chain target;
+	int nmatch = pRep->replace(src, target);
+
+	if ( pExpr4 )
+	{
+	    CegoFieldValue matchVal(INT_TYPE, Chain(nmatch));
+	    pBlock->setValue(matchVar, matchVal); 
+	}
+	
+	CegoFieldValue retVal(VARCHAR_TYPE, target);
+	return retVal;
+    }
+
     case CegoFunction::LENGTH:
     {
 	CegoExpr** pExpr1 = _exprList.First();
@@ -2349,3 +2464,52 @@ ostream& operator << (ostream& s, const CegoFunction& f)
     s << f.toChain(0);
     return s;
 }
+
+
+CegoFunction* CegoFunction::map(const Chain& viewAlias, const ListT<CegoExpr*>& exprList) const
+{
+    ListT<CegoExpr*> cloneList;
+
+    bool doMap=true;
+    
+    CegoExpr** pExpr = _exprList.First();
+    while ( pExpr && doMap )
+    {
+	CegoExpr* pMapExpr = (*pExpr)->map(viewAlias, exprList);
+	if ( pMapExpr )
+	    cloneList.Insert(pMapExpr);
+	else
+	    doMap=false;
+	
+	pExpr = _exprList.Next();
+    }
+
+    if ( doMap == false )
+    {
+	CegoExpr** pMapExpr = cloneList.First();
+	while ( pMapExpr )
+	{
+	    delete (*pMapExpr);
+	    pMapExpr = cloneList.Next();
+	}
+	return 0;
+    }
+    
+    if ( _pTabMng == 0 )
+    {
+	return ( new CegoFunction(_type, cloneList) );
+    }
+    else 
+    {
+	if ( _type == CegoFunction::USERDEFINED )
+	{
+	    return ( new CegoFunction(_pTabMng, _tabSetId, _funcName, cloneList) );
+	}
+	else
+	{
+	    CegoFunction *pFunc = new CegoFunction(_pTabMng, _tabSetId, _type);
+            pFunc->setCounterId(_counterId);
+            return pFunc;
+	}
+    }
+}
