@@ -6,7 +6,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -25,7 +25,6 @@
 #include "CegoGroupSpace.h"
 #include "CegoDataType.h"
 #include "CegoTableManager.h"
-#include "CegoGroupEntry.h"
 #include "CegoTypeConverter.h"
 
 // POSIX INCLUDES
@@ -35,7 +34,7 @@
 CegoGroupSpace::CegoGroupSpace()
 {
     _pAVL = new AVLTreeT<CegoGroupNode>;
-    _pCountAgg = new CegoAggregation();
+    _pCountAgg = new CegoAggregation(false);
 }
 
 CegoGroupSpace::~CegoGroupSpace()
@@ -46,7 +45,7 @@ CegoGroupSpace::~CegoGroupSpace()
 	delete _pCountAgg;
 }
 
-void CegoGroupSpace::initGroupSpace(const ListT<CegoField>& schema, ListT<CegoAggregation*>& aggList, long maxOrderSize)
+void CegoGroupSpace::initGroupSpace(const ListT<CegoField>& schema, ListT<CegoAggregation*>& aggList, unsigned long long maxOrderSize)
 {
     _aggList = aggList;
     _maxOrderSize = maxOrderSize;
@@ -59,7 +58,6 @@ void CegoGroupSpace::initGroupSpace(const ListT<CegoField>& schema, ListT<CegoAg
     CegoField* pF = schema.First();
     while ( pF )
     {
-
 	pF->setId(id);
 	
 	_groupSchema.Insert(*pF);
@@ -75,13 +73,12 @@ void CegoGroupSpace::initGroupSpace(const ListT<CegoField>& schema, ListT<CegoAg
     CegoAggregation** pAgg = _aggList.First();
     while ( pAgg )
     {
-
 	CegoField f;
 	f.setId(id);
 	(*pAgg)->setAggregationId(id);
 
 	f.setType(LONG_TYPE);
-	f.setAttrName((*pAgg)->toChain());
+	f.setAttrName((*pAgg)->toChain(0));
 
 	// _groupSchema.Insert(f);
 	_aggSchema.Insert(f);
@@ -142,11 +139,9 @@ ListT<CegoField> CegoGroupSpace::getSchema()
 
 void CegoGroupSpace::insertTuple(ListT<CegoField>& groupTuple)
 {
-
     // we create a copy of addKey with local field values
     // this is required, since field values references might be invalidated during the grouping query
         
-    int id=1;
     CegoField* pSF = _groupSchema.First();
     while (pSF)
     {
@@ -187,14 +182,13 @@ CegoGroupCursor* CegoGroupSpace::getCursor()
     return pGC;
 }
 
-long CegoGroupSpace::numAllocated() const
+unsigned long long CegoGroupSpace::numAllocated() const
 {
     return _orderSize;
 }
 
 ListT<CegoField> CegoGroupSpace::initGrouping(ListT<CegoField>& dataTuple)
 {
-
     ListT<CegoField> initValue;
 
     CegoAggregation** pAgg = _aggList.First();
@@ -203,7 +197,9 @@ ListT<CegoField> CegoGroupSpace::initGrouping(ListT<CegoField>& dataTuple)
     {
 	CegoField f;
 	f.setId(i);
-        
+
+	// (*pAgg)->setInit(true);
+	
 	switch ( (*pAgg)->getType() )
 	{	    
 	case CegoAggregation::COUNT:
@@ -231,8 +227,7 @@ ListT<CegoField> CegoGroupSpace::initGrouping(ListT<CegoField>& dataTuple)
 }
 
 ListT<CegoField> CegoGroupSpace::aggGrouping(ListT<CegoField>& aggValues, ListT<CegoField>& dataTuple)
-{
-    
+{   
     CegoAggregation** pAgg = _aggList.First();
     CegoField *pF = aggValues.First();
 
@@ -264,8 +259,7 @@ ListT<CegoField> CegoGroupSpace::aggGrouping(ListT<CegoField>& aggValues, ListT<
 }
 
 CegoFieldValue CegoGroupSpace::getValueForAgg(CegoAggregation* pAgg, ListT<CegoField>& dataTuple)
-{
-	
+{	
     CegoField f;
     CegoFieldValue fv;
 
@@ -277,10 +271,12 @@ CegoFieldValue CegoGroupSpace::getValueForAgg(CegoAggregation* pAgg, ListT<CegoF
 	fv.setType(LONG_TYPE);
     }
     else
-    {	
-	pAE->setFieldListArray( &dataTuple );
-	
-	fv = pAE->evalFieldValue();
+    {
+	ListT<CegoField>* fl[2];
+	fl[0] = &dataTuple;
+	fl[1] = 0;
+
+	fv = pAE->evalFieldValue(fl, 0 );
 
 	if ( fv.getValue() == 0 && pAgg->getType() != CegoAggregation::MIN )
 	{
@@ -298,7 +294,7 @@ CegoFieldValue CegoGroupSpace::getValueForAgg(CegoAggregation* pAgg, ListT<CegoF
 	    }
 	    case VARCHAR_TYPE:
 	    {
-		fv = CegoFieldValue(VARCHAR_TYPE, Chain(""));
+		fv = CegoFieldValue(VARCHAR_TYPE, Chain());
 		break;
 	    }
 	    case DECIMAL_TYPE:
@@ -306,12 +302,17 @@ CegoFieldValue CegoGroupSpace::getValueForAgg(CegoAggregation* pAgg, ListT<CegoF
 		fv = CegoFieldValue(DECIMAL_TYPE, Chain("0.0"));
 		break;		   
 	    }
-	    case FIXED_TYPE:
-	    {
-		fv = CegoFieldValue(FIXED_TYPE, Chain("0.0"));
-		break;		   
-	    }
-	    default:
+	    case NULL_TYPE:
+	    case FLOAT_TYPE:
+	    case DOUBLE_TYPE:
+	    case SMALLINT_TYPE:
+	    case TINYINT_TYPE:
+	    case BIGINT_TYPE:
+	    case BOOL_TYPE:
+	    case DATETIME_TYPE:
+	    case BLOB_TYPE:
+	    case CLOB_TYPE:
+	    case PAGEID_TYPE:
 	    {
 		Chain msg = Chain("Aggregation not supported on datatype <") + CEGO_TYPE_MAP[ (int) fv.getType()] + Chain(">");
 		throw Exception(EXLOC, msg);
@@ -321,5 +322,5 @@ CegoFieldValue CegoGroupSpace::getValueForAgg(CegoAggregation* pAgg, ListT<CegoF
 	}
     }
     
-    return fv;
+    return fv.getLocalCopy();
 }
