@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -38,7 +38,7 @@ public:
     CegoGroupSpace();
     ~CegoGroupSpace();
        
-    void initGroupSpace(const ListT<CegoField>& schema, ListT<CegoAggregation*>& aggList, long maxGroupSize);
+    void initGroupSpace(const ListT<CegoField>& schema, ListT<CegoAggregation*>& aggList, unsigned long long maxGroupSize);
 
     void insertTuple(ListT<CegoField>& groupTuple);
 
@@ -48,7 +48,7 @@ public:
 
     CegoGroupCursor* getCursor();
 
-    long numAllocated() const;
+    unsigned long long numAllocated() const;
 
 private:
 
@@ -68,7 +68,7 @@ private:
 
     AVLTreeT<CegoGroupNode>* _pAVL;
 
-    long _maxOrderSize;
-    long _orderSize;
+    unsigned long long _maxOrderSize;
+    unsigned long long _orderSize;
 };
 #endif
