@@ -6,7 +6,7 @@
 //      
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -19,10 +19,9 @@
 ///////////////////////////////////////////////////////////////////////////////
 
 #include "CegoJDBCInfo.h"
-
-#include "CegoTypeConverter.h"
 #include "CegoField.h"
 #include "CegoOutput.h"
+#include "CegoTypeConverter.h"
 
 CegoJDBCInfo::CegoJDBCInfo(CegoDistManager* pTabMng, CegoDbHandler* pDbHandle)
 {
@@ -44,21 +43,18 @@ void CegoJDBCInfo::checkAccess() const
 
 void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 {
-
     Chain *pReq = argList.First();
 
     if ( pReq )
     {
-
 	if ( *pReq == Chain("catalog") )
 	{
-
 	    ListT<CegoField> schema;
 	    schema.Insert( CegoField("JDBC", "JDBC", "TABLE_CAT", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 	    
 	    CegoOutput o(schema);
 	    if ( _pDbHandle )
-		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 	    
 	    o.headOut();
 	    
@@ -69,8 +65,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    o.tailOut();
 	}
 	else if ( *pReq == Chain("schema") )
-	{
-	    
+	{	    
 	    checkAccess();
 
 	    ListT<Chain> tsList;
@@ -81,7 +76,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    
 	    CegoOutput o(schema);
 	    if ( _pDbHandle )
-		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 	    
 	    o.headOut();
 	    
@@ -96,14 +91,13 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    o.tailOut();
 	}
 	else if ( *pReq == Chain("tabletype") )
-	{
-	    
+	{	    
 	    ListT<CegoField> schema;
 	    schema.Insert( CegoField("JDBC", "JDBC", "TABLE_TYPE", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 	    
 	    CegoOutput o(schema);
 	    if ( _pDbHandle )
-		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 	    
 	    o.headOut();
 	    
@@ -113,9 +107,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    
 	    ListT<CegoFieldValue> t2;
 	    t2.Insert( CegoFieldValue(VARCHAR_TYPE, Chain("VIEW")));
-	    o.rowOut(t2);
-	    
-	    
+	    o.rowOut(t2);	    	    
 	    o.tailOut();
 	}
 	else if ( *pReq == Chain("tab") )
@@ -125,10 +117,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    Chain *pTS = argList.Next();
 
 	    if ( pTS )
-	    {
-		ListT<Chain> tabList;
-		_pTabMng->getDistObjectList(*pTS, CegoObject::TABLE, tabList);
-		
+	    {		
 		ListT<CegoField> schema;
 		schema.Insert( CegoField("JDBC", "JDBC", "TABLE_CAT", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "TABLE_SCHEM", VARCHAR_TYPE, MAX_OBJNAME_LEN));
@@ -143,10 +132,13 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		
 		CegoOutput o(schema);
 		if ( _pDbHandle )
-		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 		
 		o.headOut();
-		
+
+		ListT<Chain> tabList;
+		_pTabMng->getDistObjectList(*pTS, CegoObject::TABLE, tabList);
+
 		Chain *pS = tabList.First();
 		while ( pS )
 		{
@@ -165,11 +157,43 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		    o.rowOut(f1);
 		    pS = tabList.Next();
 		}
+		o.tailOut();
+	    }
+	    else
+	    {
+		throw Exception(EXLOC, "Missing args for JDBC info");
+	    }	
+	}
+	else if ( *pReq == Chain("vie") )
+	{
+	    checkAccess();
+
+	    Chain *pTS = argList.Next();
+
+	    if ( pTS )
+	    {		
+		ListT<CegoField> schema;
+		schema.Insert( CegoField("JDBC", "JDBC", "TABLE_CAT", VARCHAR_TYPE, MAX_OBJNAME_LEN));
+		schema.Insert( CegoField("JDBC", "JDBC", "TABLE_SCHEM", VARCHAR_TYPE, MAX_OBJNAME_LEN));
+		schema.Insert( CegoField("JDBC", "JDBC", "TABLE_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN)); 
+		schema.Insert( CegoField("JDBC", "JDBC", "TABLE_TYPE", VARCHAR_TYPE, MAX_OBJNAME_LEN));
+		schema.Insert( CegoField("JDBC", "JDBC", "REMARKS", VARCHAR_TYPE, MAX_OBJNAME_LEN));
+		schema.Insert( CegoField("JDBC", "JDBC", "TYPE_CAT", VARCHAR_TYPE, MAX_OBJNAME_LEN));
+		schema.Insert( CegoField("JDBC", "JDBC", "TYPE_SCHEM", VARCHAR_TYPE, MAX_OBJNAME_LEN));
+		schema.Insert( CegoField("JDBC", "JDBC", "TYPE_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
+		schema.Insert( CegoField("JDBC", "JDBC", "SELF_REFERENCING_COL_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
+		schema.Insert( CegoField("JDBC", "JDBC", "REF_GENERATION", VARCHAR_TYPE, MAX_OBJNAME_LEN));
+		
+		CegoOutput o(schema);
+		if ( _pDbHandle )
+		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
+		
+		o.headOut();
 
 		ListT<Chain> viewList;
 		_pTabMng->getDistObjectList(*pTS, CegoObject::VIEW, viewList);
 				
-		pS = viewList.First();
+		Chain *pS = viewList.First();
 		while ( pS )
 		{
 		    ListT<CegoFieldValue> f1;
@@ -187,19 +211,15 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		    o.rowOut(f1);
 		    pS = viewList.Next();
 		}
-				
-
 		o.tailOut();
 	    }
 	    else
 	    {
 		throw Exception(EXLOC, "Missing args for JDBC info");
-	    }
-		
+	    }		
 	}
 	else if ( *pReq == Chain("proc") )
 	{
-
 	    checkAccess();
 
 	    Chain *pTS = argList.Next();
@@ -208,8 +228,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    {
 		ListT<Chain> objList;
 		_pTabMng->getDistObjectList(*pTS, CegoObject::PROCEDURE, objList);
-		
-		
+				
 		ListT<CegoField> schema;
 		schema.Insert( CegoField("JDBC", "JDBC", "PROCEDURE_CAT", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "PROCEDURE_SCHEMA", VARCHAR_TYPE, MAX_OBJNAME_LEN));
@@ -222,7 +241,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		
 		CegoOutput o(schema);
 		if ( _pDbHandle )
-		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 		
 		o.headOut();
 		
@@ -251,22 +270,22 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	}
 	else if ( *pReq == Chain("pkey") )
 	{
-
 	    checkAccess();
 	    
 	    Chain *pTS = argList.Next();
 	    Chain *pTable = argList.Next();
 	    
 	    if ( pTS  && pTable)
-	    {
-		
+	    {		
 		ListT<CegoTableObject> idxList;
 		ListT<CegoBTreeObject> btreeList;
 		ListT<CegoKeyObject> keyList;
 		ListT<CegoCheckObject> checkList;
+		ListT<CegoTriggerObject> triggerList;
+		ListT<CegoAliasObject> aliasList;
 		int numInvalid;
 
-		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList, numInvalid);
+		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList, triggerList, aliasList, numInvalid);
 		
 		ListT<CegoField> schema;
 		schema.Insert( CegoField("JDBC", "JDBC", "TABLE_CAT", VARCHAR_TYPE, MAX_OBJNAME_LEN));
@@ -278,14 +297,13 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		
 		CegoOutput o(schema);
 		if ( _pDbHandle )
-		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 		
 		o.headOut();
 		
 		CegoTableObject *pIdx = idxList.First();
 		while ( pIdx )
-		{
-		    
+		{		    
 		    if ( pIdx->getType() == CegoObject::PAVLTREE )
 		    {
 			CegoTableObject oe;
@@ -306,16 +324,14 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 			    o.rowOut(f1);
 			    
 			    i++;
-			    pF = oe.getSchema().Next();
-			    
+			    pF = oe.getSchema().Next();			    
 			}
 		    }
 		    pIdx = idxList.Next();
 		    
 		    CegoBTreeObject *pBTree = btreeList.First();
 		    while ( pBTree )
-		    {
-			
+		    {			
 			if ( pBTree->getType() == CegoObject::PBTREE )
 			{
 			    CegoBTreeObject oe;
@@ -336,8 +352,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 				o.rowOut(f1);
 				
 				i++;
-				pF = oe.getSchema().Next();
-				
+				pF = oe.getSchema().Next();				
 			    }
 			}
 			pBTree = btreeList.Next();			
@@ -352,7 +367,6 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	}
 	else if ( *pReq == Chain("ikey") )
 	{
-
 	    checkAccess();
 
 	    Chain *pTS = argList.Next();
@@ -360,45 +374,41 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 
 	    if ( pTS  && pTable)
 	    {
-
 		ListT<CegoTableObject> idxList;
 		ListT<CegoBTreeObject> btreeList;
 		ListT<CegoKeyObject> keyList;
 		ListT<CegoCheckObject> checkList;
+		ListT<CegoTriggerObject> triggerList;
+		ListT<CegoAliasObject> aliasList;
 		int numInvalid;
 
-		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList, numInvalid);
+		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList, triggerList, aliasList, numInvalid);
 
 		ListT<CegoField> schema;
  		schema.Insert( CegoField("JDBC", "JDBC", "PKTABLE_CAT", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "PKTABLE_SCHEM", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "PKTABLE_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "PKCOLUMN_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
-
 		schema.Insert( CegoField("JDBC", "JDBC", "FKTABLE_CAT", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "FKTABLE_SCHEM", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "FKTABLE_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "FKCOLUMN_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
-
 		schema.Insert( CegoField("JDBC", "JDBC", "KEQ_SEQ", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "UPADATE_RULE", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "DELETE_RULE", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "FK_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "PK_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
-
 		schema.Insert( CegoField("JDBC", "JDBC", "DEFERRABILITY", VARCHAR_TYPE, MAX_OBJNAME_LEN));
-
 		
 		CegoOutput o(schema);
 		if ( _pDbHandle )
-		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 		
 		o.headOut();
 		
 		CegoKeyObject *pKey = keyList.First();
 		while ( pKey )
-		{
-		    
+		{		    
 		    if ( pKey->getType() == CegoObject::FKEY )
 		    {
 			CegoKeyObject ko;
@@ -414,28 +424,22 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, *pTS));
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, ko.getTabName()));
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, pKF->getAttrName()));
-
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, _pTabMng->getDBMng()->getDBName()));
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, *pTS));
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, ko.getRefTable()));
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, pRF->getAttrName()));
-
-
 			    f1.Insert( CegoFieldValue(SMALLINT_TYPE, i));
 			    f1.Insert( CegoFieldValue(SMALLINT_TYPE, 0)); // update rule
 			    f1.Insert( CegoFieldValue(SMALLINT_TYPE, 0)); // delete rule
-
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, ko.getName()));
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, ko.getName()));
-
 			    f1.Insert( CegoFieldValue(SMALLINT_TYPE, 0)); // deferrability
 			    
 			    o.rowOut(f1);
 			    
 			    i++;
 			    pKF = ko.getKeySchema().Next();
-			    pRF = ko.getRefSchema().Next();
-			    
+			    pRF = ko.getRefSchema().Next();			    
 			}
 		    }
 		    pKey = keyList.Next();
@@ -449,7 +453,6 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	}
 	else if ( *pReq == Chain("ekey") )
 	{
-
 	    checkAccess();
 
 	    Chain *pTS = argList.Next();
@@ -457,14 +460,15 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 
 	    if ( pTS  && pTable)
 	    {
-
 		ListT<CegoTableObject> idxList;
 		ListT<CegoBTreeObject> btreeList;
 		ListT<CegoKeyObject> keyList;
 		ListT<CegoCheckObject> checkList;
+		ListT<CegoTriggerObject> triggerList;
+		ListT<CegoAliasObject> aliasList;
 		int numInvalid;
 
-		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList, numInvalid);
+		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList, triggerList, aliasList, numInvalid);
 
 		ListT<CegoField> schema;
 
@@ -472,30 +476,26 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		schema.Insert( CegoField("JDBC", "JDBC", "PKTABLE_SCHEM", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "PKTABLE_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "PKCOLUMN_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
-
 		schema.Insert( CegoField("JDBC", "JDBC", "FKTABLE_CAT", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "FKTABLE_SCHEM", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "FKTABLE_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "FKCOLUMN_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
-
 		schema.Insert( CegoField("JDBC", "JDBC", "KEQ_SEQ", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "UPADATE_RULE", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "DELETE_RULE", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "FK_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "PK_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
-
 		schema.Insert( CegoField("JDBC", "JDBC", "DEFERRABILITY", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		
 		CegoOutput o(schema);
 		if ( _pDbHandle )
-		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 		
 		o.headOut();
 
 		CegoKeyObject *pKey = keyList.First();
 		while ( pKey )
-		{
-		    
+		{		    
 		    if ( pKey->getType() == CegoObject::FKEY )
 		    {
 			CegoKeyObject ko;
@@ -511,20 +511,15 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, *pTS));
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, ko.getTabName()));
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, pKF->getAttrName()));
-
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, _pTabMng->getDBMng()->getDBName()));
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, *pTS));
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, ko.getRefTable()));
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, pRF->getAttrName()));
-
-
 			    f1.Insert( CegoFieldValue(SMALLINT_TYPE, i));
 			    f1.Insert( CegoFieldValue(SMALLINT_TYPE, 0)); // update rule
 			    f1.Insert( CegoFieldValue(SMALLINT_TYPE, 0)); // delete rule
-
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, ko.getName()));
 			    f1.Insert( CegoFieldValue(VARCHAR_TYPE, ko.getName()));
-
 			    f1.Insert( CegoFieldValue(SMALLINT_TYPE, 0)); // deferrability
 			    
 			    o.rowOut(f1);
@@ -532,7 +527,6 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 			    i++;
 			    pKF = ko.getKeySchema().Next();
 			    pRF = ko.getRefSchema().Next();
-			    
 			}
 		    }
 		    pKey = keyList.Next();
@@ -547,7 +541,6 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	}
 	else if ( *pReq == Chain("idx") || *pReq == Chain("uidx") )
 	{
-
 	    checkAccess();
 
 	    Chain *pTS = argList.Next();
@@ -555,14 +548,15 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 
 	    if ( pTS  && pTable)
 	    {
-
 		ListT<CegoTableObject> idxList;
 		ListT<CegoBTreeObject> btreeList;
 		ListT<CegoKeyObject> keyList;
 		ListT<CegoCheckObject> checkList;
+		ListT<CegoTriggerObject> triggerList;
+		ListT<CegoAliasObject> aliasList;
 		int numInvalid;
 
-		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList, numInvalid);
+		_pTabMng->getDistObjectByTableList(*pTS, *pTable, idxList, btreeList, keyList, checkList, triggerList, aliasList, numInvalid);
 
 		ListT<CegoField> schema;
 
@@ -572,28 +566,23 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		schema.Insert( CegoField("JDBC", "JDBC", "NON_UNIQUE", BOOL_TYPE, sizeof(char)));
 		schema.Insert( CegoField("JDBC", "JDBC", "INDEX_QUALIFIER", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "INDEX_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
-
 		schema.Insert( CegoField("JDBC", "JDBC", "TYPE", SMALLINT_TYPE, sizeof(short)));
 		schema.Insert( CegoField("JDBC", "JDBC", "ORDINAL_POSITION", SMALLINT_TYPE, sizeof(short)));
 		schema.Insert( CegoField("JDBC", "JDBC", "COLUMN_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "ASC_OR_DESC", VARCHAR_TYPE, MAX_OBJNAME_LEN));
-
 		schema.Insert( CegoField("JDBC", "JDBC", "CARDINALITY", INT_TYPE, sizeof(int)));
 		schema.Insert( CegoField("JDBC", "JDBC", "PAGES", INT_TYPE, sizeof(int)));
-
 		schema.Insert( CegoField("JDBC", "JDBC", "FILTER_CONDITION", VARCHAR_TYPE, MAX_OBJNAME_LEN));
-
 		
 		CegoOutput o(schema);
 		if ( _pDbHandle )
-		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 		
 		o.headOut();
 
 		CegoTableObject *pIdx = idxList.First();
 		while ( pIdx )
-		{
-		    
+		{	    
 		    CegoTableObject io;
 		    _pTabMng->getDistObject(*pTS, pIdx->getName(), CegoObject::AVLTREE, io);
 		    
@@ -615,15 +604,12 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 			f1.Insert( CegoFieldValue(VARCHAR_TYPE, io.getName())); // index name
 			f1.Insert( CegoFieldValue(SMALLINT_TYPE, 3)); // tableIndexOther
 			f1.Insert( CegoFieldValue(SMALLINT_TYPE, i)); // column pos
-			
 			f1.Insert( CegoFieldValue(VARCHAR_TYPE, pIF->getAttrName())); // column name
-			
 			f1.Insert( CegoFieldValue()); // sort sequence
 			f1.Insert( CegoFieldValue(INT_TYPE, 0)); // cardinality - not supported
 			f1.Insert( CegoFieldValue(INT_TYPE, 0)); // index pages - not supported
 			f1.Insert( CegoFieldValue()); // filter condition
-					      
-					      
+					      					      
 			o.rowOut(f1);
 			
 			i++;
@@ -634,8 +620,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 
 		CegoBTreeObject *pBTree = btreeList.First();
 		while ( pBTree )
-		{
-		    
+		{		   
 		    CegoBTreeObject bto;
 		    _pTabMng->getDistObject(*pTS, pBTree->getName(), CegoObject::BTREE, bto);
 		    
@@ -656,25 +641,20 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 			f1.Insert( CegoFieldValue()); // index catalog
 			f1.Insert( CegoFieldValue(VARCHAR_TYPE, bto.getName())); // index name
 			f1.Insert( CegoFieldValue(SMALLINT_TYPE, 3)); // tableIndexOther
-			f1.Insert( CegoFieldValue(SMALLINT_TYPE, i)); // column pos
-			
+			f1.Insert( CegoFieldValue(SMALLINT_TYPE, i)); // column pos			
 			f1.Insert( CegoFieldValue(VARCHAR_TYPE, pIF->getAttrName())); // column name
-			
 			f1.Insert( CegoFieldValue()); // sort sequence
 			f1.Insert( CegoFieldValue(INT_TYPE, 0)); // cardinality - not supported
 			f1.Insert( CegoFieldValue(INT_TYPE, 0)); // index pages - not supported
 			f1.Insert( CegoFieldValue()); // filter condition
-					      
-					      
+					      					      
 			o.rowOut(f1);
 			
 			i++;
 			pIF = bto.getSchema().Next();
 		    }
 		    pBTree = btreeList.Next();
-		}
-
-		
+		}		
 		o.tailOut();
 	    }
 	    else
@@ -684,7 +664,6 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	}
 	else if ( *pReq == Chain("colinfo") )
 	{
-
 	    checkAccess();
 
 	    Chain *pTS = argList.Next();
@@ -692,7 +671,6 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 
 	    if ( pTS && pTable )
 	    {
-
 		CegoTableObject oe;
 
 		if ( _pTabMng->distObjectExists(*pTS, *pTable, CegoObject::TABLE) )
@@ -704,7 +682,6 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		    Chain msg = "Unknown object " + *pTable;
 		    throw Exception (EXLOC, msg);
 		} 
-
 		
 		ListT<CegoField> schema;
 		schema.Insert( CegoField("JDBC", "JDBC", "TABLE_CAT", VARCHAR_TYPE, MAX_OBJNAME_LEN));
@@ -730,15 +707,13 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		schema.Insert( CegoField("JDBC", "JDBC", "SCOPE_TABLE", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "SCOPE_TABLE", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 		schema.Insert( CegoField("JDBC", "JDBC", "SOURCE_DATA_TYPE", SMALLINT_TYPE, sizeof(short)));
-
 		
 		CegoOutput o(schema);
 		if ( _pDbHandle )
-		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		    o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 		
 		o.headOut();
 		
-
 		int i=1;
 		CegoField *pF = oe.getSchema().First();
 		while ( pF )
@@ -775,8 +750,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		    o.rowOut(f1);
 		    i++;
 		    pF = oe.getSchema().Next();
-		}
-		
+		}		
 		o.tailOut();
 	    }
 	    else
@@ -785,14 +759,13 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    }
 	}
 	else if ( *pReq == Chain("prodname") )
-	{
-	    
+	{	    
 	    ListT<CegoField> schema;
 	    schema.Insert( CegoField("JDBC", "JDBC", "PROD_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 	    
 	    CegoOutput o(schema);
 	    if ( _pDbHandle )
-		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 	    
 	    o.headOut();
 	    
@@ -804,33 +777,29 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    
 	}
 	else if ( *pReq == Chain("prodversion") )
-	{
-	    
+	{	   
 	    ListT<CegoField> schema;
 	    schema.Insert( CegoField("JDBC", "JDBC", "PROD_VERSION", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 	    
 	    CegoOutput o(schema);
 	    if ( _pDbHandle )
-		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 	    
 	    o.headOut();
 	    
 	    ListT<CegoFieldValue> t1;
 	    t1.Insert( CegoFieldValue(VARCHAR_TYPE, Chain(CEGO_VERSION)));
-	    o.rowOut(t1);
-	    
+	    o.rowOut(t1);	    
 	    o.tailOut();
-	    
 	}
 	else if ( *pReq == Chain("commitmode") )
-	{
-	    
+	{	    
 	    ListT<CegoField> schema;
 	    schema.Insert( CegoField("JDBC", "JDBC", "COMMIT_MODE", BOOL_TYPE, sizeof(char)));
 	    
 	    CegoOutput o(schema);
 	    if ( _pDbHandle )
-		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 	    
 	    o.headOut();
 
@@ -839,20 +808,17 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 		t1.Insert( CegoFieldValue(BOOL_TYPE, Chain("true")));
 	    else
 		t1.Insert( CegoFieldValue(BOOL_TYPE, Chain("false")));
-	    o.rowOut(t1);
-	    
-	    o.tailOut();
-	    
+	    o.rowOut(t1);	    
+	    o.tailOut();	    
 	}
 	else if ( *pReq == Chain("isolevel") )
-	{
-	    
+	{	    
 	    ListT<CegoField> schema;
 	    schema.Insert( CegoField("JDBC", "JDBC", "ISOLATION", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 	    
 	    CegoOutput o(schema);
 	    if ( _pDbHandle )
-		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 	    
 	    o.headOut();
 	    
@@ -862,22 +828,18 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    else if ( _pTabMng->getIsolationLevel() == CegoTableManager::READ_UNCOMMITTED)
 		t1.Insert( CegoFieldValue(VARCHAR_TYPE, Chain("read_uncommitted")));
 
-	    o.rowOut(t1);
-	    
-	    o.tailOut();
-	    
+	    o.rowOut(t1);	    
+	    o.tailOut();	    
 	}
 	else if ( *pReq == Chain("typeinfo") )
-	{
-	    
+	{	    
 	    ListT<CegoField> schema;
 	    schema.Insert( CegoField("JDBC", "JDBC", "TYPE_NAME", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 	    schema.Insert( CegoField("JDBC", "JDBC", "DATA_TYPE", INT_TYPE, sizeof(int)));
 	    schema.Insert( CegoField("JDBC", "JDBC", "PRECISION", INT_TYPE, sizeof(int)));
 	    schema.Insert( CegoField("JDBC", "JDBC", "LITERAL_PREFIX", VARCHAR_TYPE, MAX_OBJNAME_LEN));
 	    schema.Insert( CegoField("JDBC", "JDBC", "LITERAL_SUFFIX", VARCHAR_TYPE, MAX_OBJNAME_LEN));
-	    schema.Insert( CegoField("JDBC", "JDBC", "CREATE_PARAMS", VARCHAR_TYPE, MAX_OBJNAME_LEN));
-	    
+	    schema.Insert( CegoField("JDBC", "JDBC", "CREATE_PARAMS", VARCHAR_TYPE, MAX_OBJNAME_LEN));	    
 	    schema.Insert( CegoField("JDBC", "JDBC", "NULLABLE", SMALLINT_TYPE, sizeof(short)));
 	    schema.Insert( CegoField("JDBC", "JDBC", "CASE_SENSITIVE", BOOL_TYPE, sizeof(char)));
 	    schema.Insert( CegoField("JDBC", "JDBC", "SEARCHABLE", SMALLINT_TYPE, sizeof(short)));
@@ -893,7 +855,7 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    
 	    CegoOutput o(schema);
 	    if ( _pDbHandle )
-		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
+		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
 	    
 	    o.headOut();
 	    
@@ -940,7 +902,6 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    t2.Insert( CegoFieldValue(INT_TYPE, 0)); // SQL datetime sub
 	    t2.Insert( CegoFieldValue(INT_TYPE, 0)); // num prec radix
 	    o.rowOut(t2);
-
 	    
 	    // LONG type	    
 	    ListT<CegoFieldValue> t3;
@@ -986,7 +947,6 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    t4.Insert( CegoFieldValue(INT_TYPE, 0)); // num prec radix
 	    o.rowOut(t4);
 
-
 	    // DATETIME type	    
 	    ListT<CegoFieldValue> t5;
 	    t5.Insert( CegoFieldValue(VARCHAR_TYPE, Chain("datetime"))); // type name
@@ -1009,8 +969,6 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    t5.Insert( CegoFieldValue(INT_TYPE, 0)); // num prec radix
 	    o.rowOut(t5);
 
-
-
 	    // BIGINT type	    
 	    ListT<CegoFieldValue> t6;
 	    t6.Insert( CegoFieldValue(VARCHAR_TYPE, Chain("bigint"))); // type name
@@ -1055,7 +1013,6 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    t7.Insert( CegoFieldValue(INT_TYPE, 0)); // num prec radix
 	    o.rowOut(t7);
 
-
 	    // DOUBLE type	    
 	    ListT<CegoFieldValue> t8;
 	    t8.Insert( CegoFieldValue(VARCHAR_TYPE, Chain("double"))); // type name
@@ -1100,10 +1057,10 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    t9.Insert( CegoFieldValue(INT_TYPE, 0)); // num prec radix
 	    o.rowOut(t9);
 
-	    // FIXED type	    
+	    // SMALLINT type	    
 	    ListT<CegoFieldValue> t10;
-	    t10.Insert( CegoFieldValue(VARCHAR_TYPE, Chain("fixed"))); // type name
-	    t10.Insert( CegoFieldValue(INT_TYPE, CEGO_JAVATYPE_MAP[FIXED_TYPE])); // SQL type
+	    t10.Insert( CegoFieldValue(VARCHAR_TYPE, Chain("smallint"))); // type name
+	    t10.Insert( CegoFieldValue(INT_TYPE, CEGO_JAVATYPE_MAP[SMALLINT_TYPE])); // SQL type
 	    t10.Insert( CegoFieldValue(INT_TYPE, 0)); // precision
 	    t10.Insert( CegoFieldValue()); // literal prefix
 	    t10.Insert( CegoFieldValue()); // literal suffix
@@ -1122,11 +1079,10 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    t10.Insert( CegoFieldValue(INT_TYPE, 0)); // num prec radix
 	    o.rowOut(t10);
 
-
-	    // SMALLINT type	    
+	    // TINYINT type	    
 	    ListT<CegoFieldValue> t11;
-	    t11.Insert( CegoFieldValue(VARCHAR_TYPE, Chain("smallint"))); // type name
-	    t11.Insert( CegoFieldValue(INT_TYPE, CEGO_JAVATYPE_MAP[SMALLINT_TYPE])); // SQL type
+	    t11.Insert( CegoFieldValue(VARCHAR_TYPE, Chain("tinyint"))); // type name
+	    t11.Insert( CegoFieldValue(INT_TYPE, CEGO_JAVATYPE_MAP[TINYINT_TYPE])); // SQL type
 	    t11.Insert( CegoFieldValue(INT_TYPE, 0)); // precision
 	    t11.Insert( CegoFieldValue()); // literal prefix
 	    t11.Insert( CegoFieldValue()); // literal suffix
@@ -1143,41 +1099,16 @@ void CegoJDBCInfo::handleRequest(const ListT<Chain>& argList) const
 	    t11.Insert( CegoFieldValue(INT_TYPE, 0)); // SQL data type
 	    t11.Insert( CegoFieldValue(INT_TYPE, 0)); // SQL datetime sub
 	    t11.Insert( CegoFieldValue(INT_TYPE, 0)); // num prec radix
-	    o.rowOut(t11);
-
-	    // TINYINT type	    
-	    ListT<CegoFieldValue> t12;
-	    t12.Insert( CegoFieldValue(VARCHAR_TYPE, Chain("tinyint"))); // type name
-	    t12.Insert( CegoFieldValue(INT_TYPE, CEGO_JAVATYPE_MAP[TINYINT_TYPE])); // SQL type
-	    t12.Insert( CegoFieldValue(INT_TYPE, 0)); // precision
-	    t12.Insert( CegoFieldValue()); // literal prefix
-	    t12.Insert( CegoFieldValue()); // literal suffix
-	    t12.Insert( CegoFieldValue()); // create params	
-	    t12.Insert( CegoFieldValue(SMALLINT_TYPE, 0)); // nullable
-	    t12.Insert( CegoFieldValue(BOOL_TYPE, Chain("true"))); // case sensitive
-	    t12.Insert( CegoFieldValue(SMALLINT_TYPE, 0)); // searchable
-	    t12.Insert( CegoFieldValue(BOOL_TYPE, Chain("true"))); // unsigned
-	    t12.Insert( CegoFieldValue(BOOL_TYPE, Chain("false"))); // fixed prec scale
-	    t12.Insert( CegoFieldValue(BOOL_TYPE, Chain("false"))); // autoincrement
-	    t12.Insert( CegoFieldValue()); // local type name
-	    t12.Insert( CegoFieldValue(SMALLINT_TYPE, 0)); // minumum scale
-	    t12.Insert( CegoFieldValue(SMALLINT_TYPE, 0)); // maximum scale
-	    t12.Insert( CegoFieldValue(INT_TYPE, 0)); // SQL data type
-	    t12.Insert( CegoFieldValue(INT_TYPE, 0)); // SQL datetime sub
-	    t12.Insert( CegoFieldValue(INT_TYPE, 0)); // num prec radix
-	    o.rowOut(t12);
-
-	    o.tailOut();
 	    
+	    o.rowOut(t11);
+	    o.tailOut();	    
 	}
 	else
-	{
-	    
+	{	    
 	    CegoOutput o;
 	    if ( _pDbHandle )
-		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT);
-	    o.chainOut("Unknown JDBC command");
-	    
+		o.setDbHandle(_pDbHandle, NETMNG_MAXTUPLECOUNT, NETMNG_MAXBYTECOUNT);
+	    o.chainOut("Unknown JDBC command");	    
 	}
     }
 }
