@@ -294,19 +294,9 @@ Chain CegoJoinObject::toChain(int defTabSetId) const
     return s;
 }
 
-Chain CegoJoinObject::getFormatted() const
+Chain CegoJoinObject::getInfo() const
 {
-    Chain s;
-	
-    int maxAttrLen = 12;
-    CegoField* pF = _schema.First();
-    while (pF)
-    {
-	if (maxAttrLen < pF->getAttrName().length())
-	    maxAttrLen = pF->getAttrName().length();
-	pF = _schema.Next();
-    }
-
+    Chain s("NA");
     return s;
 }
 
@@ -334,19 +324,6 @@ ListT<Chain> CegoJoinObject::getSubAliasList() const
     return subList;
 }
 
-
-Chain CegoJoinObject::fill(const Chain& s, int num) const
-{
-    Chain fs = Chain();
-    while (num > 0)
-    {
-	fs = fs + s;
-	num--;
-    }
-
-    return fs;
-}
-
 CegoContentObject* CegoJoinObject::clone(bool isAttrRef)
 {
     CegoJoinObject *pClone = new CegoJoinObject( _joinType, 
