@@ -53,7 +53,7 @@ class CegoJoinObject : public CegoContentObject {
     Chain getId(CegoProcBlock *pBlock) const;
     Chain toChain(int defTabSetId) const;
     CegoContentObject* clone(bool isAttrRef = false);    
-    Chain getFormatted() const;
+    Chain getInfo() const;
 
     ListT<Chain> getSubAliasList() const;
     
@@ -62,8 +62,6 @@ class CegoJoinObject : public CegoContentObject {
 
  private:
 
-    Chain fill(const Chain& s, int num) const;
-
     JoinType _joinType;
     CegoContentObject *_pLeftObject;
     CegoContentObject *_pRightObject;
