@@ -6,7 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -33,7 +33,9 @@
 #include <string.h>
 #include <stdlib.h>
 
-CegoLogManager::CegoLogManager(const Chain& xmlDef, const Chain& logFile, const Chain& progName) : CegoFileHandler(logFile, progName), CegoXMLSpace(xmlDef)
+extern bool __fsyncOn;
+
+CegoLogManager::CegoLogManager(const Chain& xmlDef, const Chain& logFile, const Chain& progName) : CegoXMLSpace(xmlDef, logFile, progName)
 {
     for (int i=0;i< TABMNG_MAXTABSET;i++)
     {
@@ -42,6 +44,8 @@ CegoLogManager::CegoLogManager(const Chain& xmlDef, const Chain& logFile, const
 	_logActive[i] = false;
 	_pLogHandler[i] = 0;
 	_pNetHandler[i] = 0;
+	_logBuf[i] = 0;
+	_logBufLen[i] = 0;
     }
     _modId = getModId("CegoLogManager");
 }
@@ -55,29 +59,41 @@ CegoLogManager::~CegoLogManager()
 	    _pLog[i]->close();
 	    delete _pLog[i];
 	}
+	if ( _logBuf[i] )
+	    free(_logBuf[i]);
     }
 }
 
-void CegoLogManager::setLSN(int tabSetId, unsigned long long lsn)
+void CegoLogManager::setCurrentLSN(int tabSetId, unsigned long long lsn)
 {
     _lsn[tabSetId] = lsn;
 }
 
-unsigned long long CegoLogManager::getLSN(int tabSetId)
+unsigned long long CegoLogManager::getCurrentLSN(int tabSetId)
 {
     return _lsn[tabSetId];
 }
 
+unsigned long long CegoLogManager::nextLSN(int tabSetId)
+{
+    // we just increase, if log is active
+    if ( _logActive[tabSetId] == false )
+	return 0;
+
+    _lsn[tabSetId]++;
+    return _lsn[tabSetId];
+}
+
 Chain CegoLogManager::getArchiveLogName(const Chain& tableSet, unsigned long long lsn)
 {
     Chain lsnStr = Chain("000000000000") + Chain(lsn);
     Chain lsnFix = lsnStr.subChain(lsnStr.length() - 12, lsnStr.length());
-    return tableSet + "-" + lsnFix + ".dbf";
+    Chain branch = getTableSetBackupBranch(tableSet);
+    return tableSet + Chain(LOGMNG_ARCHLOGSEP) + branch + Chain(LOGMNG_ARCHLOGSEP) + lsnFix + LOGMNG_ARCHLOGSUFFIX;
 }
 
 void CegoLogManager::setLogFile(int tabSetId, const Chain& logFile, bool readOnly)
 {
-
     if (_pLog[tabSetId])
     {
 	_pLog[tabSetId]->close();
@@ -94,23 +110,22 @@ void CegoLogManager::setLogFile(int tabSetId, const Chain& logFile, bool readOnl
 	_pLog[tabSetId]->open(File::READWRITE);
 
     _logSize[tabSetId] = _pLog[tabSetId]->Size();    
-    int c = _pLog[tabSetId]->readByte((char*)&_logOffset[tabSetId], sizeof(int));	
+
+    _pLog[tabSetId]->readByte((char*)&_logOffset[tabSetId], sizeof(int));	
 
     _logPos[tabSetId] = sizeof(int);
     _logActive[tabSetId] = false;
 }
 
-void CegoLogManager::allocateLogConnection(int tabSetId, const Chain& tableSet, const Chain& logHost, int logPort)
+void CegoLogManager::allocateLogConnection(int tabSetId, const Chain& tableSet, const Chain& logHost, int logPort, const Chain& logUser, const Chain& logPwd)
 {
-
-    Net n ( NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN );
+    Net n ( NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN, NETMNG_MAXSENDLEN );
     
     log(_modId, Logger::NOTICE, Chain("Connecting to loghost ") + logHost + Chain(":") + Chain(logPort) + Chain(" ..."));    
 
-    try  {
-
-	_pNetHandler[tabSetId] = n.connect(logHost, logPort);
-        
+    try
+    {
+	_pNetHandler[tabSetId] = n.connect(logHost, logPort);        
     }
     catch ( Exception e )
     {
@@ -119,7 +134,7 @@ void CegoLogManager::allocateLogConnection(int tabSetId, const Chain& tableSet,
 	throw Exception(EXLOC, msg); 
     }
     _pLogHandler[tabSetId] = new CegoLogHandler(this, _pNetHandler[tabSetId]);
-    _pLogHandler[tabSetId]->requestLogSession(tableSet);
+    _pLogHandler[tabSetId]->requestLogSession(tableSet, logUser, logPwd);
 }
 
 bool CegoLogManager::hasLogConnection(int tabSetId)
@@ -133,10 +148,8 @@ bool CegoLogManager::hasLogConnection(int tabSetId)
 
 void CegoLogManager::releaseLogConnection(int tabSetId)
 {
-
     if ( _pLogHandler[tabSetId] )
-    { 
-	
+    { 	
 	try 
 	{
 	    _pLogHandler[tabSetId]->closeSession();
@@ -154,7 +167,6 @@ void CegoLogManager::releaseLogConnection(int tabSetId)
 
 void CegoLogManager::initLog(int tabSetId, int size)
 {
-    
     if (_pLog[tabSetId] == 0)
     {
 	Chain msg = "No logfile set up for tableset";
@@ -191,18 +203,13 @@ void CegoLogManager::resetLog(int tabSetId)
 {
     _logOffset[tabSetId] = sizeof(int);
     _pLog[tabSetId]->seek(0);
-    _pLog[tabSetId]->writeByte((char*)&_logOffset[tabSetId], sizeof(int));    
+    _pLog[tabSetId]->writeByte((char*)&_logOffset[tabSetId], sizeof(int));
+    if ( __fsyncOn )
+	_pLog[tabSetId]->flush();
 }
 
 void CegoLogManager::stopLog(int tabSetId)
 {
-    if (_pLog[tabSetId])
-    {
-	_pLog[tabSetId]->close();
-	delete _pLog[tabSetId];
-	_pLog[tabSetId] = 0;
-    }
-
     _logActive[tabSetId] = false;
 }
 
@@ -217,43 +224,64 @@ void CegoLogManager::startLog(int tabSetId)
     _logActive[tabSetId] = true;
 }
 
-CegoLogManager::LogResult CegoLogManager::logAction(int tabSetId, CegoLogRecord& logRec)
+bool CegoLogManager::isActive(int tabSetId)
 {
+    return _logActive[tabSetId];
+}
 
-    if ( _logActive[tabSetId] == false )
+CegoLogManager::LogResult CegoLogManager::logAction(int tabSetId, CegoLogRecord& logRec, bool flushLog)
+{
+    if ( _logActive[tabSetId] == false )	
 	return LOG_SUCCESS;
-
+    
     // cout << "Logging " <<  _lsn[tabSetId] << " " << logRec << endl;
 
-    logRec.setLSN(_lsn[tabSetId]);
-
+    if ( logRec.getLSN() == 0 )
+    {
+	logRec.setLSN(nextLSN(tabSetId));
+    }
+	
     Datetime ts;
-    logRec.setTS( ts.asInt());
+    logRec.setTS( ts.asLong());
     
     int len = logRec.getEncodingLength();
-	
-    char* pBuf = (char*)malloc(len);
+
+    // cout << "Logging record of len " << len << endl;
+
+    if ( _logBuf[tabSetId] == 0 )
+    {
+	_logBuf[tabSetId] = (char*)malloc(len);
+	_logBufLen[tabSetId] = len;
+    }
+    else
+    {
+	if ( _logBufLen[tabSetId] < len )
+	{
+	    // reallocate buffer
+	    free ( _logBuf[tabSetId] );
+	    _logBuf[tabSetId] = (char*)malloc(len);
+	    _logBufLen[tabSetId] = len;	    
+	}
+    }
         
-    logRec.encode(pBuf);
+    logRec.encode(_logBuf[tabSetId]);
 
     bool isFull = false;
     
     if ( _pLogHandler[tabSetId] )
     {
 #ifdef CGDEBUG
-	log(_modId, Logger::DEBUG, Chain("Sending log msg ( lsn=") + Chain(_lsn[tabSetId]) + Chain(", size=") + Chain(len) + Chain(")"));    
+	log(_modId, Logger::DEBUG, Chain("Sending log msg ( lsn=") + Chain(logRec.getLSN()) + Chain(", size=") + Chain(len) + Chain(")"));    
 #endif
-	if ( _pLogHandler[tabSetId]->sendLogEntry(pBuf, len) == false )
+	if ( _pLogHandler[tabSetId]->sendLogEntry(_logBuf[tabSetId], len) == false )
 	{
-	    free(pBuf);
 	    return LOG_ERROR;
 	}
     }
     else
     {
-
 #ifdef CGDEBUG
-	log(_modId, Logger::DEBUG, Chain("Logging local ( lsn=") + Chain(_lsn[tabSetId]) + Chain(", size=") + Chain(len) + Chain(")"));    
+	log(_modId, Logger::DEBUG, Chain("Logging local ( lsn=") + Chain(logRec.getLSN()) + Chain(", size=") + Chain(len) + Chain(")"));    
 #endif
 
 	if ( _logOffset[tabSetId] + len > _logSize[tabSetId] )
@@ -262,18 +290,21 @@ CegoLogManager::LogResult CegoLogManager::logAction(int tabSetId, CegoLogRecord&
 	    isFull=true;
 	}
         
-	_pLog[tabSetId]->writeByte((char*)&len, sizeof(int));       
-	_pLog[tabSetId]->writeByte(pBuf, len);		
+	_pLog[tabSetId]->writeByte((char*)&len, sizeof(int));
+	_pLog[tabSetId]->writeByte(_logBuf[tabSetId], len);
 	_logOffset[tabSetId] += len + sizeof(int);
 
 	_pLog[tabSetId]->seek(0);
 	_pLog[tabSetId]->writeByte((char*)&_logOffset[tabSetId], sizeof(int));
 	_pLog[tabSetId]->seek(_logOffset[tabSetId]);
+
+	if ( flushLog )
+	{
+	    if ( __fsyncOn )
+		_pLog[tabSetId]->flush();
+	}
     }
-
-    _lsn[tabSetId]++;
-
-    free(pBuf);
+    
     if ( isFull )
 	return LOG_FULL;
     else
@@ -282,7 +313,6 @@ CegoLogManager::LogResult CegoLogManager::logAction(int tabSetId, CegoLogRecord&
 
 unsigned long long CegoLogManager::getMinLSN(int tabSetId)
 {
-
     if (_pLog[tabSetId] == 0)
     {
 	Chain msg = "No logfile set up for tableset";
@@ -290,7 +320,7 @@ unsigned long long CegoLogManager::getMinLSN(int tabSetId)
     }
 
     _pLog[tabSetId]->seek(0); 
-    int c = _pLog[tabSetId]->readByte((char*)&_logOffset[tabSetId], sizeof(int));	
+    _pLog[tabSetId]->readByte((char*)&_logOffset[tabSetId], sizeof(int));	
     
     _logPos[tabSetId] = sizeof(int);
 
@@ -298,9 +328,8 @@ unsigned long long CegoLogManager::getMinLSN(int tabSetId)
 
     if ( _logPos[tabSetId] < _logOffset[tabSetId] )
     {
-
 	int len;
-	c = _pLog[tabSetId]->readByte((char*)&len, sizeof(int));	
+	_pLog[tabSetId]->readByte((char*)&len, sizeof(int));	
 
 	if ( len > LOGMNG_RECBUFSIZE )
 	{
@@ -322,10 +351,8 @@ unsigned long long CegoLogManager::getMinLSN(int tabSetId)
     return minlsn;
 }
 
-
 unsigned long long CegoLogManager::getMaxLSN(int tabSetId)
 {
-
     if (_pLog[tabSetId] == 0)
     {
 	Chain msg = "No logfile set up for tableset";
@@ -333,7 +360,7 @@ unsigned long long CegoLogManager::getMaxLSN(int tabSetId)
     }
 
     _pLog[tabSetId]->seek(0);    
-    int c = _pLog[tabSetId]->readByte((char*)&_logOffset[tabSetId], sizeof(int));	
+    _pLog[tabSetId]->readByte((char*)&_logOffset[tabSetId], sizeof(int));	
 
 #ifdef CGDEBUG
     log(_modId, Logger::DEBUG, Chain("Read logoffset ") + Chain(_logOffset[tabSetId]));
@@ -345,9 +372,8 @@ unsigned long long CegoLogManager::getMaxLSN(int tabSetId)
 
     while (_logPos[tabSetId] < _logOffset[tabSetId])
     {
-
 	int len;
-	c = _pLog[tabSetId]->readByte((char*)&len, sizeof(int));	
+	_pLog[tabSetId]->readByte((char*)&len, sizeof(int));	
 
 	if ( len > LOGMNG_RECBUFSIZE )
 	{
@@ -360,8 +386,9 @@ unsigned long long CegoLogManager::getMaxLSN(int tabSetId)
 	    
 	CegoLogRecord lr;
 	lr.decode(buf);
-	
-	maxlsn = lr.getLSN();
+
+	if ( lr.getLSN() > maxlsn )
+	    maxlsn = lr.getLSN();
 
 	_logPos[tabSetId] += len + sizeof(int);
 
@@ -378,7 +405,7 @@ void CegoLogManager::seekToStart(int tabSetId)
     }
 
     _pLog[tabSetId]->seek(0);    
-    int c = _pLog[tabSetId]->readByte((char*)&_logOffset[tabSetId], sizeof(int));    
+    _pLog[tabSetId]->readByte((char*)&_logOffset[tabSetId], sizeof(int));    
     _logPos[tabSetId] = sizeof(int);
 }
 
@@ -391,7 +418,7 @@ bool CegoLogManager::logSeek(int tabSetId, unsigned long long lsn)
     }
 
     _pLog[tabSetId]->seek(0);    
-    int c = _pLog[tabSetId]->readByte((char*)&_logOffset[tabSetId], sizeof(int));	
+    _pLog[tabSetId]->readByte((char*)&_logOffset[tabSetId], sizeof(int));	
     
     _logPos[tabSetId] = sizeof(int);
 
@@ -399,18 +426,16 @@ bool CegoLogManager::logSeek(int tabSetId, unsigned long long lsn)
 
     while (_logPos[tabSetId] < _logOffset[tabSetId])
     {
-
 	int len;
-	c = _pLog[tabSetId]->readByte((char*)&len, sizeof(int));	
-
+	_pLog[tabSetId]->readByte((char*)&len, sizeof(int));	
+	
 	if ( len > LOGMNG_RECBUFSIZE )
 	{
 	    Chain msg = "Log read buffer exceeded";
 	    throw Exception(EXLOC, msg);		    
 	}
 	char buf[LOGMNG_RECBUFSIZE];		
-	
-	
+		
 	_pLog[tabSetId]->readByte(buf, len);
 	    
 	CegoLogRecord lr;
@@ -431,7 +456,6 @@ bool CegoLogManager::logSeek(int tabSetId, unsigned long long lsn)
 	} 
 
 	_logPos[tabSetId] += len + sizeof(int);
-
     }
 
     if ( loglsn + 1 == lsn )
@@ -447,11 +471,10 @@ bool CegoLogManager::logSeek(int tabSetId, unsigned long long lsn)
 
 bool CegoLogManager::logRead(int tabSetId, CegoLogRecord& logRec)
 {
-
     if (_logPos[tabSetId] < _logOffset[tabSetId])
     {
 	int len;
-	int c = _pLog[tabSetId]->readByte((char*)&len, sizeof(int));	
+	_pLog[tabSetId]->readByte((char*)&len, sizeof(int));	
 
 	if ( len > LOGMNG_RECBUFSIZE )
 	{
@@ -473,10 +496,12 @@ bool CegoLogManager::logRead(int tabSetId, CegoLogRecord& logRec)
 }
 
 bool CegoLogManager::switchLogFile(int tabSetId)
-{    
+{   
+    if ( _logActive[tabSetId] == false )
+	return true;
+
     if ( _pLogHandler[tabSetId] == 0 )
     {
-
 	Chain tableSet = getTabSetName(tabSetId);
 	
 	ListT<Chain> lfList;
@@ -490,37 +515,41 @@ bool CegoLogManager::switchLogFile(int tabSetId)
 	
 	bool found = false;
 	while ( pLogFile && pStatus && found == false) 
-	{
-	    
+	{	    
 	    if ( *pStatus == Chain(XML_ACTIVE_VALUE))
-	    {
-	    	
+	    {   	
 		Chain *pNextLogFile = lfList.Next();
 		Chain *pNextLogStatus = statusList.Next();
+
 		if ( pNextLogFile == 0)
 		{
 		    pNextLogFile = lfList.First();
 		    pNextLogStatus = statusList.First();
 		}
-		
+			       		
 		if ( isArchiveMode(tabSetId) )
-		{	    
+		{
+		    // if next logfile is still occupied, we skip switch and return false
 		    if ( *pNextLogStatus == Chain(XML_OCCUPIED_VALUE))
 		    {
 			return false;
 		    }
+
+		    // first we have to switch log file to synchronize with CegoLogThreadPool::shiftRedoLogs()
+		    setLogFile(tabSetId, *pNextLogFile, false);		    
 		    setLogFileStatus(tableSet, *pLogFile, XML_OCCUPIED_VALUE); 
 		}
 		else
+		{
+		    // switch log file first also for non archiving mode
+		    setLogFile(tabSetId, *pNextLogFile, false);
 		    setLogFileStatus(tableSet, *pLogFile, XML_FREE_VALUE); 
-		
+		}
 		
 		setLogFileStatus(tableSet, *pNextLogFile, XML_ACTIVE_VALUE);
 				
 		log(_modId, Logger::NOTICE, Chain("Logfile switch to logfile ") + *pNextLogFile + Chain(" for tableSet ") +  tableSet);
-		
-		setLogFile(tabSetId, *pNextLogFile, false);
-		
+			     		
 		found = true;
 	    }
 	    else
@@ -529,7 +558,9 @@ bool CegoLogManager::switchLogFile(int tabSetId)
 		pStatus = statusList.Next();
 	    }	
 	}
+
 	doc2Xml();
+	
 	resetLog(tabSetId);
 	startLog(tabSetId);	
     }
