@@ -8,7 +8,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -31,7 +31,7 @@
 #include "CegoXMLSpace.h"
 #include "CegoDefs.h"
 
-class CegoLogManager : public CegoFileHandler, public CegoXMLSpace {
+class CegoLogManager : public CegoXMLSpace {
     
 public:
     
@@ -45,26 +45,21 @@ public:
     void startLog(int tabSetId);
     int getLogOffset(int tabSetId);
     void resetLog(int tabSetId);
-    
-    LogResult logAction(int tabSetId, CegoLogRecord& logRec);
-
-    void allocateLogConnection(int tabSetId, const Chain& tableSet, const Chain& logHost, int logPort);
+    bool isActive(int tabSetId);
+    LogResult logAction(int tabSetId, CegoLogRecord& logRec, bool flushLog = true);
+    void allocateLogConnection(int tabSetId, const Chain& tableSet, const Chain& logHost, int logPort, const Chain& logUser, const Chain& logPwd);
     bool hasLogConnection(int tabSetId);
     void releaseLogConnection(int tabSetId);
     void setLogFile(int tabSetId, const Chain& logFile, bool readOnly);
-
     unsigned long long getMinLSN(int tabSetId);
     unsigned long long getMaxLSN(int tabSetId);
-
     void seekToStart(int tabSetId);
     bool logSeek(int tabSetId, unsigned long long lsn);
     bool logRead(int tabSetId, CegoLogRecord& logRec);
-
-    void setLSN(int tabSetId, unsigned long long lsn);
-    unsigned long long getLSN(int tabSetId);
-
+    unsigned long long nextLSN(int tabSetId);
+    void setCurrentLSN(int tabSetId, unsigned long long lsn);
+    unsigned long long getCurrentLSN(int tabSetId);
     Chain getArchiveLogName(const Chain& tableSet, unsigned long long lsn);
-
     bool switchLogFile(int tabSetId);
     void setActiveLogFile(const Chain& tableSet);
     
@@ -75,18 +70,15 @@ private:
     
     File *_pLog[TABMNG_MAXTABSET];
     File *_pLFID[TABMNG_MAXTABSET];
-
     Chain _logFile[TABMNG_MAXTABSET];
     int _logSize[TABMNG_MAXTABSET];
     int _logOffset[TABMNG_MAXTABSET];
-    
+    char* _logBuf[TABMNG_MAXTABSET];
+    int _logBufLen[TABMNG_MAXTABSET];
     int _numGroup[TABMNG_MAXTABSET];
-	
     unsigned long long _lsn[TABMNG_MAXTABSET];
     int _logPos[TABMNG_MAXTABSET];
-
     bool _logActive[TABMNG_MAXTABSET];
-
     CegoLogHandler* _pLogHandler[TABMNG_MAXTABSET];
     NetHandler* _pNetHandler[TABMNG_MAXTABSET];
 
