@@ -23,6 +23,7 @@
 #include <lfcbase/GetLongOpt.h>
 #include <lfcbase/Chain.h>
 #include <lfcbase/Datetime.h>
+#include <lfcbase/Tokenizer.h>
 
 // CEGO INCLUDES
 #include "CegoDefs.h"
@@ -33,19 +34,30 @@
 #include "CegoKeyObject.h"
 #include "CegoViewObject.h"
 #include "CegoProcObject.h"
+#include "CegoBTreeObject.h"
+#include "CegoCheckObject.h"
+#include "CegoTriggerObject.h"
+#include "CegoTypeConverter.h"
 
 // POSIX INCLUDES
 #include <string.h>
 #include <stdlib.h>
+#include <stdio.h>
+
+#define PIPE_TIMEOUT 30 /* checking input pipe data timeout in ms */
 
 #define USAGE "Usage: cglog --log=<logfile>\n\
           [ --version  ] [ --help ]"
 
 extern char __lfcVersionString[];
 extern char __lfcxmlVersionString[];
+extern Chain __dateTimeFormat;
 
 int main(int argc, char **argv)
 {
+
+    __dateTimeFormat = Chain(DEFAULTDATETIMEFORMAT1);
+    
     try 
     {
 	GetLongOpt longOpt(argc, argv);
@@ -81,35 +93,44 @@ int main(int argc, char **argv)
 	    cout << CEGO_COPYRIGHT << endl;
 	    exit(0);
 	}
+
+	File *pLog = 0;
 	
 	if ( longOpt.isSet( Chain("log") ) == false )
 	{
-	    throw Exception(EXLOC, "Redo logfile file must be specified");
+	    pLog = new File(File::IN);
+	    if ( pLog->hasData(PIPE_TIMEOUT) == false )
+	    {
+		delete pLog;
+		pLog = 0;
+		throw Exception(EXLOC, "No log data");
+	    }	  
 	}
+	else
+	{	    
+	    Chain logFileName = longOpt.getOptValue("log");	    
+	    pLog = new File(logFileName);	
+	    pLog->open(File::READ);
+	}
+
 	
-	Chain logFileName = longOpt.getOptValue("log");
-	
-	File logFile(logFileName);
-	
-	logFile.open(File::READ);
-		
 	int offset;
-	int c = logFile.readByte((char*)&offset, sizeof(int));	
+	int c = pLog->readByte((char*)&offset, sizeof(int));	
 	
 	int pos = sizeof(int);
 	       
 	while (pos < offset)
 	{
 	    int len;
-	    c = logFile.readByte((char*)&len, sizeof(int));	
+	    c = pLog->readByte((char*)&len, sizeof(int));	
 	    
-	    cout << "Read len : " << len << endl;
+	    // cout << "Read len : " << len << endl;
 	    
 	    pos += len + sizeof(int);
 	    
 	    char* buf = new char[len];
 	    
-	    logFile.readByte(buf, len);
+	    pLog->readByte(buf, len);
 	    CegoLogRecord lr;
 	    
 	    lr.decode(buf);
@@ -118,27 +139,33 @@ int main(int argc, char **argv)
 
 	    Datetime dt( lr.getTS() );
 
+	    Chain logData;
+	    
 	    cout << "LSN=" << lr.getLSN() << ":";
 	    cout << "TS=" << dt.asChain() << ":";
 	    cout << "TID=" << lr.getTID() << ":";
 	    cout << "TAStep=" << lr.getTAStep() << ":";
 
-	    if ( lr.getObjName().length() > 0 )
-	    { 
-		cout << "ObjName=" << lr.getObjName() << ":";
-		cout << "ObjType=" << lr.getObjType() << ":";
-	    }
-
 	    cout << "LogAction=";
 
+	    if ( lr.getObjName().length() > 0 )
+	    {
+		logData += Chain("Name=") + lr.getObjName() + Chain("\n");
+		logData += Chain("Type=") + CEGO_OBJECT_MAP[(int)lr.getObjType()] + Chain("\n");
+	    }
+	    
 	    switch ( lr.getAction() )
 	    {
 	    case CegoLogRecord::LOGREC_CREATE:
+	    case CegoLogRecord::LOGREC_ALTER:
 	    {
-		cout << "CREATE:" << endl;
-		
-		cout << "--- Log Data Start ---" << endl;
+		if ( lr.getAction() == CegoLogRecord::LOGREC_CREATE )
+		    cout << "CREATE:" << endl;
+		else
+		    cout << "ALTER:" << endl;
 		
+		logData += Chain("--- Data Start ---\n");
+	
 		switch ( lr.getObjType() )
 		{
 		case CegoObject::TABLE:
@@ -149,50 +176,58 @@ int main(int argc, char **argv)
 		{
 		    CegoTableObject to;
 		    to.decode(lr.getData());
-		    cout << to.toChain(0);
+		    logData += to.getInfo();
 		    break;
 		}
 		case CegoObject::PBTREE:
 		case CegoObject::UBTREE:
 		case CegoObject::BTREE:
 		{
-		    // TODO
+		    CegoBTreeObject bo;
+		    bo.decode(lr.getData());
+		    logData += bo.getInfo();
 		    break;
 		}
 		case CegoObject::VIEW:
 		{
 		    CegoViewObject vo;
 		    vo.decode(lr.getData());
-		    cout << vo.toChain(0);
+		    logData += vo.getInfo();
 		    break;		    
 		}
 		case CegoObject::PROCEDURE:
 		{
 		    CegoProcObject po;
 		    po.decode(lr.getData());
-		    cout << po.toChain();
+		    logData += po.getInfo();
 		    break;		    
 		}
 		case CegoObject::FKEY:
 		{
 		    CegoKeyObject ko;
 		    ko.decode(lr.getData());
-		    cout << ko.toChain();
+		    logData += ko.getInfo();
 		    break;		    
 		}
 		case CegoObject::CHECK:
 		{
-		    // TODO
-		    break;
+		    CegoCheckObject co;
+		    co.decode(lr.getData());
+		    logData += co.getInfo();
+		    break;		    
 		}
 		case CegoObject::TRIGGER:
 		{
-		    // TODO
-		    break;
+		    CegoTriggerObject to;
+		    to.decode(lr.getData());
+		    logData += to.getInfo();
+		    break;		    
 		}
 		case CegoObject::ALIAS:
 		{
-		    // TODO
+		    CegoAliasObject ao;
+		    ao.decode(lr.getData());
+		    logData += ao.getInfo();
 		    break;
 		}
 		case CegoObject::RBSEG:
@@ -204,108 +239,37 @@ int main(int argc, char **argv)
 		}
 		}
 		
-		cout << "--- Log Data End   ---" << endl;
+		logData += Chain("--- Data End ---\n");
        
 		break;
 	    }
-	    case CegoLogRecord::LOGREC_ALTER:
-	    {
-		cout << "ALTER:" << endl;
 
-		cout << "--- Log Data Start ---" << endl;
-
-		switch ( lr.getObjType() )
-		{
-		case CegoObject::TABLE:
-		case CegoObject::SYSTEM:
-		case CegoObject::AVLTREE:
-		case CegoObject::PAVLTREE:
-		case CegoObject::UAVLTREE:
-		{
-		    CegoTableObject to;
-		    to.decode(lr.getData());
-		    cout << to.toChain(0);
-		    break;
-		}
-		case CegoObject::PBTREE:
-		case CegoObject::UBTREE:
-		case CegoObject::BTREE:
-		{
-		    // TODO
-		    break;
-		}
-		
-		case CegoObject::VIEW:
-		{
-		    CegoViewObject vo;
-		    vo.decode(lr.getData());
-		    cout << vo.toChain(0);
-		    break;		    
-		}
-		case CegoObject::PROCEDURE:
-		{
-		    CegoProcObject po;
-		    po.decode(lr.getData());
-		    cout << po.toChain();
-		    break;		    
-		}
-		case CegoObject::FKEY:
-		{
-		    CegoKeyObject ko;
-		    ko.decode(lr.getData());
-		    cout << ko.toChain();
-		    break;		    
-		}
-		case CegoObject::CHECK:
-		{
-		    // TODO
-		    break;
-		}
-		case CegoObject::TRIGGER:
-		{
-		    // TODO
-		    break;
-		}
-		case CegoObject::ALIAS:
-		{
-		    // TODO
-		    break;
-		}
-		case CegoObject::RBSEG:
-		case CegoObject::JOIN:
-		case CegoObject::UNDEFINED:
-		{
-		    // ignore
-		    break;
-		}
-		}		
-		cout << "--- Log Data End   ---" << endl;
-		break;
-	    }
 	    case CegoLogRecord::LOGREC_RENAME:
 	    {
 		cout << "RENAME:" << endl;
-		cout << "--- Log Data Start ---" << endl;
+		
+		logData += Chain("--- Data Start ---\n");
 		Chain newObjName(lr.getData(), lr.getDataLen());
-		cout << "NewName=" << newObjName << endl; 
-		cout << "--- Log Data End   ---" << endl;
+		logData += Chain("NewName=") + newObjName + Chain("\n"); 
+		logData += Chain("--- Data End ---\n");
+		break;
 	    }
 	    case CegoLogRecord::LOGREC_DROP:
 	    {
 		cout << "DROP:" << endl;
-		cout << "--- No log data ---" << endl;
+		logData += Chain("--- No data ---\n");
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_TRUNCATE:
 	    {
-		cout << "TRUNCATE:" << endl;	       
-		cout << "--- No log data ---" << endl;
+		cout << "TRUNCATE:" << endl;
+		logData += Chain("--- No data ---\n");
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_INSERT:
 	    {
 		cout << "INSERT:" << endl;	
-		cout << "--- Log Data Start ---" << endl;
+		logData += Chain("--- Data Start ---\n");
 		
 		unsigned long long tid;
 		unsigned long long tastep;
@@ -327,19 +291,19 @@ int main(int argc, char **argv)
 		{
 		    if ( pFV->getLength() > 0 )
 		    {
-			cout << *pFV << "|";
+			logData += pFV->valAsChain(false) + Chain("|");
 		    }
 		    
 		    // cout << "Field " << pF->getId() << " Len=" << pF->getFieldValue().getLength() << endl;
 		    pFV = fvl.Next();
 		}
-		cout << endl;
+		logData += Chain("\n");
 		if ( blobList.Size() > 0 )
 		{
 		    CegoBlob *pBlob = blobList.First();
 		    while ( pBlob )
 		    {
-			cout << "Blob [ ... ] " << pBlob->getSize() << " bytes" << endl;
+			logData += Chain("Blob [ ... ] ") + Chain(pBlob->getSize()) + Chain(" bytes\n");
 			pBlob = blobList.Next();
 		    }
 		}
@@ -348,43 +312,41 @@ int main(int argc, char **argv)
 		    CegoClob *pClob = clobList.First();
 		    while ( pClob )
 		    {
-			cout << "Clob [" << pClob->getHead() << "] " << pClob->getSize() << " bytes" << endl;
+			logData += Chain("Clob [") +  pClob->getHead() + Chain("] ") + Chain(pClob->getSize()) + Chain(" bytes\n");
 			pClob = clobList.Next();
 		    }
 		}
 
-		cout << "--- Log Data End   ---" << endl;
+		logData += Chain("--- Data End ---\n");
 		
-		// cout << lr << endl;;
-
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_DELETE:
 	    {
 		cout << "DELETE:" << endl;
 		
-		cout << "--- Log Data Start ---" << endl;	
+		logData += Chain("--- Log Data Start ---\n");
 
 		CegoPredicate *pPred = 0;
 		Chain tableAlias;
 
 		CegoQueryHelper::decodeDelRec(tableAlias, pPred, lr.getData(), lr.getDataLen(), 0, 0);
 
-		cout << " TableAlias=" << tableAlias << endl;
+		logData += Chain("TableAlias=") + tableAlias + Chain("\n");
 		
 		if ( pPred )
 		{
-		    cout << "where " << pPred->toChain(0) << endl;
+		    logData += Chain("where ") + pPred->toChain(0) + Chain("\n");
 		}
 		
-		cout << "--- Log Data End   ---" << endl;
+		logData += Chain("--- Log Data End ---\n");
 		
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_UPDATE:
 	    {
 		cout << "UPDATE:" << endl;		
-		cout << "--- Log Data Start ---" << endl;	
+		logData += Chain("--- Data Start ---\n");	
 		Chain tableAlias;
 		CegoPredicate *pPred = 0;
 		ListT<CegoField> updList;
@@ -398,82 +360,108 @@ int main(int argc, char **argv)
 					      returnOnFirst,
 					      lr.getData(), lr.getDataLen(), 0, 0);
 				
-		cout << "TableAlias=" << tableAlias << endl;
+		logData += Chain("TableAlias=") + tableAlias + Chain("\n");
 		CegoField *pF = updList.First();
 		CegoExpr **pExpr = exprList.First();
 		while ( pF && pExpr )
 		{
-		    cout << "Set " << pF->getAttrName() << " = " << (*pExpr)->toChain(0) << endl;
+		    logData += Chain("set ") + pF->getAttrName() + Chain("=") +  (*pExpr)->toChain(0) + Chain("\n");
 		    pF = updList.Next();
 		    pExpr = exprList.Next();
 		}
 		if ( pPred )
-		    cout << "where " << pPred->toChain(0) << endl;
-		cout << "--- Log Data End   ---" << endl;
+		    logData += Chain("where ") + pPred->toChain(0) + Chain("\n");
+		logData += Chain("--- Data End ---\n");
 		
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_BEGIN:
 	    {
 		cout << "BEGIN:" << endl;
-		cout << "--- No log data ---" << endl;
+		logData += Chain("--- No data ---\n");
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_COMMIT:
 	    {
 		cout << "COMMIT:" << endl;
-		cout << "TID=" << lr.getTID() << ":";
+		logData += Chain("--- Data Start ---\n");	
+		logData += Chain("TID=") + Chain(lr.getTID()) + Chain("\n");
+		logData += Chain("--- Data End ---\n");
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_ABORT:
 	    {
 		cout << "ABORT:" << endl;
-		cout << "TID=" << lr.getTID() << endl;
+		logData += Chain("--- Data Start ---\n");	
+		logData += Chain("TID=") + Chain(lr.getTID()) + Chain("\n");
+		logData += Chain("--- Data End ---\n");
 		break;
 	    }	    
 	    case CegoLogRecord::LOGREC_SYNC:
 	    {
 		cout << "SYNC:" << endl;
-		cout << "--- No log data ---" << endl;
+		logData += Chain("--- No data ---\n");
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_BUPAGE:
 	    {
 		cout << "BUPAGE:" << endl;
-		cout << "[... page data ...]" << endl;
+		logData += Chain("--- Data Start ---\n");
+		logData += Chain("[... page data ...]\n");
+		logData += Chain("--- Data End ---\n");;
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_BUFBM:
 	    {
 		cout << "BUFBM:" << endl;
-		cout << "[ ... file bitmap ...]" << endl;
+		logData += Chain("--- Data Start ---\n");
+		logData += Chain("[... file bitmap ...]\n");
+		logData += Chain("--- Data End ---\n");;
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_BUFIN:
 	    {
 		cout << "BUFIN:" << endl;
-		cout << "--- No log data ---" << endl;
+		logData += Chain("--- No data ---\n");
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_ADDCOUNTER:
 	    {
 		cout << "ADDCOUNTER:" << endl;
+		logData += Chain("--- Data Start ---\n");
 		Chain counterName(lr.getData(), lr.getDataLen());		
-		cout << "Counter="<< counterName << endl;
+		logData += Chain("Counter=") + counterName + Chain("\n");
+		cout << "--- Data End ---" << endl;
 		break;
 	    }
 	    case CegoLogRecord::LOGREC_DELCOUNTER:
 	    {
 		cout << "DELCOUNTER:" << endl;
-		Chain counterName(lr.getData(), lr.getDataLen());		
-		cout << "Counter="<< counterName << endl;
+		logData += Chain("--- Data Start ---\n");
+		Chain counterName(lr.getData(), lr.getDataLen());
+		logData += Chain("Counter=") + counterName + Chain("\n");
+
+		logData += Chain("--- Data End ---\n");
 		break;
 	    }
 	    default:
-		cout << " -- Unknown logaction " << lr.getAction() << endl;
+		cout << "UNKNOWN:" << endl;
+		logData += Chain(" -- Unknown logaction ---\n");
 		break;   		             	
 	    }
+
+	    Tokenizer logTok(logData, "\n");
+
+	    Chain logLine;
+	    while ( logTok.nextToken(logLine) )
+	    {
+		cout << "   " << logLine << endl;
+	    }
 	}
+
+	pLog->close();
+	delete pLog;
+	
     }
     catch ( Exception e)
     {
