@@ -8,7 +8,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -24,6 +24,7 @@
 #include <lfcbase/Chain.h>
 
 // CEGO INCLUDES
+#include "CegoDefs.h"
 #include "CegoObject.h"
 
 class CegoLogRecord {
@@ -36,9 +37,7 @@ public:
 		     LOGREC_RENAME, 
 		     LOGREC_INSERT,
 		     LOGREC_DELETE,
-		     LOGREC_DELETE_COMMIT,
 		     LOGREC_UPDATE, 
-		     LOGREC_UPDATE_COMMIT, 
 		     LOGREC_BEGIN, 
 		     LOGREC_COMMIT, 
 		     LOGREC_ABORT, 
@@ -47,73 +46,59 @@ public:
 		     LOGREC_BUFBM, 
 		     LOGREC_BUFIN,
 		     LOGREC_ADDCOUNTER,
-		     LOGREC_DELCOUNTER };
+		     LOGREC_DELCOUNTER,
+    		     LOGREC_TRUNCATE };
 
     CegoLogRecord();
     ~CegoLogRecord();
 
-    // log sequence number
     unsigned long long getLSN() const;
     void setLSN(unsigned long long lsn);
-
-    // log timestamp
-    int getTS() const;
-    void setTS(int ts);
-
-    // tid
+    long long getTS() const;
+    void setTS(long long ts);
     unsigned long long getTID() const;
     void setTID(unsigned long long tid);
-
+    unsigned long long getTAStep() const;
+    void setTAStep(unsigned long long tastep);
     void setObjectInfo(const Chain& objName, CegoObject::ObjectType type);
-
     const Chain& getObjName() const;
     CegoObject::ObjectType getObjType() const;
-
     LogAction getAction() const;
     void setAction(LogAction a);
-
     char* getData() const;
     void setData(char* data);
-
     int getDataLen() const;
     void setDataLen(int l);
-
     int getFileId() const;
     void setFileId(int fileId);
-    int getPageId() const;
-    void setPageId(int pageId);
-    
+    PageIdType getPageId() const;
+    void setPageId(PageIdType pageId);
     int getEncodingLength() const;
-
     void encode(void* p) const;
     void decode(void* p);
     void decodeLSN(void* p);
     void decodeLogAction(void* p);
-
     void clean();
-
-    CegoLogRecord& operator = ( const CegoLogRecord& d);
-    
-    friend ostream& operator << (ostream& s, const CegoLogRecord& logRecord);
-
     Chain toChain() const;
+    
+    CegoLogRecord& operator = ( const CegoLogRecord& d);    
+    friend ostream& operator << (ostream& s, const CegoLogRecord& logRecord);
 
 private:
     
     unsigned long long _lsn;
     unsigned long long _tid;
+    unsigned long long _tastep;
     Chain _objName;
     CegoObject::ObjectType _objType;
-
     enum LogAction _logAction;
     int _dataLen;
     char* _data;
     bool _encObj;
-
-    // for page logging 
+    // for file BM logging
     int _fileId;
-    int _pageId;
-
-    int _ts;
+    // for page logging 
+    PageIdType _pageId;
+    long long _ts;
 };
 #endif
