@@ -6,7 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke
 //               
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -56,20 +56,20 @@ void* CegoLogThread::job(void* arg)
     _pTim = new NanoTimer();
     
     _pPool->setTid(_idx, getTid());
+
+    int queueDelay = _pDBMng->getQueueDelay();
     
     while ( ! _pPool->isTerminated() )
     {	
 	try
-	{	    
-
+	{
 	    _pTim->reset();
 	    _pTim->start();
 
 	    _pRequest = _pPool->nextRequest();
 	    
 	    if ( _pRequest ) 
-	    {
-		
+	    {		
 		_pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(": Serving service request"));
 		_pPool->setState(_idx, CegoLogThreadPool::CONNECTED);
 
@@ -77,8 +77,8 @@ void* CegoLogThread::job(void* arg)
 
 		CegoLogHandler *pLH = 0;
 
-		try {
-
+		try
+		{
 		    pLH = new CegoLogHandler(_pDBMng, _pRequest);		    
 		    serveSession(pLH);
 		    delete pLH;
@@ -88,7 +88,6 @@ void* CegoLogThread::job(void* arg)
 		    if ( pLH )
 			delete pLH;
 
-
 		    Chain msg;
 		    Chain module;
 		    int line;
@@ -100,7 +99,6 @@ void* CegoLogThread::job(void* arg)
 		    }
 
 		    _pDBMng->log(_modId, Logger::LOGERR, Chain("Thread ") + Chain(_idx) + Chain(": Log session aborted : ") + exep);
-
 		}
 
 		_pDBMng->decreaseActiveLogThread();
@@ -112,12 +110,11 @@ void* CegoLogThread::job(void* arg)
 	    else
 	    {
 		Sleeper s;
-		s.nanoSleep(NETMNG_QUEUE_DELAY);
+		s.microSleep(queueDelay);
 	    }
 
 	    _pTim->stop();
-	    _pPool->addThreadIdle(_idx, _pTim->getSum());
-	
+	    _pPool->addThreadIdle(_idx, _pTim->getSum());       
 	}
 	catch ( Exception e)
 	{
@@ -131,13 +128,21 @@ void* CegoLogThread::job(void* arg)
 
 void CegoLogThread::serveSession(CegoLogHandler *pLH)
 {   
-
     _pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(": serving session"));
     
     Chain tableSet;
-
-    if ( pLH->acceptLogSession(tableSet) )
+    Chain logUser;
+    Chain logPwd;
+    
+    if ( pLH->acceptLogSession(tableSet, logUser, logPwd) )
     {
+	Chain msg;
+	bool isTrace;
+	if ( _pDBMng->checkUser(logUser, logPwd, msg, isTrace) == false )
+	{
+	    throw Exception(EXLOC, msg); 
+	}
+
 	char* logEntry;
 	int len;
 	int logCount=0;
@@ -147,8 +152,7 @@ void CegoLogThread::serveSession(CegoLogHandler *pLH)
 	int offset=0;
 
 	_pPool->setState(_idx, CegoLogThreadPool::BUSY);
-	
-       
+	      
 	Chain dbHost;
 	_pDBMng->getDBHost(dbHost);
 	_pDBMng->setSecondary(tableSet, dbHost);
@@ -160,7 +164,6 @@ void CegoLogThread::serveSession(CegoLogHandler *pLH)
 
 	while ( pLH->receiveLogEntry(logEntry, len) )
 	{
-
 	    CegoLogRecord lr;
 	    lr.decodeLSN(logEntry);
 	    lr.decodeLogAction(logEntry);
@@ -168,8 +171,7 @@ void CegoLogThread::serveSession(CegoLogHandler *pLH)
 	    _pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(": receiving lsn ") + Chain(lr.getLSN()));
 
 	    if ( logCount % LOGMNG_MAXLOGCOUNT == 0 || lr.getAction() == CegoLogRecord::LOGREC_SYNC )
-	    {
-		
+	    {		
 		_pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(": switching logfiles on lsn ") + Chain(lr.getLSN()));
 		if ( pLogFile )
 		{
@@ -213,7 +215,6 @@ void CegoLogThread::serveSession(CegoLogHandler *pLH)
 	    {
 		pLH->sendAck();
 	    }
-
 	    
 	    pLogFile->writeByte((char*)&len, sizeof(int));
 	    pLogFile->writeByte(logEntry, len);
@@ -221,8 +222,6 @@ void CegoLogThread::serveSession(CegoLogHandler *pLH)
 	    offset = offset + len + sizeof(int);
 
 	    _pPool->incNumRequest(_idx);
-
-	    // writing logentry
 	}
 	
 	_pDBMng->log(_modId, Logger::DEBUG, Chain("Thread ") + Chain(_idx) + Chain(": Closing logfile ") + pLogFile->getFileName());
@@ -240,8 +239,7 @@ void CegoLogThread::serveSession(CegoLogHandler *pLH)
 	_pTim->reset();
 	_pTim->start();
 	
-	_pPool->setState(_idx, CegoLogThreadPool::CONNECTED);
-	
+	_pPool->setState(_idx, CegoLogThreadPool::CONNECTED);	
     }
 
     _pTim->stop();
