@@ -6,7 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke
 //               
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -25,11 +25,13 @@
 #include <lfcbase/Sleeper.h>
 #include <lfcbase/ThreadLock.h>
 #include <lfcxml/XMLSuite.h>
+
 // CEGO INCLUDES
 #include "CegoDefs.h"
 #include "CegoXMLdef.h"
 #include "CegoLogThreadPool.h"
 #include "CegoLogRecord.h"
+
 // POSIX INCLUDES
 #include <string.h>
 #include <stdlib.h>
@@ -81,8 +83,7 @@ CegoLogThreadPool::CegoLogThreadPool(int poolLimit, CegoDatabaseManager *pDBMng)
 
     int i=0;
     while ( i < _poolLimit )
-    {
-	
+    {	
 	_threadState[i] = READY;
 	_threadList[i]= new CegoLogThread(this, pDBMng);
 	_threadId[i]=i;
@@ -181,7 +182,6 @@ void CegoLogThreadPool::setTid(int i, unsigned long long tid)
 
 void* CegoLogThreadPool::job(void* arg)
 {
-
     // special mode for batch import 
     if ( _poolLimit == 0 )
     {
@@ -200,31 +200,33 @@ void* CegoLogThreadPool::job(void* arg)
 		_pDBMng->log(_modId, Logger::LOGERR, Chain("Log thread failed : ") + msg); 
 		// _terminated=true;
 	    }
-
 	}
-	_joined=true;	
-
+	_joined=true;
     }
     else
     {
-	try {
-
+	try
+	{
 	    NanoTimer tim;
 	    
-	    Net net ( NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN );
-	    
-	    net.serve(_logHostName, _logPortNo);
+	    Net net ( NETMNG_MSG_BUFLEN, NETMNG_SIZEBUFLEN, NETMNG_MAXSENDLEN );
 
+	    if ( _logHostName != Chain() )
+		net.serve(_logHostName, _logPortNo);
+	    else
+		net.serve6(_logPortNo);
+	    
 	    unsigned long long usedTime[THRMNG_NUMLOADSAMPLE];
 	    for ( int i = 0; i < THRMNG_NUMLOADSAMPLE; i++ )
 	    {
 		usedTime[i] = 0;
 	    }
 
+	    int selectTimeout = _pDBMng->getSelectTimeout();
+	    int queueDelay = _pDBMng->getQueueDelay();
 	    
 	    while ( ! _terminated ) 
 	    {
-
 		usedTime[_samplePos] = 0;
 		
 		tim.reset();
@@ -240,12 +242,11 @@ void* CegoLogThreadPool::job(void* arg)
 		if ( noReq )
 		    lockQueue();
 		
-		NetHandler* pHandle;
+		NetHandler* pHandle = 0;
 		
-		try {
-		    
-		    pHandle = net.nextRequest( NETMNG_SELECT_TIMEOUT );
-		    
+		try
+		{		    
+		    pHandle = net.nextRequest( selectTimeout );		    
 		}
 		catch ( Exception e )
 		{
@@ -259,18 +260,24 @@ void* CegoLogThreadPool::job(void* arg)
 		    unlockQueue();
 
 		    Sleeper s;
-		    s.nanoSleep(NETMNG_QUEUE_DELAY);
-
+		    s.microSleep(queueDelay);
 		}
 		
 		if ( pHandle )
-		{
-		    
-		    // pHandle->setSendTimeout(NETMNG_SEND_TIMEOUT);
-		    // pHandle->setRecvTimeout(NETMNG_RECV_TIMEOUT);
-		    
+		{		    
+		    _pDBMng->log(_modId, Logger::NOTICE, Chain("Connection request from <") + pHandle->getSource() + Chain(">"));
 		    lockQueue();
-		    _requestQueue.Insert(pHandle);
+		    
+		    if ( _requestQueue.Size() < NETMNG_MAXQUEUELEN )
+		    {
+			_requestQueue.Insert(pHandle);
+		    }
+		    else
+		    {
+			delete pHandle;
+			_pDBMng->log(_modId, Logger::NOTICE, Chain("Rejected incoming request since connection queue is full ( ") +  Chain(NETMNG_MAXQUEUELEN) + Chain(" max )"));
+		    }
+		    		    		    		   
 		    unlockQueue();
 		}
 		
@@ -284,7 +291,6 @@ void* CegoLogThreadPool::job(void* arg)
 		    Chain msg;
 		    e.pop(msg);
 		    _pDBMng->log(_modId, Logger::LOGERR, Chain("Log thread failed : ") + msg); 
-		    // _terminated=true;
 		}
 
 		tim.stop();
@@ -295,8 +301,6 @@ void* CegoLogThreadPool::job(void* arg)
 		
 		// calculate load
 		int i=0;
-		// cout << "___" << endl;
-		// cout << "UsedTime=" << usedTime << endl;
 		while ( i < _poolLimit )
 		{
 		    unsigned long long ut=0;
@@ -394,23 +398,20 @@ void CegoLogThreadPool::getLockStat(Chain& lockName, int& lockCount, unsigned lo
 	sumRdDelay = queueLock.sumReadDelay() / LCKMNG_DELRES;
     if ( queueLock.numWriteLock() > 0 )
 	sumWrDelay = queueLock.sumWriteDelay() / LCKMNG_DELRES;
-
 }
 
 void CegoLogThreadPool::shiftRedoLogs()
-{
-
+{    
     ListT<Chain> tsList;
 
     Chain dbHost;
     _pDBMng->getDBHost(dbHost);
-    _pDBMng->getActiveTableSet(dbHost, tsList); 
+    _pDBMng->getActiveTableSet(dbHost, tsList, true, true); 
 
     Chain *pActiveTS = tsList.First();
 
     while ( pActiveTS )
     {
-
 	int tabSetId = _pDBMng->getTabSetId(*pActiveTS);
 
 	ListT<Chain> archPathList;
@@ -444,7 +445,6 @@ void CegoLogThreadPool::shiftRedoLogs()
 
 void CegoLogThreadPool::copyLog(const Chain& tableSet, const Chain& logFileName, const Chain& archLogPath)
 {
-
     File logFile(logFileName);
     
     logFile.open(File::READ);
@@ -455,9 +455,7 @@ void CegoLogThreadPool::copyLog(const Chain& tableSet, const Chain& logFileName,
     int pos = sizeof(int);
 
     if ( pos < offset)
-    {
-	// logFile.seek(sizeof(int));
-	
+    {	
 	int len;
 	c = logFile.readByte((char*)&len, sizeof(int));	
 	
@@ -469,7 +467,6 @@ void CegoLogThreadPool::copyLog(const Chain& tableSet, const Chain& logFileName,
 	logRec.decode(buf);    
 	delete[] buf;
 	logFile.close();
-
 	
 	Chain archLogFile = _pDBMng->getArchiveLogName(tableSet, logRec.getLSN());
 
@@ -489,8 +486,7 @@ void CegoLogThreadPool::copyLog(const Chain& tableSet, const Chain& logFileName,
 	archFile.trunc(offset);
 	archFile.close();
 	
-	archFile.rename(archFileName);
-	
+	archFile.rename(archFileName);	
     }
     return;
 }
