@@ -8,7 +8,7 @@
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -22,21 +22,22 @@
 
 // LFC INCLUDES
 #include <lfcbase/ListT.h>
+
 // CEGO INCLUDE
 #include "CegoField.h"
 
 class CegoOrderNode {
 
- public:
+public:
     
     enum Ordering { ASC, DESC };
     
     CegoOrderNode();
-    CegoOrderNode(ListT<CegoFieldValue>& key, ListT<CegoField>& tuple, ListT<Ordering>* pOrdering);
+    CegoOrderNode(ListT<CegoFieldValue>& key, ListT<CegoFieldValue>& tuple, ListT<Ordering>* pOrdering);
     ~CegoOrderNode();
     
     ListT<CegoFieldValue>& getKey();
-    ListT<CegoField>& getTuple();
+    ListT<CegoFieldValue>& getTuple();
 
     CegoOrderNode& operator = ( const CegoOrderNode& n);
     bool operator == ( const CegoOrderNode& n);
@@ -47,8 +48,6 @@ class CegoOrderNode {
     
     ListT<Ordering>* _pOrdering;
     ListT<CegoFieldValue> _key;
-    ListT<CegoField> _tuple;
-    
+    ListT<CegoFieldValue> _tuple;    
 };
-
 #endif
