@@ -1,12 +1,12 @@
 ///////////////////////////////////////////////////////////////////////////////
 //                                                         
 // CegoOrderSpace.cc
-// ------------------
+// -----------------
 // Cego order space implementation
 //     
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -20,13 +20,11 @@
 
 // LFC INCLUDES
 #include <lfcbase/Exception.h>
+
 // CEGO INCLUDES
 #include "CegoOrderSpace.h"
-#include "CegoDataType.h"
-#include "CegoTableManager.h"
-
-#include <string.h>
-#include <stdlib.h>
+// #include "CegoDataType.h"
+// #include "CegoTableManager.h"
 
 CegoOrderSpace::CegoOrderSpace()
 {
@@ -39,13 +37,14 @@ CegoOrderSpace::~CegoOrderSpace()
 	delete _pAVL;
 }
 
-void CegoOrderSpace::initOrderSpace(ListT<CegoExpr*>* pOrderList, ListT<CegoOrderNode::Ordering>* pOrderOptList, long maxOrderSize)
+void CegoOrderSpace::initOrderSpace(ListT<CegoExpr*>* pOrderList, ListT<CegoOrderNode::Ordering>* pOrderOptList, unsigned long long maxOrderSize)
 {
+    _schemaSet = false;
     _pOrderList = pOrderList;
     _pOrderOptList = pOrderOptList;
     _maxOrderSize = maxOrderSize;
     _orderSize = 0;
-
+    
     CegoExpr **pExpr = pOrderList->First();
     int id=1;
     while ( pExpr )
@@ -89,29 +88,34 @@ void CegoOrderSpace::initOrderSpace(ListT<CegoExpr*>* pOrderList, ListT<CegoOrde
 
 void CegoOrderSpace::insertTuple(ListT<CegoField>& orderKey, ListT<CegoField>& orderTuple)
 {    
+    if ( _schemaSet == false )
+    {
+	_selectSchema = orderTuple;
+	_schemaSet = true;
+    }
+    
     CegoField* pKF = orderKey.First();
     CegoField* pOF = _orderSchema.First();
     while (pKF && pOF )
-    {        
+    {
         pKF->setId(pOF->getId());
         pKF = orderKey.Next();
         pOF = _orderSchema.Next();        
     }
-
-    ListT<CegoField> localTuple;
     
-    int id=1;
+    ListT<CegoFieldValue> localTuple;
+
+    int orderEntryLen = 0;
+    
     CegoField *pF = orderTuple.First();
     while (pF)
     {
-	CegoField f = *pF;
-	f.setId(id);
-	f.setValue(pF->getValue().getLocalCopy());
-	localTuple.Insert(f);
+	CegoFieldValue fv = pF->getValue().getLocalCopy();
+	localTuple.Insert(fv);
 	
-	// pF->setId(id);
+	// calculate entry size
+	orderEntryLen += fv.usedMemory(); 
 	
-	id++;
 	pF = orderTuple.Next();
     }
 
@@ -121,20 +125,25 @@ void CegoOrderSpace::insertTuple(ListT<CegoField>& orderKey, ListT<CegoField>& o
     while ( pExpr )
     {	
 	setAggregationValue(*pExpr, orderKey);
-
-	(*pExpr)->setFieldListArray(&orderKey);
 	
 	(*pExpr)->clearAttrCache();
-	
-	CegoFieldValue fv = (*pExpr)->evalFieldValue();
+
+	ListT<CegoField>* fl[2];
+	fl[0] = &orderKey;
+	fl[1] = 0;
+
+	CegoFieldValue fv = (*pExpr)->evalFieldValue(fl, 0).getLocalCopy();
 	
 	localKey.Insert(fv);
-	
+
+	// calculate entry size
+	orderEntryLen += fv.usedMemory();
+       	
 	pExpr = _pOrderList->Next();
     }
     
     CegoOrderNode n(localKey, localTuple, _pOrderOptList);
-    _orderSize += sizeof(n);
+    _orderSize += orderEntryLen;
 
     if ( _orderSize > _maxOrderSize )
     {
@@ -152,11 +161,11 @@ void CegoOrderSpace::resetOrderSpace()
 
 CegoOrderCursor* CegoOrderSpace::getCursor()
 {
-    CegoOrderCursor *pOC = new CegoOrderCursor(_pAVL);
+    CegoOrderCursor *pOC = new CegoOrderCursor(_pAVL, _selectSchema);
     return pOC;
 }
 
-long CegoOrderSpace::numAllocated() const
+unsigned long long CegoOrderSpace::numAllocated() const
 {
     return _orderSize;
 }
