@@ -3,12 +3,12 @@
 ///////////////////////////////////////////////////////////////////////////////
 //                                                         
 // CegoOrderSpace.h
-// -----------------
+// ----------------
 // Cego order space class definition
 //     
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -22,6 +22,7 @@
 
 // LFC INCLUDES
 #include <lfcbase/Chain.h>
+
 // CEGO INCLUDES
 #include "CegoDefs.h"
 #include "CegoAttrDesc.h"
@@ -30,7 +31,7 @@
 #include "CegoQueryHelper.h"
 
 // forward declaration
-class CegoTableManager;
+// class CegoTableManager;
 
 class CegoOrderSpace {
     
@@ -39,27 +40,23 @@ class CegoOrderSpace {
     CegoOrderSpace();
     ~CegoOrderSpace();
        
-    void initOrderSpace(ListT<CegoExpr*> *pOrderList, ListT<CegoOrderNode::Ordering> *pOrderOptList, long maxOrderSize);
+    void initOrderSpace(ListT<CegoExpr*> *pOrderList, ListT<CegoOrderNode::Ordering> *pOrderOptList, unsigned long long maxOrderSize);
     void insertTuple(ListT<CegoField>& orderKey, ListT<CegoField>& orderTuple);
-
     void resetOrderSpace();
-
     CegoOrderCursor* getCursor();
-
-    long numAllocated() const;
+    unsigned long long numAllocated() const;
  
  private:
 
-    AVLTreeT<CegoOrderNode>* _pAVL;
-
     void setAggregationValue(CegoExpr *pExpr, ListT<CegoField>& fl);
-
     ListT<CegoExpr*>* _pOrderList;
     ListT<CegoOrderNode::Ordering>* _pOrderOptList;
+
+    AVLTreeT<CegoOrderNode>* _pAVL;
+    bool _schemaSet;
+    ListT<CegoField> _selectSchema;
     ListT<CegoField> _orderSchema;
-
-    long _maxOrderSize;
-    long _orderSize;
-
+    unsigned long long _maxOrderSize;
+    unsigned long long _orderSize;
 };
 #endif
