@@ -1,7 +1,7 @@
 ///////////////////////////////////////////////////////////////////////////////
 //                                                         
 // CegoPredicate-.cc
-// ---------------
+// -----------------
 // Cego predicate structure class definition
 //                                                 
 // Design and Implementation by Bjoern Lemke               
@@ -70,7 +70,7 @@ CegoPredicate::CegoPredicate(CegoExpr *pExpr1, CegoExpr* pExpr2, CegoExpr* pExpr
     _pMatcher = 0;
 }
 
-CegoPredicate::CegoPredicate(CegoExpr *pExpr, const Chain& pattern, bool isNegated)
+CegoPredicate::CegoPredicate(CegoExpr *pExpr, const Chain& pattern, bool isNegated, bool isCaseSensitive)
 {
     _pExpr1 = pExpr;
     _pExpr2 = 0;
@@ -78,11 +78,15 @@ CegoPredicate::CegoPredicate(CegoExpr *pExpr, const Chain& pattern, bool isNegat
     _pattern = pattern;
     _pMatcher = 0;
         
-    if ( isNegated )
-	_mode = CegoPredicate::ISNOTLIKE;
-    else
+    if ( ! isNegated && isCaseSensitive)
 	_mode = CegoPredicate::ISLIKE;
-
+    else if ( ! isNegated && isCaseSensitive)
+	_mode = CegoPredicate::ISNOTLIKE;
+    else if ( ! isNegated && ! isCaseSensitive)
+	_mode = CegoPredicate::ISNCLIKE;
+    else if ( isNegated && ! isCaseSensitive)
+	_mode = CegoPredicate::ISNOTNCLIKE;
+    
     _isChecked = false;
     _pSelect = 0;
     _pC = 0;
@@ -259,13 +263,14 @@ void CegoPredicate::setTabSetId(int tabSetId)
     
 void CegoPredicate::getPlanList(ListT<Element*>& planList)
 {
+
     if (_pSelect)
     {
 	planList.Insert( _pSelect->getPlan() );
     }
     if ( _pC )
     {
-	_pC->getPlanList(planList);
+	_pC->getPlanList(planList);	
     }
     if (_pNotPred)
     {
@@ -291,6 +296,7 @@ CegoPredicate& CegoPredicate::operator = ( const CegoPredicate& p)
     _pExpr2 = p._pExpr2;
     _pExpr3 = p._pExpr3;
     _exprList = p._exprList;
+    _pattern = p._pattern;
     _pMatcher = p._pMatcher;
     _comp = p._comp;
     _pSelect = p._pSelect;
@@ -423,7 +429,7 @@ CegoExpr* CegoPredicate::getExpr3()
     return _pExpr3;
 }
 
-ListT<CegoExpr*> CegoPredicate::getExprList()
+ListT<CegoExpr*>& CegoPredicate::getExprList()
 {
     return _exprList;
 }
@@ -524,13 +530,51 @@ bool CegoPredicate::isChecked() const
     return _isChecked;
 }
 
+bool CegoPredicate::isCursorPred() const
+{
+
+    switch (_mode )
+    {
+    case CegoPredicate::CONDITION:
+    {
+	return _pC->isCursorPred();	
+    }
+    case CegoPredicate::NOTPRED:
+    {
+	return _pNotPred->isCursorPred();
+    }
+    case CegoPredicate::EXPRCOMP:
+    case CegoPredicate::BETWEEN:
+    case CegoPredicate::EXPRONLY:
+    case CegoPredicate::NULLCOMP:
+    case CegoPredicate::NOTNULLCOMP:
+    case CegoPredicate::IN:
+    case CegoPredicate::ISLIKE:
+    case CegoPredicate::ISNOTLIKE:
+    case CegoPredicate::ISNCLIKE:
+    case CegoPredicate::ISNOTNCLIKE:
+    {
+	return true;
+    }
+    case CegoPredicate::EXISTSCOMP:
+    case CegoPredicate::INSUB:
+    case CegoPredicate::NOTINSUB:
+    case CegoPredicate::NOTIN:
+    {
+	return false;
+    }
+    }    
+}
+
 bool CegoPredicate::prioOnJoin() const
 {
     if ( _mode == CegoPredicate::EXPRCOMP
 	 && _comp == NOT_EQUAL )
 	return false;
     if ( _mode == CegoPredicate::ISLIKE
-	 ||  _mode == CegoPredicate::ISNOTLIKE )
+	 ||  _mode == CegoPredicate::ISNOTLIKE
+	 ||  _mode == CegoPredicate::ISNCLIKE 
+	 ||  _mode == CegoPredicate::ISNOTNCLIKE )
 	return false;
     if ( hasOrCond() )
 	return false;
@@ -557,16 +601,27 @@ const Chain& CegoPredicate::getPattern() const
 }
 
 bool CegoPredicate::match(const CegoFieldValue& val)
-{
+{    
     // for performance reasons, we build up matcher on demand
     // since it might occcur, that the predicate is evaluated via CegoDistCursor via CegoAttrCond
     if ( _pMatcher == 0 )
     {
-	Chain regexPattern = CegoQueryHelper::sql2Regex(_pattern);
+	
+	Chain regexPattern;
+	if ( _mode == CegoPredicate::ISLIKE || _mode == CegoPredicate::ISNOTLIKE )
+	    regexPattern = CegoQueryHelper::sql2Regex(_pattern);
+	else
+	    regexPattern = CegoQueryHelper::sql2Regex(_pattern.toLower());
+	
 	_pMatcher = new Matcher(regexPattern);	
 	_pMatcher->prepare();
     }
-    return _pMatcher->match(val.valAsChain());
+    
+    if ( _mode == CegoPredicate::ISLIKE || _mode == CegoPredicate::ISNOTLIKE )
+	return _pMatcher->match(val.valAsChain());
+    else
+	return _pMatcher->match(val.valAsChain().toLower());
+	
 }
 
 void CegoPredicate::encode(char *buf, CegoProcBlock *pBlock)
@@ -661,6 +716,8 @@ void CegoPredicate::encode(char *buf, CegoProcBlock *pBlock)
     }
     case CegoPredicate::ISLIKE:
     case CegoPredicate::ISNOTLIKE:
+    case CegoPredicate::ISNCLIKE:
+    case CegoPredicate::ISNOTNCLIKE:
     {
 	_pExpr1->encode(pP, pBlock);
 	pP = pP + _pExpr1->getEncodingLength(pBlock);
@@ -774,6 +831,8 @@ void CegoPredicate::decode(char *buf, CegoDistManager *pGTM, CegoProcBlock *pBlo
     }
     case CegoPredicate::ISLIKE:
     case CegoPredicate::ISNOTLIKE:
+    case CegoPredicate::ISNCLIKE:
+    case CegoPredicate::ISNOTNCLIKE:
     {
 	_pExpr1 = new CegoExpr(pP, pGTM, pBlock, tabSetId);
 	pP = pP + _pExpr1->getEncodingLength(pBlock);
@@ -823,6 +882,8 @@ int CegoPredicate::getEncodingLength(CegoProcBlock *pBlock) const
     }
     case CegoPredicate::ISLIKE:
     case CegoPredicate::ISNOTLIKE:
+    case CegoPredicate::ISNCLIKE:
+    case CegoPredicate::ISNOTNCLIKE:
     {
 	len += _pExpr1->getEncodingLength(pBlock);
 	len += 1; // sizeof pattern
@@ -910,6 +971,8 @@ ListT<CegoAttrDesc*> CegoPredicate::getAttrRefList() const
     }
     case CegoPredicate::ISLIKE:
     case CegoPredicate::ISNOTLIKE:
+    case CegoPredicate::ISNCLIKE:
+    case CegoPredicate::ISNOTNCLIKE:
     {
 	al += _pExpr1->getAttrRefList();
 	break;
@@ -931,8 +994,7 @@ ListT<CegoAttrDesc*> CegoPredicate::getAttrRefList() const
     case CegoPredicate::IN:
     case CegoPredicate::NOTIN:
     {
-	al = _pExpr1->getAttrRefList();
-
+	al = _pExpr1->getAttrRefList();	
 	CegoExpr** pExpr = _exprList.First();
 	while ( pExpr )
 	{
@@ -981,6 +1043,8 @@ int CegoPredicate::evalReferences(CegoContentObject *pCO, const ListT<CegoField>
     }
     case CegoPredicate::ISLIKE:
     case CegoPredicate::ISNOTLIKE:
+    case CegoPredicate::ISNCLIKE:
+    case CegoPredicate::ISNOTNCLIKE:
     {
 	refCount += _pExpr1->evalReferences(pCO, fl);
 	break;
@@ -1092,6 +1156,8 @@ void CegoPredicate::getFieldList(ListT<CegoField>& fl, ListT<CegoField>** pFLA)
     }
     case CegoPredicate::ISLIKE:
     case CegoPredicate::ISNOTLIKE:
+    case CegoPredicate::ISNCLIKE:
+    case CegoPredicate::ISNOTNCLIKE:
     {
 	_pExpr1->getFieldList(fl, pFLA);
 	break;
@@ -1190,11 +1256,10 @@ bool CegoPredicate::eval(ListT<CegoField>** pParentJoinBuf,
 	}
 	else if ( _mode == CegoPredicate::CONDITION )
 	{
-	    
+	    // should not occur, since condition case is already handled above
 	}
 	else
 	{
-
 	    // merge join buf and parent join buf
 	    
 	    ListT<CegoField>** flArray = 0;
@@ -1260,7 +1325,7 @@ bool CegoPredicate::eval(ListT<CegoField>** pParentJoinBuf,
 		while ( pF )
 		{
 		    cout << "Attr = " << pF->getTableAlias()  << "/" <<  pF->getTableName()
-				   << "/" << pF->getAttrName() << endl;
+			 << "/" << pF->getAttrName() << "="  << pF->getValue().valAsChain() << endl;
 		    pF = flArray[flaPos]->Next();
 		}
 		flaPos++;
@@ -1274,7 +1339,7 @@ bool CegoPredicate::eval(ListT<CegoField>** pParentJoinBuf,
 	    {	
 
 	    case CegoPredicate::EXPRCOMP:
-	    {
+	    {		
 		retVal = evalFields(_comp,
 				    _pExpr1->evalFieldValue(flArray, pBlock),
 				    _pExpr2->evalFieldValue(flArray, pBlock));
@@ -1314,13 +1379,13 @@ bool CegoPredicate::eval(ListT<CegoField>** pParentJoinBuf,
 	    }
 	    case CegoPredicate::EXPRONLY:
 	    {		
-		CegoFieldValue fv = _pExpr1->evalFieldValue(flArray, pBlock);
-		retVal = fv.asBool();
+		_exprOnlyVal = _pExpr1->evalFieldValue(flArray, pBlock);
+		retVal = _exprOnlyVal.asBool();
 		break;
 	    }
 	    case CegoPredicate::EXISTSCOMP:
 	    {		
-		_pSelect->reset(false);		
+		_pSelect->reset();		
 		if ( flArray )
 		{		    
 		    _pSelect->setParentJoinBuf(flArray);		    
@@ -1346,13 +1411,15 @@ bool CegoPredicate::eval(ListT<CegoField>** pParentJoinBuf,
 		}
 		_pSelect->setProcBlock(0);		
 		_pSelect->setParentJoinBuf();
-		_pSelect->reset(false);
+		_pSelect->reset();
 		break;
 	    }
 	    case CegoPredicate::ISLIKE:
 	    case CegoPredicate::ISNOTLIKE:
+	    case CegoPredicate::ISNCLIKE:
+	    case CegoPredicate::ISNOTNCLIKE:
 	    {
-		if ( _mode == CegoPredicate::ISLIKE )
+		if ( _mode == CegoPredicate::ISLIKE || _mode == CegoPredicate::ISNCLIKE)
 		    retVal = match(_pExpr1->evalFieldValue(flArray, pBlock));
 		else
 		    retVal =  ! match(_pExpr1->evalFieldValue(flArray, pBlock));
@@ -1382,7 +1449,7 @@ bool CegoPredicate::eval(ListT<CegoField>** pParentJoinBuf,
 	    case CegoPredicate::INSUB:
 	    case CegoPredicate::NOTINSUB:
 	    {
-		_pSelect->reset(false);
+		_pSelect->reset();
 
 		if ( flArray )
 		{		    
@@ -1423,7 +1490,7 @@ bool CegoPredicate::eval(ListT<CegoField>** pParentJoinBuf,
 		}
 		_pSelect->setProcBlock(0);
 		_pSelect->setParentJoinBuf();
-		_pSelect->reset(false);
+		_pSelect->reset();
 		
 		break;    
 	    }
@@ -1436,12 +1503,641 @@ bool CegoPredicate::eval(ListT<CegoField>** pParentJoinBuf,
 		// already handled
 		break;
 	    }
+
+	    // we avoid cleanUp here for performance reasons
+	    // still not clear, if really needed ..
+	    // cleanUp();
+	    
 	    return retVal;
 	}
     }
     return false;
 }
 
+CegoField CegoPredicate::evalField(const ListT<CegoField>& fl, bool graceful) const
+{
+    CegoField f;
+    
+    switch (_mode )
+    {
+    case CegoPredicate::CONDITION:
+    {
+	f = _pC->evalField(fl, graceful);
+	break;
+    }
+    case CegoPredicate::NOTPRED:
+    case CegoPredicate::EXPRCOMP:
+    case CegoPredicate::BETWEEN:
+    case CegoPredicate::ISLIKE:
+    case CegoPredicate::ISNOTLIKE:
+    case CegoPredicate::ISNCLIKE:
+    case CegoPredicate::ISNOTNCLIKE:
+    case CegoPredicate::INSUB:
+    case CegoPredicate::NOTINSUB:
+    case CegoPredicate::IN:
+    case CegoPredicate::NOTIN:
+    case CegoPredicate::EXISTSCOMP:
+    case CegoPredicate::NULLCOMP:
+    case CegoPredicate::NOTNULLCOMP:
+    {
+	f = CegoField( Chain("PRED"), Chain("PRED"), Chain("PRED"), BOOL_TYPE, sizeof(char));
+	break;
+    }
+    case CegoPredicate::EXPRONLY:
+    {
+	f = _pExpr1->evalField(fl, graceful);
+	break;
+    }
+    }
+    return f;
+}
+    
+CegoFieldValue CegoPredicate::getExprOnlyValue() const
+{
+    return _exprOnlyVal;
+}
+
+CegoPredicate* CegoPredicate::map(const Chain& viewAlias, ListT<CegoExpr*> exprList) const
+{
+    CegoPredicate* pMap;
+    
+    switch (_mode )
+    {
+    case CegoPredicate::CONDITION:
+    {
+	CegoPredicate* pLeft = 0;
+	CegoPredicate* pRight = 0;
+
+	bool mapOk=true;
+	if ( _pC->Left() )
+	{
+	    pLeft = _pC->Left()->map(viewAlias, exprList);
+	    if ( pLeft == 0 )
+	    {
+		mapOk=false;
+	    }
+	}
+	if ( _pC->Right() )
+	{
+	    pRight = _pC->Right()->map(viewAlias, exprList);
+	    if ( pRight == 0 )
+	    {
+		mapOk = false;
+	    }
+	}
+	if ( mapOk )
+	{
+	    CegoCondition* pC =  new CegoCondition(_pC->getCondType(), pLeft, pRight);
+	    return new CegoPredicate(pC);
+	}
+	else
+	{
+	    if ( pLeft )
+		delete pLeft;
+	    if ( pRight )
+		delete pRight;
+	}
+	break;
+    }
+    case CegoPredicate::NOTPRED:
+    {
+	CegoPredicate* pMap = _pNotPred->map(viewAlias, exprList);
+	if ( pMap )
+	    return new CegoPredicate(pMap);
+	break;
+    }
+    case CegoPredicate::EXPRCOMP:
+    {
+	bool mapOk=true;
+	CegoExpr* pExpr1=0;
+	if ( _pExpr1 )
+	{
+	    pExpr1 = mapExpr(viewAlias, exprList, _pExpr1);
+	    if ( pExpr1 == 0 )
+		mapOk=false;
+	}
+	CegoExpr* pExpr2=0;
+	if ( _pExpr2 )
+	{
+	    pExpr2 = mapExpr(viewAlias, exprList, _pExpr2);
+	    if ( pExpr2 == 0 )
+		mapOk=false;
+	}
+	if ( mapOk )
+	{
+	    return new CegoPredicate(pExpr1, pExpr2, _comp);
+	}
+	else
+	{
+	    if ( pExpr1 )
+		delete pExpr1;
+	    if ( pExpr2 )
+		delete pExpr2;
+	}	
+	break;
+    }
+    case CegoPredicate::BETWEEN:
+    {
+	bool mapOk=true;
+	CegoExpr* pExpr1=0;
+	if ( _pExpr1 )
+	{
+	    pExpr1 = mapExpr(viewAlias, exprList, _pExpr1);
+	    if ( pExpr1 == 0 )
+		mapOk=false;
+	}
+	CegoExpr* pExpr2=0;
+	if ( _pExpr2 )
+	{
+	    pExpr2 = mapExpr(viewAlias, exprList, _pExpr2);
+	    if ( pExpr2 == 0 )
+		mapOk=false;
+	}
+	CegoExpr* pExpr3=0;
+	if ( _pExpr3 )
+	{
+	    pExpr3 = mapExpr(viewAlias, exprList, _pExpr3);
+	    if ( pExpr3 == 0 )
+		mapOk=false;
+	}
+
+	if ( mapOk )
+	{
+	    return new CegoPredicate(pExpr1, pExpr2, pExpr3);
+	}
+	else
+	{
+	    if ( pExpr1 )
+		delete pExpr1;
+	    if ( pExpr2 )
+		delete pExpr2;
+	    if ( pExpr3 )
+		delete pExpr3;	    
+	}
+	break;
+    }
+    case CegoPredicate::ISLIKE:
+    case CegoPredicate::ISNOTLIKE:
+    case CegoPredicate::ISNCLIKE:
+    case CegoPredicate::ISNOTNCLIKE:
+    {
+	CegoExpr* pExpr1=0;
+	if ( _pExpr1 )
+	{
+	    pExpr1 = mapExpr(viewAlias, exprList, _pExpr1);
+	}
+
+	if ( pExpr1 )
+	{
+	    if ( _mode == CegoPredicate::ISLIKE )
+		return new CegoPredicate(pExpr1, _pattern, false, true);
+	    if ( _mode == CegoPredicate::ISNOTLIKE )
+		return new CegoPredicate(pExpr1, _pattern, true, true);
+	    if ( _mode == CegoPredicate::ISNCLIKE )
+		return new CegoPredicate(pExpr1, _pattern, false, false);
+	    if ( _mode == CegoPredicate::ISNOTNCLIKE )
+		return new CegoPredicate(pExpr1, _pattern, true, false);	    	    
+	}
+    }
+    case CegoPredicate::NULLCOMP:
+    case CegoPredicate::NOTNULLCOMP:
+    {
+	CegoExpr* pExpr1=0;
+	if ( _pExpr1 )
+	{
+	    pExpr1 = mapExpr(viewAlias, exprList, _pExpr1);
+	}
+	if ( _mode == CegoPredicate::NULLCOMP )
+	    return new CegoPredicate(pExpr1, true);
+	if ( _mode == CegoPredicate::NOTNULLCOMP )
+	    return new CegoPredicate(pExpr1, false);	
+	break;
+    }
+    case CegoPredicate::IN:
+    case CegoPredicate::NOTIN:
+    {
+
+	bool isValid=true;
+	
+	CegoExpr* pExpr1=0;
+	ListT<CegoExpr*> exprList;
+	
+	if ( _pExpr1 )
+	{
+	    pExpr1 = mapExpr(viewAlias, exprList, _pExpr1);
+	}
+	
+	if ( pExpr1 )
+	{	  
+	    CegoExpr** pInExpr = _exprList.First();
+	    while ( pInExpr && isValid )
+	    {
+		CegoExpr* pExpr = mapExpr(viewAlias, exprList,  *pInExpr);
+		if ( pExpr )
+		    exprList.Insert(pExpr);
+		else
+		    isValid = false;
+		pInExpr = _exprList.Next();
+	    }
+	}
+	if ( pExpr1 && isValid )
+	{
+	    if ( _mode == CegoPredicate::IN )
+		return new CegoPredicate(pExpr1, exprList, false);
+	    if ( _mode == CegoPredicate::NOTIN )
+		return new CegoPredicate(pExpr1, exprList, true);
+	}
+	else
+	{
+	    if ( pExpr1 )
+		delete pExpr1;
+	    CegoExpr** pInExpr = exprList.First();
+	    while ( pInExpr  )
+	    {
+		delete *pInExpr;
+		pInExpr = exprList.Next();
+	    }	    
+	}
+	break;
+    }
+	
+    case CegoPredicate::INSUB:
+    case CegoPredicate::NOTINSUB:
+    case CegoPredicate::EXISTSCOMP:
+    {
+	// not handled
+	break;
+    }
+    case CegoPredicate::EXPRONLY:
+    {
+	CegoExpr* pExpr1=0;
+	if ( _pExpr1 )
+	{
+	    pExpr1 = mapExpr(viewAlias, exprList, _pExpr1);
+	}
+		
+	if ( pExpr1 )
+	{
+	    return new CegoPredicate(pExpr1);
+	}
+	break;
+    }
+    }
+
+    return 0;
+}
+
+CegoPredicate* CegoPredicate::getReduced(const Chain& tableAlias) const
+{
+    ListT<Chain> aliasList;
+    aliasList.Insert(tableAlias);
+    return getReduced(aliasList);
+}
+
+CegoPredicate* CegoPredicate::getReduced(const ListT<Chain>& aliasList) const
+{
+    CegoPredicate* pReduced;
+    
+    switch (_mode )
+    {
+    case CegoPredicate::CONDITION:
+    {
+	CegoPredicate* pLeft = 0;
+	CegoPredicate* pRight = 0;
+
+	bool reduceOk=true;
+	if ( _pC->Left() )
+	{
+	    pLeft = _pC->Left()->getReduced(aliasList);
+	}
+	if ( _pC->Right() )
+	{
+	    pRight = _pC->Right()->getReduced(aliasList);
+	}
+
+	if ( _pC->getCondType() == CegoCondition::PRED && pLeft )
+	{
+	    CegoCondition* pC =  new CegoCondition(_pC->getCondType(), pLeft, 0);
+	    return new CegoPredicate(pC);	    
+	}
+	if ( pRight && pLeft )
+	{
+	    CegoCondition* pC =  new CegoCondition(_pC->getCondType(), pLeft, pRight);
+	    return new CegoPredicate(pC);
+	}
+	if ( pRight && _pC->getCondType() == CegoCondition::AND)
+	{
+	    return pRight;
+	}
+	if ( pLeft && _pC->getCondType() == CegoCondition::AND)
+	{
+	    return pLeft;
+	}
+	
+	if ( pRight )
+	    delete pRight;
+	if ( pLeft )
+	    delete pLeft;
+	
+	break;
+    }
+    case CegoPredicate::NOTPRED:
+    {
+	CegoPredicate* pReduced = _pNotPred->getReduced(aliasList);
+	return pReduced;
+    }
+    case CegoPredicate::EXPRCOMP:
+    {
+	CegoExpr* pExpr1=0;
+	CegoExpr* pExpr2=0;
+	if ( _pExpr1 )
+	{
+	    pExpr1 = reduceExpr(aliasList, _pExpr1);
+	}
+	if ( _pExpr2 )
+	{
+	    pExpr2 = reduceExpr(aliasList, _pExpr2);
+	}
+	if ( pExpr1 && pExpr2 )
+	{
+	    return new CegoPredicate(pExpr1, pExpr2, _comp);
+	}
+	else
+	{
+	    if ( pExpr1 )
+		delete pExpr1;
+	    if ( pExpr2 )
+		delete pExpr2;
+	}	
+	break;
+    }
+    case CegoPredicate::BETWEEN:
+    {
+	CegoExpr* pExpr1=0;
+	CegoExpr* pExpr2=0;
+	CegoExpr* pExpr3=0;
+	if ( _pExpr1 )
+	{
+	    pExpr1 = reduceExpr(aliasList, _pExpr1);
+	}
+	if ( _pExpr2 )
+	{
+	    pExpr2 = reduceExpr(aliasList, _pExpr2);
+	}
+	if ( _pExpr3 )
+	{
+	    pExpr3 = reduceExpr(aliasList, _pExpr3);
+	}
+
+	if ( pExpr1 && pExpr2 && pExpr3 )
+	{
+	    return new CegoPredicate(pExpr1, pExpr2, pExpr3);
+	}
+	else
+	{
+	    if ( pExpr1 )
+		delete pExpr1;
+	    if ( pExpr2 )
+		delete pExpr2;
+	    if ( pExpr3 )
+		delete pExpr3;
+	    
+	}	
+	break;       
+    }
+    case CegoPredicate::ISLIKE:
+    case CegoPredicate::ISNOTLIKE:
+    case CegoPredicate::ISNCLIKE:
+    case CegoPredicate::ISNOTNCLIKE:
+    {
+	CegoExpr* pExpr1=0;
+	if ( _pExpr1 )
+	{
+	    pExpr1 = reduceExpr(aliasList, _pExpr1);
+	}
+
+	if ( pExpr1  )
+	{
+	    if ( _mode == CegoPredicate::ISLIKE )
+		return new CegoPredicate(pExpr1, _pattern, false, true);
+	    if ( _mode == CegoPredicate::ISNOTLIKE )
+		return new CegoPredicate(pExpr1, _pattern, true, true);
+	    if ( _mode == CegoPredicate::ISNCLIKE )
+		return new CegoPredicate(pExpr1, _pattern, false, false);
+	    if ( _mode == CegoPredicate::ISNOTNCLIKE )
+		return new CegoPredicate(pExpr1, _pattern, true, false);	    	    
+	}
+	break;
+    }
+    case CegoPredicate::IN:
+    case CegoPredicate::NOTIN:
+    {
+	bool isValid=true;
+	
+	CegoExpr* pExpr1=0;
+	ListT<CegoExpr*> exprList;
+	
+	if ( _pExpr1 )
+	{
+	    pExpr1 = reduceExpr(aliasList, _pExpr1);
+	}
+
+	if ( pExpr1 )
+	{	  
+	    CegoExpr** pInExpr = _exprList.First();
+	    while ( pInExpr && isValid )
+	    {
+		CegoExpr* pExpr = reduceExpr(aliasList, *pInExpr);
+		if ( pExpr )
+		    exprList.Insert(pExpr);
+		else
+		    isValid = false;
+		pInExpr = _exprList.Next();
+	    }
+	}
+	if ( pExpr1 && isValid )
+	{
+	    if ( _mode == CegoPredicate::IN )
+		return new CegoPredicate(pExpr1, exprList, false);
+	    if ( _mode == CegoPredicate::NOTIN )
+		return new CegoPredicate(pExpr1, exprList, true);
+	}
+	else
+	{
+	    if ( pExpr1 )
+		delete pExpr1;
+	    CegoExpr** pInExpr = exprList.First();
+	    while ( pInExpr  )
+	    {
+		delete *pInExpr;
+		pInExpr = exprList.Next();
+	    }	    
+	}
+	break;
+    }
+    case CegoPredicate::NULLCOMP:
+    case CegoPredicate::NOTNULLCOMP:
+    {
+
+	CegoExpr* pExpr1=0;
+	if ( _pExpr1 )
+	{
+	    pExpr1 = reduceExpr(aliasList, _pExpr1);
+	}
+	
+	if ( pExpr1  )
+	{	    
+	    if ( _mode == CegoPredicate::NULLCOMP )
+		return new CegoPredicate(pExpr1, true);
+	    if ( _mode == CegoPredicate::NOTNULLCOMP )
+		return new CegoPredicate(pExpr1, false);
+	}
+	break;
+    }
+    case CegoPredicate::INSUB:
+    case CegoPredicate::NOTINSUB:
+    case CegoPredicate::EXISTSCOMP:
+    {
+	// not handled
+	break;
+    }
+    case CegoPredicate::EXPRONLY:
+    {
+	CegoExpr* pExpr1=0;
+	if ( _pExpr1 )
+	{
+	    pExpr1 = reduceExpr(aliasList, _pExpr1);
+	}
+		
+	if ( pExpr1 )
+	{
+	    return new CegoPredicate(pExpr1);
+	}
+	break;
+    }
+    }
+    return 0;
+}
+
+CegoExpr* CegoPredicate::mapExpr(const Chain& viewAlias, const ListT<CegoExpr*>& exprList, CegoExpr* pExpr) const
+{
+    if ( pExpr->checkConst() )
+	return pExpr->clone();
+
+    CegoCondition* pC = pExpr->checkCondition();
+    if ( pC )
+    {
+	bool mapOk=true;
+	
+	CegoPredicate *pLeft=0;
+	if ( pC->Left() )
+	{
+	    pLeft = pC->Left()->map(viewAlias, exprList);
+	    if ( pLeft == 0 )
+	    {
+		mapOk=false;
+	    }
+	}
+	CegoPredicate *pRight=0;
+	if ( pC->Right() )
+	{
+	    pRight = pC->Right()->map(viewAlias, exprList);
+	    if ( pRight == 0 )
+	    {
+		mapOk=false;
+	    }
+	}
+
+	if ( mapOk )
+	{
+	    CegoCondition* pMapCond = new CegoCondition(pC->getCondType(), pLeft, pRight);
+	    return new CegoExpr( new CegoTerm(  new CegoFactor(pMapCond)));	    
+	}
+	else
+	{
+	    if ( pLeft )
+		delete pLeft;
+	    if ( pRight )
+		delete pRight;
+	    return 0;
+	}
+    }
+
+    return pExpr->map(viewAlias, exprList);
+    
+}
+
+CegoExpr* CegoPredicate::reduceExpr(const ListT<Chain>& aliasList, CegoExpr* pExpr) const
+{
+    if ( pExpr->checkConst() )
+	return pExpr->clone();
+
+    CegoCondition* pC = pExpr->checkCondition();
+    if ( pC )
+    {	
+	CegoPredicate *pLeft=0;
+	CegoPredicate *pRight=0;
+	if ( pC->Left() )
+	{
+	    pLeft = pC->Left()->getReduced(aliasList);
+	}
+
+	if ( pC->Right() )
+	{
+	    pRight = pC->Right()->getReduced(aliasList);	    
+	}
+
+	if ( pLeft && pRight )
+	{
+	    CegoCondition* pReducedCond = new CegoCondition(pC->getCondType(), pLeft, pRight);
+	    return new CegoExpr( new CegoTerm(  new CegoFactor(pReducedCond)));	    
+	}
+
+	if ( pLeft && pC->getCondType() == CegoCondition::AND )
+	{
+	    CegoCondition* pReducedCond = new CegoCondition(CegoCondition::PRED, pLeft, 0);	    
+	    return new CegoExpr( new CegoTerm(  new CegoFactor(pReducedCond)));	    
+	}
+	if ( pRight && pC->getCondType() == CegoCondition::AND )
+	{
+	    CegoCondition* pReducedCond = new CegoCondition(CegoCondition::PRED, pRight, 0);	    
+	    return new CegoExpr( new CegoTerm(  new CegoFactor(pReducedCond)));	    
+	}
+
+	if ( pLeft )
+	    delete pLeft;
+	if ( pRight )
+	    delete pRight;	
+    }
+
+    ListT<CegoAttrDesc*> refList = pExpr->getAttrRefList();
+    
+    bool hasForeign = false;
+    CegoAttrDesc** pAD = refList.First();
+    while ( pAD && hasForeign == false )
+    {
+	Chain *pAlias = aliasList.First();
+	bool aliasFound=false;
+	while ( pAlias )
+	{
+	    if ( (*pAD)->getTableName() == *pAlias )
+	    {
+		aliasFound=true;
+	    }
+	    pAlias = aliasList.Next();
+	}
+	if ( aliasFound == false )
+	    hasForeign=true;
+	
+	pAD = refList.Next();	
+    }
+    
+    if ( hasForeign == false )
+    {
+	return pExpr->clone();
+    }
+    return 0;
+}
 
 bool CegoPredicate::evalFields(CegoComparison comp, const CegoFieldValue& f1, const CegoFieldValue& f2)
 {
@@ -1471,7 +2167,6 @@ bool CegoPredicate::evalBetween(const CegoFieldValue& f1, const CegoFieldValue&
     return false;
 }
 
-
 void CegoPredicate::setAggregationValue(ListT<CegoField>& jfl)
 {
     switch (_mode )
@@ -1501,6 +2196,8 @@ void CegoPredicate::setAggregationValue(ListT<CegoField>& jfl)
     }
     case CegoPredicate::ISLIKE:
     case CegoPredicate::ISNOTLIKE:
+    case CegoPredicate::ISNCLIKE:
+    case CegoPredicate::ISNOTNCLIKE:
     {
 	setAggregationValueForExpr(_pExpr1, jfl);
 	break;
@@ -1559,7 +2256,6 @@ void CegoPredicate::setAggregationValueForExpr(CegoExpr* pExpr, ListT<CegoField>
     }
 }
 
-
 CegoPredicate* CegoPredicate::clone(bool isAttrRef)
 {    
     if ( _mode == CegoPredicate::EXPRCOMP )
@@ -1576,11 +2272,19 @@ CegoPredicate* CegoPredicate::clone(bool isAttrRef)
     }
     else if ( _mode == CegoPredicate::ISLIKE )
     {
-	return new CegoPredicate(_pExpr1->clone(isAttrRef), _pattern, false);
+	return new CegoPredicate(_pExpr1->clone(isAttrRef), _pattern, false, true);
     }
     else if ( _mode == CegoPredicate::ISNOTLIKE )
     {
-	return new CegoPredicate(_pExpr1->clone(isAttrRef), _pattern, true);
+	return new CegoPredicate(_pExpr1->clone(isAttrRef), _pattern, true, true);
+    }
+    else if ( _mode == CegoPredicate::ISNCLIKE )
+    {
+	return new CegoPredicate(_pExpr1->clone(isAttrRef), _pattern, false, false);
+    }
+    else if ( _mode == CegoPredicate::ISNOTNCLIKE )
+    {
+	return new CegoPredicate(_pExpr1->clone(isAttrRef), _pattern, true, false);
     }
     else if ( _mode == CegoPredicate::CONDITION )
     {
@@ -1725,6 +2429,20 @@ Chain CegoPredicate::getId(CegoProcBlock *pBlock) const
 	s += Chain("'") + _pattern + Chain("'");
 	break;
     }
+    case CegoPredicate::ISNCLIKE:
+    {
+	s = _pExpr1->getId(pBlock);
+	s += Chain("ncl");
+	s += Chain("'") + _pattern + Chain("'");
+	break;
+    }
+    case CegoPredicate::ISNOTNCLIKE:
+    {
+	s = _pExpr1->getId(pBlock);
+	s += Chain("nncl");
+	s += Chain("'") + _pattern + Chain("'");
+	break;
+    }
     case CegoPredicate::EXPRONLY:
     {
 	s =  _pExpr1->getId(pBlock);
@@ -1807,30 +2525,7 @@ Chain CegoPredicate::toChain(int defTabSetId, const Chain& indent) const
     {
     case CegoPredicate::CONDITION:
     {
-	if (_pC->Left())
-	{
-	    s = Chain("( ");
-	    s += _pC->Left()->toChain(defTabSetId, indent);
-
-	    s += Chain("\n") + indent;
-	    Chain addIndent;
-	    switch(_pC->getCondType())
-	    {
-	    case CegoCondition::AND:
-		s += Chain("and ");
-		addIndent = Chain(DEFAULTINDENT);
-		s += _pC->Right()->toChain(defTabSetId, indent + addIndent);
-		break;
-	    case CegoCondition::OR:
-		s += Chain("or ");
-		addIndent = Chain(DEFAULTINDENT);
-		s += _pC->Right()->toChain(defTabSetId, indent + addIndent);
-		break;
-	    case CegoCondition::PRED:
-		break;
-	    }
-	    s += Chain(" )");
-	}
+	s =  _pC->toChain(defTabSetId, indent);	
 	break;
     }
     case CegoPredicate::NOTPRED:
@@ -1903,6 +2598,20 @@ Chain CegoPredicate::toChain(int defTabSetId, const Chain& indent) const
 	s += Chain("'") + _pattern + Chain("'");
 	break;
     }
+    case CegoPredicate::ISNCLIKE:
+    {
+	s = _pExpr1->toChain(defTabSetId, indent);
+	s += Chain(" nclike ");
+	s += Chain("'") + _pattern + Chain("'");
+	break;
+    }
+    case CegoPredicate::ISNOTNCLIKE:
+    {
+	s = _pExpr1->toChain(defTabSetId, indent);
+	s += Chain(" not nclike ");
+	s += Chain("'") + _pattern + Chain("'");
+	break;
+    }
     case CegoPredicate::EXPRONLY:
     {
 	s = _pExpr1->toChain(defTabSetId, indent);
