@@ -6,7 +6,7 @@
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -20,6 +20,7 @@
 
 // LFC INCLUDES
 #include <lfcbase/Exception.h>
+
 // CEGO INCLUDES
 #include "CegoSelect.h"
 #include "CegoProcCursorCloseStmt.h"
@@ -43,7 +44,11 @@ CegoException CegoProcCursorCloseStmt::execute()
     return NONE_EXCEP;
 }
 
-Chain CegoProcCursorCloseStmt::toChain(const Chain& indent) const
+void CegoProcCursorCloseStmt::cleanUp()
+{
+}	
+
+Chain CegoProcCursorCloseStmt::toChain(int defTabSetId, const Chain& indent) const
 {
     Chain s;
 
@@ -56,3 +61,8 @@ Chain CegoProcCursorCloseStmt::dbFormat(CegoDatabaseFormater *pForm) const
 {
     return pForm->formatProcCursorCloseStmt(_pCur->getName());
 }
+
+bool CegoProcCursorCloseStmt::isStatic() const
+{
+    return false;
+}
