@@ -6,7 +6,7 @@
 //      
 // Design and Implementation by Bjoern Lemke
 //     
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -48,6 +48,11 @@ CegoException CegoProcException::execute()
     return NONE_EXCEP;
 }
 
+void CegoProcException::cleanUp()
+{
+    _pBlock->cleanUp();
+}
+
 CegoProcException& CegoProcException::operator = ( const CegoProcException& pe)
 {
     _excep = pe._excep;
@@ -62,19 +67,11 @@ bool CegoProcException::operator == ( const CegoProcException& pe)
     return false;
 }
 
-Chain CegoProcException::toChain(const Chain& indent) const
+Chain CegoProcException::toChain(int defTabSetId, const Chain& indent) const
 {
     Chain s = indent;
 
-    if ( _excep == INVALID_OBJECT_EXCEP )
-    {
-	s += Chain(INVALID_OBJECT_EXCEP_ID);
-    }
-    else if ( _excep == INVALID_ATTRIBUTE_EXCEP )
-    {
-	s += Chain(INVALID_ATTRIBUTE_EXCEP_ID);
-    }
-    else if ( _excep == COREOP_EXCEP ) 
+    if ( _excep == COREOP_EXCEP ) 
     {
 	s += Chain(COREOP_EXCEP_ID);
     }
@@ -95,8 +92,21 @@ Chain CegoProcException::dbFormat(CegoDatabaseFormater *pForm) const
     return pForm->formatProcException(_excep);
 }
 
+bool CegoProcException::isStatic() const
+{
+    ListT<CegoProcStmt*> stmtList = _pBlock->getStmtList();
+	
+    CegoProcStmt **pStmt = stmtList.First();
+    while ( pStmt )
+    {
+	if ( (*pStmt)->isStatic() == false )
+	    return false;
+    }
+    return true;
+}
+
 ostream& operator << (ostream& s, const CegoProcException& pe)
 {    
-    s << pe.toChain();
+    s << pe.toChain(0);
     return s;
 }
