@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -36,21 +36,19 @@ public:
     
     CegoException getException() const;
     CegoProcBlock* getBlock();
+    CegoException execute();
+    void cleanUp();
+    Chain toChain(int defTabSetId, const Chain& indent = Chain("")) const;
+    Chain dbFormat(CegoDatabaseFormater *pForm) const;
+    bool isStatic() const;
 
     CegoProcException& operator = ( const CegoProcException& pe);
     bool operator == ( const CegoProcException& pe);
-
-    CegoException execute();
-
-    Chain toChain(const Chain& indent = Chain("")) const;
-    Chain dbFormat(CegoDatabaseFormater *pForm) const;
-    
     friend ostream& operator << (ostream& s, const CegoProcException& pe);
     
 private:
     
     CegoException _excep;
     CegoProcBlock* _pBlock;
-    
 };
 #endif
