@@ -6,7 +6,7 @@
 //                                                 
 // Design and Implementation by Bjoern Lemke               
 //
-// (C)opyright 2000-2013 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -22,22 +22,23 @@
 #include "CegoProcIfStmt.h"
 #include "CegoDatabaseFormater.h"
 
-CegoProcIfStmt::CegoProcIfStmt(ListT<CegoProcCond*>& condList,
+CegoProcIfStmt::CegoProcIfStmt(ListT<CegoPredicate*>& predList,
 				   ListT<CegoProcBlock*>& ifBlockList,
 				   CegoProcBlock *pBlock) : CegoProcStmt(pBlock)
 {
-    _condList = condList;
+    _predList = predList;
     _ifBlockList = ifBlockList;
+    _pBlock = pBlock;
 }
 
 CegoProcIfStmt::~CegoProcIfStmt()  
 {
-    CegoProcCond** pCond = _condList.First();
+    CegoPredicate** pPred = _predList.First();
     
-    while  ( pCond )
+    while  ( pPred )
     {
-	delete *pCond;
-	pCond = _condList.Next();
+	delete *pPred;
+	pPred = _predList.Next();
     }
 
     CegoProcBlock** pBlock = _ifBlockList.First();
@@ -50,22 +51,30 @@ CegoProcIfStmt::~CegoProcIfStmt()
 
 CegoException CegoProcIfStmt::execute()
 {
-    CegoProcCond** pCond = _condList.First();
+    CegoPredicate** pPred = _predList.First();
     CegoProcBlock** pBlock = _ifBlockList.First();
     
-    while ( pCond )
-    {
-	
-	(*pCond)->setBlock(getParentBlock());
-	
-	if ( (*pCond)->eval() == true )
+    while ( pPred )
+    {	
+	// (*pPred)->setBlock(_pBlock);
+
+	try
 	{
-	    return (*pBlock)->execute();		
+	    if ( (*pPred)->eval(0,0,0,0, _pBlock) == true )
+	    {
+		return (*pBlock)->execute();		
+	    }
+	    else
+	    {
+		pPred = _predList.Next();
+		pBlock = _ifBlockList.Next();
+	    }
 	}
-	else
+	catch ( Exception e )
 	{
-	    pCond = _condList.Next();
-	    pBlock = _ifBlockList.Next();
+	    Chain excepMsg = e.getBaseMsg();	    
+	    getParentBlock()->setExceptionMsg(excepMsg);
+	    return COREOP_EXCEP;
 	}
     }
     
@@ -77,28 +86,44 @@ CegoException CegoProcIfStmt::execute()
     return NONE_EXCEP;
 }
 
-Chain CegoProcIfStmt::toChain(const Chain& indent) const
+void CegoProcIfStmt::cleanUp()
+{
+    CegoPredicate** pPred = _predList.First();
+    CegoProcBlock** pBlock = _ifBlockList.First();
+    
+    while ( pPred )
+    {	
+	(*pPred)->cleanUp();
+	(*pBlock)->cleanUp();
+		
+	pPred = _predList.Next();
+	pBlock = _ifBlockList.Next();
+    }
+    
+    if ( pBlock )
+    { 
+	return (*pBlock)->cleanUp();
+    }
+}
+	
+Chain CegoProcIfStmt::toChain(int defTabSetId, const Chain& indent) const
 {
-
     Chain s;
 
-    CegoProcCond** pCond = _condList.First();
+    CegoPredicate** pPred = _predList.First();
     CegoProcBlock** pBlock = _ifBlockList.First();
     
     s = indent + Chain("if ");
     
-    while ( pCond )
+    while ( pPred )
     {
+	s += (*pPred)->toChain(defTabSetId);	
+	s += Chain("\n") + indent + Chain("then\n");	    
+	s += (*pBlock)->toChain(defTabSetId, indent + Chain(DEFAULTINDENT));
 	
-	s += (*pCond)->toChain();
-	
-	s += Chain("\n") + indent + Chain("then\n");
-	    
-	s += (*pBlock)->toChain(indent + indent);
-	
-	pCond = _condList.Next();
+	pPred = _predList.Next();
 	pBlock = _ifBlockList.Next();
-	if ( pCond )
+	if ( pPred )
 	    s += indent + Chain("elsif ");
 	else if ( pBlock == 0 )
 	    s += indent + Chain("end");
@@ -106,7 +131,7 @@ Chain CegoProcIfStmt::toChain(const Chain& indent) const
     if ( pBlock )
     {
 	s += indent + Chain("else\n");
-	s += indent + (*pBlock)->toChain();
+	s += (*pBlock)->toChain(defTabSetId, indent + Chain(DEFAULTINDENT));
 	s += indent + Chain("end");
     }
 
@@ -115,5 +140,18 @@ Chain CegoProcIfStmt::toChain(const Chain& indent) const
 
 Chain CegoProcIfStmt::dbFormat(CegoDatabaseFormater *pForm) const
 {
-    return pForm->formatProcIfStmt(_condList, _ifBlockList);
+    return pForm->formatProcIfStmt(_predList, _ifBlockList);
+}
+
+bool CegoProcIfStmt::isStatic() const
+{
+    CegoProcBlock** pBlock = _ifBlockList.First();
+    
+    while ( pBlock )
+    {
+	if ( (*pBlock)->isStatic() == false )
+	    return false;	
+	pBlock = _ifBlockList.Next();
+    }
+    return true;
 }
