@@ -8,7 +8,7 @@
 //
 // Design and Implementation by Bjoern Lemke               
 //
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // INTERFACE MODULE
 //
@@ -25,7 +25,7 @@
 #include <lfcbase/ListT.h>
 
 // cego includes
-#include "CegoProcPred.h"
+#include "CegoPredicate.h"
 #include "CegoProcBlock.h"
 #include "CegoProcStmt.h"
 
@@ -33,20 +33,21 @@ class CegoProcIfStmt : public CegoProcStmt {
     
 public:
     
-    CegoProcIfStmt(ListT<CegoProcCond*>& condList,
-		     ListT<CegoProcBlock*>& ifBlockList,
-		     CegoProcBlock *pBlock);
+    CegoProcIfStmt(ListT<CegoPredicate*>& predList,
+		   ListT<CegoProcBlock*>& ifBlockList,
+		   CegoProcBlock *pBlock);
     ~CegoProcIfStmt();
 
     CegoException execute();
- 
-    Chain toChain(const Chain& indent = Chain("")) const;
+    void cleanUp();
+    Chain toChain(int defTabSetId, const Chain& indent = Chain("")) const;
     Chain dbFormat(CegoDatabaseFormater *pForm) const;
+    bool isStatic() const;
     
 private:
 
-    ListT<CegoProcCond*> _condList;
+    ListT<CegoPredicate*> _predList;
     ListT<CegoProcBlock*> _ifBlockList;
-
+    CegoProcBlock* _pBlock;
 };
 #endif
