@@ -1,12 +1,12 @@
 ///////////////////////////////////////////////////////////////////////////////
 //                                                         
 // CegoProcQueryStmt.cc  
-// --------------------                                                     
+// --------------------
 // Cego procedure query statement implementation
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // IMPLEMENTATION MODULE
 //
@@ -37,8 +37,7 @@ CegoProcQueryStmt::~CegoProcQueryStmt()
 
 CegoException CegoProcQueryStmt::execute()
 {
-    
-    _pQuery->setBlock(getParentBlock());    
+    // _pQuery->setBlock(getParentBlock());    
     
     try 
     {
@@ -49,20 +48,26 @@ CegoException CegoProcQueryStmt::execute()
 	CegoProcBlock *pBlock = getParentBlock();
 	if ( pBlock )
 	{
-	    Chain excepMsg;
-	    qe.pop(excepMsg);
+	    Chain excepMsg;	    
+	    excepMsg = qe.getBaseMsg();
 	    pBlock->setExceptionMsg(excepMsg);
 	}
 	
 	return qe.getException();
     }
+    
     return NONE_EXCEP;
 }
 
-Chain CegoProcQueryStmt::toChain(const Chain& indent) const
+void CegoProcQueryStmt::cleanUp()
+{
+    _pQuery->cleanUp();
+}
+
+Chain CegoProcQueryStmt::toChain(int defTabSetId, const Chain& indent) const
 {
     Chain s;
-    s = _pQuery->toChain(indent);
+    s = indent + _pQuery->toChain(defTabSetId, indent);
     return s;
 }
 
@@ -70,3 +75,8 @@ Chain CegoProcQueryStmt::dbFormat(CegoDatabaseFormater *pForm) const
 {
     return pForm->formatProcQueryStmt(_pQuery);
 }
+
+bool CegoProcQueryStmt::isStatic() const
+{
+    return false;
+}
