@@ -1,12 +1,12 @@
 ///////////////////////////////////////////////////////////////////////////////
 //                                                         
 // CegoReturnVar.cc  
-// ----------------                                                     
+// ----------------
 // Cego return variable implementation
 //                                                         
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2000-2016 Bjoern Lemke 
+// (C)opyright 2000-2025 Bjoern Lemke 
 //
 // IMPLEMENTATION MODULE
 //
@@ -50,10 +50,10 @@ CegoReturnVar& CegoReturnVar::operator = ( const CegoReturnVar& rv)
     return (*this);
 }
 
-Chain CegoReturnVar::toChain() const
+Chain CegoReturnVar::toChain(int defTabSetId) const
 {
     Chain s;
-    s = Chain(":")  + _varName + " = " + _pExpr->toChain();
+    s = Chain(":")  + _varName + " = " + _pExpr->toChain(defTabSetId);
     return s;
 }
 
