@@ -424,6 +424,7 @@
 #define XML_LOGMNGPROG_ATTR "LOGMNGPROG"
 #define XML_LOGMNGTIMEOUT_ATTR "LOGMNGTIMEOUT"
 #define XML_LOGUSER_ATTR "LOGUSER"
+#define XML_LOGTS_ATTR "LOGTS"
 #define XML_LSN_ATTR "LSN"
 #define XML_WRITTENLSN_ATTR "WRITTENLSN"
 #define XML_NEXTBRANCHID_ATTR "NEXTBRANCHID"
@@ -446,7 +447,6 @@
 #define XML_QUERYCACHENUMFAIL_ATTR "QUERYCACHENUMFAIL"
 #define XML_MVAL_ATTR "MVAL"
 #define XML_MAXSENDLEN_ATTR "MAXSENDLEN"
-#define XML_MAXTSID_ATTR "MAXTSID"
 #define XML_MEDIATOR_ATTR "MEDIATOR"
 #define XML_MODE_ATTR "MODE"
 #define XML_MSG_ATTR "MSG"
