@@ -8,7 +8,7 @@
 //                                               
 // Design and Implementation by Bjoern Lemke               
 //                                                         
-// (C)opyright 2009-2016 Bjoern Lemke                        
+// (C)opyright 2000-2025 Bjoern Lemke
 //
 // Description: This is a wrapper module which implements a plain C client interface to the
 //              cego database system
@@ -20,11 +20,12 @@
 #ifdef  __cplusplus
 #include <lfcbase/Logger.h>
 #include <lfcbase/NetHandler.h>
-
+#include "CegoDefs.h"
 #include "CegoDbHandler.h"
+#else
+#define PageIdType unsigned long long
 #endif
 
-
 #define CG_INT      1
 #define CG_VARCHAR  2
 #define CG_LONG     3
@@ -34,10 +35,9 @@
 #define CG_FLOAT    7
 #define CG_DOUBLE   8
 #define CG_DECIMAL  9
-#define CG_FIXED    10
-#define CG_SMALLINT 11
-#define CG_TINYINT  12
-#define CG_BLOB     13
+#define CG_SMALLINT 10
+#define CG_TINYINT  11
+#define CG_BLOB     12
 #define CG_CLOB     13
 #define CG_NULL     100
 
@@ -51,6 +51,8 @@ typedef struct CegoDBHandle
 #ifdef  __cplusplus
 {
     CegoModule *pModule;
+    Chain host;
+    int port;
     NetHandler *pNet;
     CegoDbHandler *pDB;
 } 
@@ -76,12 +78,25 @@ typedef struct CegoValue
 
 typedef struct CegoBlobValue
 {
-    int fileId;
-    int pageId;
+    PageIdType pageId;
     int len;
     unsigned char *buf;
 } CGBlob;
 
+typedef struct CegoClobValue
+{
+    PageIdType pageId;
+    int len;
+    char *buf;
+} CGClob;
+
+typedef struct CegoAttr
+{
+    int type;
+    int len;
+    void *name;
+} CGAttr;
+
 #ifdef  __cplusplus
 class CegoBindValue
 {
@@ -131,16 +146,31 @@ extern "C" {
 #endif
 
     void cego_modlog(char* module, int level);
-    
+
     CGDB* cego_connect(char *servername, int portno, char *prot, char *tsname, char *user, char *pass, char* logfile);
-    void cego_disconnect (CGDB *cgdb);
+    int cego_disconnect (CGDB *cgdb);
+
+    char* cego_server(CGDB* cgdb);
+    int cego_port(CGDB* cgdb);
+    
+    char* cego_protocol(CGDB* cgdb);
+    char* cego_dbname(CGDB* cgdb);
+    char* cego_version(CGDB* cgdb);
+    char* cego_tableset(CGDB* cgdb);
+    char* cego_user(CGDB* cgdb);
+    
+    CGVal* cego_allocate_value(int type, int len, void *val);
+    int cego_free_value(CGVal *val);
 
     CGFetch* cego_allocate_fetch();
-    void cego_free_fetch(CGFetch *cgfetch);
+    int cego_free_fetch(CGFetch *cgfetch);
+    
     int cego_num_col(CGFetch* cgfetch);
     int cego_getpos(CGFetch* cgfetch, char* attr);
-    char* cego_getcol(CGFetch* cgfetch, int pos);
-    
+    char* cego_getcolname(CGFetch* cgfetch, int pos);
+    int cego_getcollen(CGFetch* cgfetch, int pos);
+    int cego_getcoltype(CGFetch* cgfetch, int pos);
+	 
     CGStmt* cego_prepare (char* stmt);
     int cego_bind_in (CGStmt* cgstmt, CGVal* cgval, int pos);
     int cego_bind_out (CGStmt* cgstmt, CGVal* cgval, int pos);
@@ -148,11 +178,13 @@ extern "C" {
 
     int cego_execute (CGDB *cgdb, CGStmt* cgstmt, CGFetch* cgfetch);
 
-    void cego_free_stmt(CGStmt* cgstmt);
+    int cego_free_stmt(CGStmt* cgstmt);
     
-    int cego_query (CGDB* cgdb, char* stmt, CGFetch* cgfetch);
+    int cego_query (CGDB* cgdb, const char* stmt, CGFetch* cgfetch);
+    int cego_isfetchable (CGFetch* cgfetch);
+    
+    unsigned long cego_getaffected(CGDB* cgdb);
     
-
     int cego_fetch (CGFetch* cgfetch, CGVal cgval[], int numCol);
     int cego_abort (CGFetch* cgfetch);
     int cego_reset (CGFetch* cgfetch);
@@ -161,10 +193,12 @@ extern "C" {
     int cego_getblob (CGDB *cgdb, CGBlob* cgblob);
     int cego_delblob (CGDB *cgdb, CGBlob* cgblob);
 
+    int cego_putclob (CGDB *cgdb, CGClob* cgclob);
+    int cego_getclob (CGDB *cgdb, CGClob* cgclob);
+    int cego_delclob (CGDB *cgdb, CGClob* cgclob);
+
     char* cego_error ();
 #ifdef  __cplusplus
 }
 #endif
-
-#endif 
-
+#endif
